/*
********************************************************************************
*                   213x --- video in driver extern module api
*                (c) Copyright 2007 - 2011, Actions Co,Ld.
*                        All Right Reserved
*
* History:
 *      <author>      <time>       <version >    <desc>
 *       houjingkun      2008/05/30     1.0      build file.
 *                                               this file is used for ov9655.
 *       houjingkun      2008/06/18     1.1      modify for code checking.
 *       houjingkun      2008/08/05     1.2      change name from sensor_param.c
 *                                               to sensor_module.c
 *       wangxin         2011/03/24     2.0      refactoring module
********************************************************************************/
/*!
 * \file   vi_ext_mod.c
 * \brief
 *      programing api of extern video_in hardware module, outside of actions's ic.
 *      such as sensor module or tvin ic module.
 * \author jamelee
 * \par GENERAL DESCRIPTION:
 *      programing api of extern video_in hardware module, outside of actions's ic.
 *      such as sensor module or tvin ic module.
 * \par EXTERNALIZED FUNCTIONS:
 *       null
 *
 *     Copyright(c) 2008-2012 Actions Semiconductor, All Rights Reserved.
 *
 * \version 2.0
 * \date  2011/03/24
 *******************************************************************************/

#include "include_vi.h"


/* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ################################### */


#ifndef VI_ALWAYS_OCCUPY_I2C_BUS
#define VI_ALWAYS_OCCUPY_I2C_BUS 1
#endif




#define  MAX_PREVIEW_SIZE  VI_RES_640X480 /* VGA */

#define SP0838_SENSOR_ID 0x27

//////////////////////////////
#define FLOOR(x, y) (((x) / (y)) * (y))

/* #####   TYPE DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ######################### */
typedef struct
{
    unsigned char const *pdata;
    unsigned int data_size;
} sensor_cfg_entry_t;


/* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ################################ */


vi_ext_mod_t vi_sensor_mod;

static volatile unsigned long *s_pJiffies;
static unsigned long s_wait_capture_ready_start;
static unsigned long s_wait_capture_ready_limit;
static unsigned int  s_need_wait_capture_ready;

/* init code for galaxycore. */
static const unsigned char sc_sensor_init_data[] =
{
	0xfd,0x00,
	0x1B,0x02,
	0x27,0xe8,
	0x28,0x0B,
	0x32,0x00,
	0x22,0xc0,
	0x26,0x10,
	0x31,0x10,//180:0x10
	0x5f,0x11,
	0xfd,0x01,
	0x25,0x1a,
	0x26,0xfb,
	0x28,0x75, 
	0x29,0x4e,

	0xfd,0x00,   
	0xe7,0x03, 
	0xe7,0x00, 
	0xfd,0x01,

	0x31,0x60,///0x60
	0x32,0x18,
	0x4d,0xdc,
	0x4e,0x53,
	0x41,0x8c,
	0x42,0x57,
	0x55,0xff,
	0x56,0x00,
	0x59,0x82,
	0x5a,0x00,
	0x5d,0xff,
	0x5e,0x6f,
	0x57,0xff,
	0x58,0x00,
	0x5b,0xff,
	0x5c,0xa8,
	0x5f,0x75,
	0x60,0x00,
	0x2d,0x00,
	0x2e,0x00,
	0x2f,0x00,
	0x30,0x00,
	0x33,0x00,
	0x34,0x00,
	0x37,0x00,
	0x38,0x00,
	0xfd,0x00,
	0x33,0x6f,
	0x51,0x3f,
	0x52,0x09,
	0x53,0x00,
	0x54,0x00,
	0x55,0x10,
	0x4f,0x08,//ff,
	0x50,0x08,//ff,
	0x57,0x10,  //Raw filter debut start
	0x58,0x10,
	0x59,0x10,
	0x56,0x70,
	0x5a,0x02,
	0x5b,0x02,
	0x5c,0x20,
	0x65,0x03,
	0x66,0x01,
	0x67,0x03,
	0x68,0x46,
	0x69,0x7f,
	0x6a,0x01,
	0x6b,0x04,
	0x6c,0x01,
	0x6d,0x03,
	0x6e,0x46,
	0x6f,0x7f,
	0x70,0x01,
	0x71,0x05,
	0x72,0x01,
	0x73,0x03,
	0x74,0x46,
	0x75,0x7f,
	0x76,0x01,
	0xcb,0x07,
	0xcc,0x04,
	0xce,0xff,
	0xcf,0x10,
	0xd0,0x20,
	0xd1,0x00,
	0xd2,0x1c,
	0xd3,0x16,
	0xd4,0x00,
	0xd6,0x1c,
	0xd7,0x16,
	0xdd,0x70,
	0xde,0x90,
	0x7f,0xd7,  //Color Correction start
	0x80,0xbc,                          
	0x81,0xed,                          
	0x82,0xd7,                          
	0x83,0xd4,                          
	0x84,0xd6,                          
	0x85,0xff,                          
	0x86,0x89,                          
	0x87,0xf8,                          
	0x88,0x3c,                          
	0x89,0x33,                          
	0x8a,0x0f,  //Color Correction end 
	0x8b,0x00,
	0x8c,0x1a,
	0x8d,0x29,
	0x8e,0x41,
	0x8f,0x62,
	0x90,0x7c,
	0x91,0x90,
	0x92,0xa2,
	0x93,0xaf,
	0x94,0xbc,
	0x95,0xc5,
	0x96,0xcd,
	0x97,0xd5,
	0x98,0xdd,
	0x99,0xe5,
	0x9a,0xed,
	0x9b,0xf5,
	0xfd,0x01,
	0x8d,0xfd,
	0x8e,0xff,
	0xfd,0x00,
	0xca,0xcf,
	0xd8,0x48,//0x65,
	0xd9,0x48,//0x65,
	0xda,0x48,//0x65,
	0xdb,0x48,//0x50,
	0xb9,0x00,
	0xba,0x04,
	0xbb,0x08,
	0xbc,0x10,
	0xbd,0x20,
	0xbe,0x30,
	0xbf,0x40,
	0xc0,0x50,
	0xc1,0x60,
	0xc2,0x70,
	0xc3,0x80,
	0xc4,0x90,
	0xc5,0xA0,
	0xc6,0xB0,
	0xc7,0xC0,
	0xc8,0xD0,
	0xc9,0xE0,
	0xfd,0x01,
	0x89,0xf0,
	0x8a,0xff,
	0xfd,0x00,
	0xe8,0x30,
	0xe9,0x30,
	0xea,0x40,
	0xf4,0x1b,
	0xf5,0x80,
	
	0xf7,0x78,///0x78,	//modify by sp_yjp,20120409
	0xf8,0x63,///0x63,
	0xf9,0x68,///0x68,
	0xfa,0x53,///0x53,
	
	0xfd,0x01,
	0x09,0x31,
	0x0a,0x85,
	0x0b,0x0b,
	0x14,0x20,
	0x15,0x0f,
	#if 1//sp0838 24M 1Ƶ 50Hz VGA 18-8fps
	0xfd,0x00,
	0x05,0x00,
	0x06,0x00,
	0x09,0x01,
	0x0a,0xff,
	0xf0,0x58,
	0xf1,0x00,
	0xf2,0x5d,
	0xf5,0x76,
	0xfd,0x01,
	0x00,0xb0,
	0x0f,0x5e,
	0x16,0x5e,
	0x17,0xa0,
	0x18,0xa8,
	0x1b,0x5e,
	0x1c,0xa8,
	0xb4,0x20,
	0xb5,0x3c,
	0xb6,0x54,
	0xb9,0x40,
	0xba,0x4f,
	0xbb,0x47,
	0xbc,0x45,
	0xbd,0x43,
	0xbe,0x42,
	0xbf,0x42,
	0xc0,0x42,
	0xc1,0x41,
	0xc2,0x41,
	0xc3,0x41,
	0xc4,0x41,
	0xc5,0x70,
	0xc6,0x41,
	0xca,0x70,
	0xcb,0x0c,
	0xfd,0x00,
	#endif
	0x32,0x15,
	0x34,0x66,
	0x35,0x40,
};

//static const unsigned char sc_sensor_non_vga_LENC_data[]=
//{
// 0x00, 0x00,
//};
//
//static const unsigned char sc_sensor_vga_LENC_data[]=
//{
// 0x00, 0x00,
//};

/* qvga */
static const unsigned char sc_sensor_320x240_data[]=
{
   
	0xfd,0x00,
	0x05,0x00,
	0x06,0x00,
	0x09,0x01,
	0x0a,0xff,
	0xf0,0x58,
	0xf1,0x00,
	0xf2,0x5d,
	0xf5,0x76,
	0xfd,0x01,
	0x00,0xb0,
	0x0f,0x5e,
	0x16,0x5e,
	0x17,0xa0,
	0x18,0xa8,
	0x1b,0x5e,
	0x1c,0xa8,
	0xb4,0x20,
	0xb5,0x3c,
	0xb6,0x54,
	0xb9,0x40,
	0xba,0x4f,
	0xbb,0x47,
	0xbc,0x45,
	0xbd,0x43,
	0xbe,0x42,
	0xbf,0x42,
	0xc0,0x42,
	0xc1,0x41,
	0xc2,0x41,
	0xc3,0x41,
	0xc4,0x41,
	0xc5,0x70,
	0xc6,0x41,
	0xca,0x70,
	0xcb,0x0c,
	0xfd,0x00,
	0x31,0x14,
	0xe7,0x03,
	0xe7,0x00

   
};  /* sp0838 QVGA: 320 * 240 */

/* vga */
static const unsigned char sc_sensor_640x480_data[] =
{  
	0xfd,0x00,
	0x05,0x00,
	0x06,0x00,
	0x09,0x01,
	0x0a,0xff,
	0xf0,0x58,
	0xf1,0x00,
	0xf2,0x5d,
	0xf5,0x76,
	0xfd,0x01,
	0x00,0xb0,
	0x0f,0x5e,
	0x16,0x5e,
	0x17,0xa0,
	0x18,0xa8,
	0x1b,0x5e,
	0x1c,0xa8,
	0xb4,0x20,
	0xb5,0x3c,
	0xb6,0x54,
	0xb9,0x40,
	0xba,0x4f,
	0xbb,0x47,
	0xbc,0x45,
	0xbd,0x43,
	0xbe,0x42,
	0xbf,0x42,
	0xc0,0x42,
	0xc1,0x41,
	0xc2,0x41,
	0xc3,0x41,
	0xc4,0x41,
	0xc5,0x70,
	0xc6,0x41,
	0xca,0x70,
	0xcb,0x0c,
	0xfd,0x00,
	0x31,0x10,
	0xe7,0x03,
	0xe7,0x00
}; /* sp0838 VGA: 640 * 480 */


/* #####   PROTOTYPES  -  LOCAL TO THIS SOURCE FILE   ############################### */


static void _vi_ext_mod_status_preconfig(void);
static int _vi_ext_mod_set_reg(unsigned int reg, unsigned int data);
static int _vi_ext_mod_get_reg(unsigned int reg, unsigned int *data);
static int _vi_ext_mod_send_batch_cfg(unsigned char const *pCfgData, unsigned int uDataSize);

static int _vi_ext_mod_start_ag_ae(void);
#if 0
static int _vi_ext_mod_stop_ag_ae(void);
static int _vi_ext_mod_start_awb(void);
static int _vi_ext_mod_freeze_awb(void);
#endif
static int _vi_ext_mod_set_mirror_flip(unsigned int mirror_en, unsigned int flip_en);

static int _vi_ext_mod_set_pclk_polarity(unsigned int polar);
static int _vi_ext_mod_set_vsync_polarity(unsigned int polar);
static int _vi_ext_mod_set_href_polarity(unsigned int polar);

/**********add to respair the edage bug of picture ***************************/
static void stuff_point_4y4u(unsigned char *src, int width, int height);
static void stuff_point_yuyv(unsigned char *src, int width, int height);
static void stuff_point_yuv420(unsigned char *src, int width, int height);



/* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ############################ */


/******************************************************************************/
/*!
* \par  Description:
*       active sensor reset and power down pins, make sensor module enter
*       working status
* \param[in]
*       null
* \retval
*       null
*******************************************************************************/ 
int vi_ext_mod_work_on(void)
{
        if(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP != GPIO_GROUP_NULL)
    {
        gpio_enable_output(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP, IMAGE_SENSOR_POWERDOWN_PIN_NUM);

#if (SENSOR_POWER_DOWN_ACTIVE_LEVEL != 0)
        gpio_set_state(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP, IMAGE_SENSOR_POWERDOWN_PIN_NUM, 0);
#else
        gpio_set_state(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP, IMAGE_SENSOR_POWERDOWN_PIN_NUM, 1);
#endif
        //mdelay(10);//qhy
    }
		
    /* reset sensor module to normal status */
    /* NOTE: all enum vars are zero while pe-compiling stage */
    if(IMAGE_SENSOR_RESET_GPIO_GROUP != GPIO_GROUP_NULL)
    {
        gpio_enable_output(IMAGE_SENSOR_RESET_GPIO_GROUP, IMAGE_SENSOR_RESET_PIN_NUM);

        // delay some time, to meet the reset timming
        mdelay(4);//qhy

#if (SENSOR_RESET_ACTIVE_LEVEL != 0)
        gpio_set_state(IMAGE_SENSOR_RESET_GPIO_GROUP, IMAGE_SENSOR_RESET_PIN_NUM, 1);
#else
        gpio_set_state(IMAGE_SENSOR_RESET_GPIO_GROUP, IMAGE_SENSOR_RESET_PIN_NUM, 0);
#endif

        mdelay(10);

#if (SENSOR_RESET_ACTIVE_LEVEL != 0)
        gpio_set_state(IMAGE_SENSOR_RESET_GPIO_GROUP, IMAGE_SENSOR_RESET_PIN_NUM, 0);
#else
        gpio_set_state(IMAGE_SENSOR_RESET_GPIO_GROUP, IMAGE_SENSOR_RESET_PIN_NUM, 1);
#endif

        mdelay(10);
    }

    printk("qhy===IMAGE_SENSOR_POWERDOWN_GPIO_GROUP = %d\n", IMAGE_SENSOR_POWERDOWN_GPIO_GROUP);
    printk("qhy===IMAGE_SENSOR_POWERDOWN_PIN_NUM = %d\n", IMAGE_SENSOR_POWERDOWN_PIN_NUM);
    printk("qhy===IMAGE_SENSOR_RESET_GPIO_GROUP = %d\n", IMAGE_SENSOR_RESET_GPIO_GROUP);
    printk("qhy===IMAGE_SENSOR_RESET_PIN_NUM = %d\n", IMAGE_SENSOR_RESET_PIN_NUM);

    return 0;
}

/******************************************************************************/
/*!
* \par  Description:
*       sensor module enter standby
* \param[in]   null
* \retval      0 sucess, else failed.
*******************************************************************************/
int vi_ext_mod_enter_standby(void)
{
    int ret = 0;

    /**
     * TODO: Set some registers here if necessary
     */

    // set the sensor standby PIN to power down status here
    /* set the sensor power down status */
    if(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP != GPIO_GROUP_NULL)
    {
        gpio_enable_output(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP, IMAGE_SENSOR_POWERDOWN_PIN_NUM);

#if (SENSOR_POWER_DOWN_ACTIVE_LEVEL != 0)
        gpio_set_state(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP, IMAGE_SENSOR_POWERDOWN_PIN_NUM, 1);
#else
        gpio_set_state(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP, IMAGE_SENSOR_POWERDOWN_PIN_NUM, 0);
#endif

        // delay some time, to meet the power-down timming
        // to keep MCLK output for 10ms
        mdelay(10);
    }

    return ret;
}

/******************************************************************************/
/*!
* \par  Description:
*       sensor module exit standby
* \param[in]   null
* \retval      0 sucess, else failed.
*******************************************************************************/
int vi_ext_mod_exit_standby(void)
{
    // set the sensor standby PIN to power on status here
    /* cancel the sensor power down status */
    if(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP != GPIO_GROUP_NULL)
    {
        gpio_enable_output(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP, IMAGE_SENSOR_POWERDOWN_PIN_NUM);

        // delay some time, to meet the power-down timming
        // keep MCLK output for 10ms before exit power-down state
        mdelay(10);

#if (SENSOR_POWER_DOWN_ACTIVE_LEVEL != 0)
        gpio_set_state(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP, IMAGE_SENSOR_POWERDOWN_PIN_NUM, 0);
#else
        gpio_set_state(IMAGE_SENSOR_POWERDOWN_GPIO_GROUP, IMAGE_SENSOR_POWERDOWN_PIN_NUM, 1);
#endif

        // delay 10ms before any register access
        mdelay(10);
    }

    /**
     * TODO: Set some registers here if necessary
     */

    return 0;
}

/******************************************************************************/
/*!
* \par  Description:
*       initialize sensor module.
* \param[in]   null
* \retval      0 sucess, else failed.
*******************************************************************************/
int vi_ext_mod_init(void)
{
    int ret;
    unsigned int reg_data;

    // preconfig some status
    _vi_ext_mod_status_preconfig();

    s_pJiffies = kernel_sym("jiffies");
    if(IS_ERR((void*)s_pJiffies)!=0)
    {
        printk_err("get kernel_sym jiffies error");
        goto label_err_exit;
    }

    /* exit pwr_down & reset */
    vi_ext_mod_work_on();

    ret = _vi_ext_mod_set_reg(0xfd, 0x00);
    if(ret != 0)
    {
        printk_err("switch to page 0 error, ret = %d", ret);
        return ret;
    }

    // check sensor module ID
    ret = _vi_ext_mod_get_reg(0x02, &reg_data);
    if(ret != 0)
    {
        printk_err("check sensor module ID error, ret = %d", ret);
        goto label_err_exit;
    }
    if(SP0838_SENSOR_ID != reg_data)
    {
        printk_err("check sensor module ID error, ret = %d", ret);
        goto label_err_exit;
    }

    printk_dbg("Sensor Module sp0838 Detected.");

    /* init sensor module. */
    ret = _vi_ext_mod_send_batch_cfg(&sc_sensor_init_data[0], \
                                     sizeof(sc_sensor_init_data));
    if (ret != 0)
    {
        printk_err("sensor module init failed!");
        goto label_err_exit;
    }
    ret = _vi_ext_mod_send_batch_cfg(&sc_sensor_640x480_data[0], \
                                     sizeof(sc_sensor_640x480_data));
    if (ret != 0)
    {
        printk_err("sensor module init failed!");
        goto label_err_exit;
    }

    _vi_ext_mod_set_pclk_polarity(1);       //data valid in  the rising-edage
    _vi_ext_mod_set_href_polarity(1);       //Hsync active low
    _vi_ext_mod_set_vsync_polarity(0);      //Vsync active high

    _vi_ext_mod_set_mirror_flip(SENSOR_IMAGE_HORIZONTAL_MIRROR,
                                SENSOR_IMAGE_VERTICAL_FLIP);
    _vi_ext_mod_start_ag_ae();

    return 0;

    label_err_exit:
    return -1;
}


/******************************************************************************/
/*!
* \par  Description:
*       set sensor module to standby status.
* \param[in]   null
* \retval      0 sucess, else failed.
*******************************************************************************/
int vi_ext_mod_exit(void)
{
    /**
     * TODO: add sensor module exit codes here if necessary
     */

    /* power off. */
    vi_ext_mod_enter_standby();
    return 0;
}

#if 1 //add by qhy
int vi_ext_mod_get_status(ss_status_t *status)
{
    // TODO

    return 0;
}

int vi_ext_mod_set_night_mode(unsigned int night_mode)
{
    // TODO

    vi_sensor_mod.night_mode = night_mode;

    return 0;
}

int vi_ext_mod_set_banding_filter(unsigned int banding_filter)
{
    return 0;
}
#endif

/******************************************************************************/
int vi_ext_mod_set_work_mode(vi_work_mode_t vi_work_mode)
{
    vi_sensor_mod.work_mode = vi_work_mode;
    return 0;
}


/******************************************************************************/
/* wait until the 3rd frame in capture mode */
void vi_ext_mod_wait_capture_ready(void)
{
    unsigned long cur_jiffies;

    if(s_need_wait_capture_ready != 0)
    {
        s_need_wait_capture_ready = 0;

        cur_jiffies = *s_pJiffies;
        if((cur_jiffies >= s_wait_capture_ready_start) && \
           (cur_jiffies < s_wait_capture_ready_limit))
        {
            mdelay((int)jiffies_to_msecs(s_wait_capture_ready_limit - cur_jiffies));
        }
    }
}

/******************************************************************************/
/*!
* \par  Description:
*       set sensor's resolution.
* \param[in]   cmd -- resolution enum, refer to vi_resolution_t.
* \retval      0 sucess, else failed.
*******************************************************************************/
int vi_ext_mod_set_res(unsigned int res)
{
    unsigned int w, h, w_start, w_end, h_start, h_end;
    sensor_cfg_entry_t cfg_data[4];
    unsigned int i, cfg_data_array_cnt;
    int ret;
    int need_update_banding_filter=0;//add by qhy

	ret = 0;
    printk_dbg("vi_sensor_mod.resolution = %d, res = %d", vi_sensor_mod.resolution, res);
    switch(res)
    {
        case VI_RES_320X240: /* QVGA */
        {
            w = 320;
            h = 240;
            w_start = 0;
            w_end = w_start + w -1;
            h_start = 0;
            h_end = h_start + h -1;
            cfg_data[0].pdata = sc_sensor_320x240_data;
            cfg_data[0].data_size = sizeof(sc_sensor_320x240_data);
            cfg_data_array_cnt = 1;
            break;
        }
        case VI_RES_640X480: /* VGA */
        {
            w = 640;
            h = 480;
            w_start = 0;
            w_end = w_start + w -1;
            h_start = 0;
            h_end = h_start + h -1;
            cfg_data[0].pdata = sc_sensor_640x480_data;
            cfg_data[0].data_size = sizeof(sc_sensor_640x480_data);
            cfg_data_array_cnt = 1;
            break;
        }

        default:
        {
            ret = -1;
            goto exit;
        }
    }

    /* ûбҪͲҪsensor, ÿֱʶreset AEC,
     * ǳ. */
    if(vi_sensor_mod.resolution != res)
    {
        // re-config sensor ...
        for(i=0; i<cfg_data_array_cnt; i++)
        {
            ret = _vi_ext_mod_send_batch_cfg(cfg_data[i].pdata,
                                             cfg_data[i].data_size);
            if (ret != 0)
            {
                printk_err("i2c error while set res!\n");
                goto exit;
            }
        }
    }
     #if 0 //qhy
        if(vi_sensor_mod.banding_filter!= SS_BANDING_FILTER_50HZ)//qhy
    {
        need_update_banding_filter = 1;//qhy
    }
    #endif
    //mdelay(200);//add by qhy
    // bt_conǱߵĲ
    //ret = vi_bt_con_set_hsync(w, w_start, w_end);
    ret = vi_bt_con_set_hsync(w_start<<1, w<<1);//add by qhy
    if (ret < 0)
    {
        printk_err("vi_bt_con_set_hsync error, ret = %d", ret);
        goto exit;
    }
    //ret = vi_bt_con_set_vsync(h, h_start, h_end);
    ret = vi_bt_con_set_vsync(h_start, h, 0, 0);//add by qhy
    if (ret < 0)
    {
        printk_err("vi_bt_con_set_vsync error, ret = %d", ret);
        goto exit;
    }

    _vi_ext_mod_set_mirror_flip(SENSOR_IMAGE_HORIZONTAL_MIRROR,
                                SENSOR_IMAGE_VERTICAL_FLIP);

    vi_sensor_mod.width = w;
    vi_sensor_mod.height = h;
    vi_sensor_mod.resolution = res;

    printk_dbg("\next width is %d.\n", vi_sensor_mod.width);
    printk_dbg("\next height is %d.\n", vi_sensor_mod.height);
    printk_dbg("\next resolution is %d.\n", vi_sensor_mod.resolution);

    #if 0 //add by qhy
	// re-config preview banding filter (only in preview mode)
    if(need_update_banding_filter == 1)
    {
        unsigned int banding_filter_saved = vi_sensor_mod.banding_filter;
        vi_sensor_mod.banding_filter = SS_BANDING_FILTER_NONE; // force update
        vi_ext_mod_set_banding_filter(banding_filter_saved);
    }
    #endif ////add by qhy
    exit:
    return ret;
}

/******************************************************************************/
/*!
 * \par  Description:
 *       get sensor module's current contrast value.
 * \param[in]   null
 * \return   value of current contrast level.
 * \retval   range: 0 ~ 6.
 *******************************************************************************/
unsigned int vi_ext_mod_get_contrast(void)
{
    return vi_sensor_mod.contrast;
}

/******************************************************************************/
/*!
 * \par  Description:
 *       set sensor module's current contrast value.
 * \param[in]   contrast -- range: 0 ~ 6.
 * \retval      0 sucess, else failed.
 *******************************************************************************/
int vi_ext_mod_set_contrast(unsigned int contrast)
{
    vi_sensor_mod.contrast = contrast;
    return 0;
}

/******************************************************************************/
/*!
 * \par  Description:
 *       get sensor image's white balance effect.
 * \param[in]   null
 * \return       current white balance effect. refer to ss_white_balance_t.
 *******************************************************************************/
unsigned int vi_ext_mod_get_white_balance(void)
{
    return vi_sensor_mod.white_balance;
}

/******************************************************************************/
/*!
* \par  Description:
*       set sensor image's white balance effect.
* \param[in]    white balance effect. refer to ss_white_balance_t.      \n
*       typedef enum {          \n
*           SS_WB_AUTO,         \n
*           SS_WB_SUNNY,        \n
*           SS_WB_CLOUDY,       \n
*           SS_WB_INCANDESCENT, \n
*           SS_WB_FLUORESCENT   \n
*       } white_balance_t;
*
* \retval       0 sucess, else failed.
*******************************************************************************/
int vi_ext_mod_set_white_balance(unsigned int white_balance)
#if 1
{
    /**
     * TODO: add the sensor white balance setting codes here
     */

    unsigned int temp_reg;
       
    switch(white_balance)
    {
        case SS_WB_AUTO:       /* auto */
        {            
						_vi_ext_mod_set_reg(0xfd, 0x01);
						_vi_ext_mod_set_reg(0x28, 0x75);
						_vi_ext_mod_set_reg(0x29, 0x4e); 
						_vi_ext_mod_set_reg(0xfd, 0x00);
						_vi_ext_mod_set_reg(0xe7, 0x03);
						_vi_ext_mod_set_reg(0xe7, 0x00);
						_vi_ext_mod_set_reg(0x32, 0x15); 
						_vi_ext_mod_set_reg(0xfd, 0x00);
            break;
        }
        case SS_WB_INCANDESCENT:     /* office */
        {
             _vi_ext_mod_set_reg(0xfd, 0x00);
             _vi_ext_mod_set_reg(0x32, 0x05);
             _vi_ext_mod_set_reg(0xfd, 0x01);
             _vi_ext_mod_set_reg(0x28, 0x41);
             _vi_ext_mod_set_reg(0x29, 0x71);
             _vi_ext_mod_set_reg(0xfd, 0x00);
            break;
        }
        case SS_WB_SUNNY:    /* sunny */
        {
             _vi_ext_mod_set_reg(0xfd, 0x00);
             _vi_ext_mod_set_reg(0x32, 0x05);
             _vi_ext_mod_set_reg(0xfd, 0x01);
             _vi_ext_mod_set_reg(0x28, 0x6b);
             _vi_ext_mod_set_reg(0x29, 0x48);
             _vi_ext_mod_set_reg(0xfd, 0x00);
            break;
        }
        case SS_WB_FLUORESCENT:     /* home */
        {
             _vi_ext_mod_set_reg(0xfd, 0x00);
             _vi_ext_mod_set_reg(0x32, 0x05);
             _vi_ext_mod_set_reg(0xfd, 0x01);
             _vi_ext_mod_set_reg(0x28, 0x5a);
             _vi_ext_mod_set_reg(0x29, 0x62);
             _vi_ext_mod_set_reg(0xfd, 0x00);
            break;
        }
        case SS_WB_CLOUDY:     /* cloudy */
        {
             _vi_ext_mod_set_reg(0xfd, 0x00);
             _vi_ext_mod_set_reg(0x32, 0x05);
             _vi_ext_mod_set_reg(0xfd, 0x01);
             _vi_ext_mod_set_reg(0x28, 0x71);
             _vi_ext_mod_set_reg(0x29, 0x41);
             _vi_ext_mod_set_reg(0xfd, 0x00);
            break;
        }

        default:
        {
            printk_err("set white balance over range!\n");
            return -1;
        }
    }

    vi_sensor_mod.white_balance = white_balance;
    return 0;
}
#endif


/******************************************************************************/
/*!
 * \par  Description:
 *       get sensor's focus level.
 * \param[in]    null
 * \return       focus level.
 *******************************************************************************/
unsigned int vi_ext_mod_get_focus(void)
{
    return 0;
}

/******************************************************************************/
/*!
* \par  Description:
*       set sensor's focus level.
* \param[in]    sensor's focus level.
* \retval      0 sucess, else failed.
*******************************************************************************/
int vi_ext_mod_set_focus(unsigned int focus)
{
    return 0;
}

/******************************************************************************/
/*!
* \par  Description:
*       get sensor image's brightness level.
* \param[in]    null
* \retval       brightness level, 0 ~ 6.
*******************************************************************************/
unsigned int vi_ext_mod_get_brightness(void)
{
    return vi_sensor_mod.brightness;
}

/******************************************************************************/
/*!
* \par  Description:
*       set sensor image's brightness level.
* \param[in]    brightness level, 0 ~ 6.
* \retval       0 sucess, else failed.
* \note         brihtness varies by diffrent explosure value
*******************************************************************************/
int vi_ext_mod_set_brightness(unsigned int brightness)
{
	/**
	 * TODO: add the sensor brightness setting codes here
	 */

	switch(brightness)
	{
		case 0:
		_vi_ext_mod_set_reg(0xfd, 0x00);
		_vi_ext_mod_set_reg(0xdc, 0xd0);
		break;		
	case 1:
		_vi_ext_mod_set_reg(0xfd, 0x00);
		_vi_ext_mod_set_reg(0xdc, 0xe0);
		break;				
	case 2:
		_vi_ext_mod_set_reg(0xfd, 0x00);
		_vi_ext_mod_set_reg(0xdc, 0xf0);
		break;				
	case 3:
		_vi_ext_mod_set_reg(0xfd, 0x00);
		_vi_ext_mod_set_reg(0xdc, 0x00);
		break;				
	case 4:
		_vi_ext_mod_set_reg(0xfd, 0x00);
		_vi_ext_mod_set_reg(0xdc, 0x10);
		break;				
	case 5:
		_vi_ext_mod_set_reg(0xfd, 0x00);
		_vi_ext_mod_set_reg(0xdc, 0x20);
		break;				
	case 6:
		_vi_ext_mod_set_reg(0xfd, 0x00);
		_vi_ext_mod_set_reg(0xdc, 0x30);
		break;				
		default:
		{
			printk_err("set brightness over range!\n");
			return -1;
		}
	}

	vi_sensor_mod.brightness = brightness;
	return 0;
}


/******************************************************************************/
/*!
 * \par  Description:
 *       get sensor image's expose mode.
 * \param[in]    null
 * \retval       refer to ss_expose_mode_t.
 *******************************************************************************/
unsigned int vi_ext_mod_get_expose_mode(void)
{
    return vi_sensor_mod.expose_mode;
}

/******************************************************************************/
/*!
* \par  Description:
*       set sensor image's expose mode.
* \param[in]    refer to ss_expose_mode_t.
* \retval       0 sucess, else failed.
*******************************************************************************/
int vi_ext_mod_set_expose_mode(unsigned int type)
{
	_vi_ext_mod_set_reg(0xfd,0x00);
	switch (type)
	{
		case SS_EXPOSURE_AUTO:
		{
		_vi_ext_mod_set_reg(0xfd, 0x00);
			break;
		}

		case SS_EXPOSURE_INDOOR:/*	night mode */
		{
		_vi_ext_mod_set_reg(0xfd, 0x00);
			break;
		}

		case SS_EXPOSURE_OUTDOOR:
		{
		_vi_ext_mod_set_reg(0xfd, 0x00);
			break;
		}

		default:
		{
			return -1;
		}
	}
	vi_sensor_mod.expose_mode = type;

	return 0;
}


/******************************************************************************/
/*!
 * \par  Description:
 *       get sensor image's special mode.
 * \param[in]    null
 * \retval       refer to ss_special_effect_t.
 *******************************************************************************/
unsigned int vi_ext_mod_get_special_mode(void)
{
    return vi_sensor_mod.special_mode;
}

/******************************************************************************/
/*!
* \par  Description:
*       set sensor image's special mode.
* \param[in]    refer to ss_special_effect_t.
* \retval       0 sucess, else failed.
*******************************************************************************/
int vi_ext_mod_set_special_mode(unsigned int mode)
{
	/**
	 * TODO: add the sensor special effect setting codes here
	 */
	unsigned int  temp_reg =0;

 //   _vi_ext_mod_get_reg(0x41	,&temp_reg);
 //    val_0x41= temp_reg;

	switch(mode)
	{
		case SS_EFFECT_NONE_SPECIAL:	   /* normal */
		{
		_vi_ext_mod_set_reg(0xfd,0x00);
		_vi_ext_mod_set_reg(0x62,0x00);
		_vi_ext_mod_set_reg(0x63,0x80);
		_vi_ext_mod_set_reg(0x64,0x80);
		break;
		}
		case SS_EFFECT_WHITE_AND_BLACK: 	   /* black and white */
		{
		_vi_ext_mod_set_reg(0xfd,0x00);
		_vi_ext_mod_set_reg(0x62,0x40);
		_vi_ext_mod_set_reg(0x63,0x80);
		_vi_ext_mod_set_reg(0x64,0x80); 
			break;
		}
		case SS_EFFECT_SEPIA:	   /* Antique */
		{
		_vi_ext_mod_set_reg(0xfd,0x00);
		_vi_ext_mod_set_reg(0x62,0x20);
		_vi_ext_mod_set_reg(0x63,0xc0);
		_vi_ext_mod_set_reg(0x64,0x20);
			break;
		}
		case SS_EFFECT_SOLARIZE:	   /* negative */
		{
		_vi_ext_mod_set_reg(0xfd,0x00);
		_vi_ext_mod_set_reg(0x62,0x10);
		_vi_ext_mod_set_reg(0x63,0x80);
		_vi_ext_mod_set_reg(0x64,0x80);
			break;
		}

		default:
		{
			printk_err("set special effect over range!\n");
			return -1;
		}
	}
	 //_vi_ext_mod_set_reg(0x41,val_0x41);	 

	vi_sensor_mod.special_mode = mode;
	return 0;
}


/******************************************************************************/
/*!
* \par  Description:
*       set sensor image's work mode.
* \param[in]    refer to ss_pix_fmt_t.
* \retval       0 sucess, else failed.
*******************************************************************************/
int vi_ext_mod_set_pix_fmt(unsigned int mode)
{
	/**
	 * TODO: add the sensor output data format setting codes here
	 */
	int ret = 0;
	ret = _vi_ext_mod_set_reg(0xfd, 0x00);
    if(ret != 0)
    {
        printk_err("switch to page 0 error, ret = %d", ret);
        return ret;
    }

	switch (mode)
	{
		case YUV_420_PLANAR:
		case YUV_422_PLANAR:
		{
			_vi_ext_mod_set_reg(0x35, 0x00);//0x40
			break;
		}
		case YUV_422_INTERLEAVED:
		{
			_vi_ext_mod_set_reg(0x35, 0x40);//0x00
			break;
		}

		default:
		{
			printk_err("set sensor module pixel fmt over range!\n");
			return -1;
		}
	}
	
		mdelay(300);
	return 0;
}

/******************************************************************************/
/*!
* \par  Description:
*       get sensor's frame rate in DV/DC mode.
* \param[in]    none
* \retval       the sensor  f_rate frames/second.
* \par      exmaple code
* \code
*           sensor_set_f_rate(24);  // set to 24 frames/second.
* \endcode
*******************************************************************************/
int vi_ext_mod_get_f_rate(void)
{

    return 0; // 20   //fps dynamic adjust between 12~25

}

/******************************************************************************/
/*!
* \par  Description:
*       set sensor's frame rate in DV mode.
* \param[in]    f_rate -- set to f_rate frames/second.
* \retval       0 sucess, else failed.
* \par      exmaple code
* \code
*           vi_ext_mod_set_f_rate(24);  // set to 24 frames/second.
* \endcode
*******************************************************************************/
int vi_ext_mod_set_f_rate(unsigned int f_rate)
{
    return 0;
}

/******************************************************************************/
/*!
* \par  Description:
*       get sensor's available resolution bitmap.
* \param[in]    int modes: 1  get available resolution bitmap for capturing pictures
*                          0  get available resolution bitmap for preview in all modes
* \retval       the max available resolution
*******************************************************************************/
unsigned long long vi_ext_mod_get_res_bitmap(int modes)
{
    unsigned long long res_bm;

    switch(vi_sensor_mod.work_mode)
    {
    case VI_DC_MODE:
    case VI_DV_MODE:
    case VI_PC_CAMERA_MODE:
    case VI_ANALOG_TVIN_MODE:
    case VI_CVBS_TVIN_MODE:
    case VI_DC_DV_MIXED_MODE:
        res_bm = vi_sensor_mod.preview_res_bitmap;
        break;
    case VI_SMALL_MEM_MODE:
    default:
        res_bm = 0;
        break;
    }

    if(modes == 1)
    {
        res_bm = res_bm | vi_sensor_mod.capture_res_bitmap;
    }

    return res_bm;
}

/******************************************************************************/
/*!
* \par  Description:
*       sensor module status pre-configuration.
* \param[in]    null
* \retval       null
*******************************************************************************/
static void _vi_ext_mod_status_preconfig(void)
{
	unsigned int framerate_bitmap;
	unsigned long long res_bitmap;

	/* device type ID */
	//vi_sensor_mod.id = 0;
	vi_sensor_mod.id = 0x27;

	/* sensor module i2c slave addr */
	vi_sensor_mod.i2c_slave_addr = SENSOR_I2C_SLAVE_ADDRESS;

	vi_sensor_mod.name[0] = 's';
	vi_sensor_mod.name[1] = 'p';
	vi_sensor_mod.name[2] = '0';
	vi_sensor_mod.name[3] = '8';
	vi_sensor_mod.name[4] = '3';
	vi_sensor_mod.name[5] = '8';
	vi_sensor_mod.name[6] = '\0';
	vi_sensor_mod.name[7] = '\0';

	res_bitmap = 0x0;
	//res_bitmap |= 0x1UL << VI_RES_224X176;
   // res_bitmap |= 0x1UL << VI_RES_320X240;
	res_bitmap |= 0x1UL << VI_RES_640X480;
	vi_sensor_mod.preview_res_bitmap = res_bitmap;

	res_bitmap = 0x0;
	//res_bitmap |= 0x1UL << VI_RES_320X240;
	res_bitmap |= 0x1UL << VI_RES_640X480;
	vi_sensor_mod.capture_res_bitmap = res_bitmap;

	framerate_bitmap = 0x0;
	//framerate_bitmap |= 0x1 << 12;	/* 12 frames per second is supported. */
	framerate_bitmap |= 0x1 << 24;	  /* 24 frames per second is supported. */
	//framerate_bitmap |= 0x1 << 30;	/* 24 frames per second is supported. */
	vi_sensor_mod.f_rate_bitmap = framerate_bitmap;

	vi_sensor_mod.work_mode 	= VI_DC_MODE;
	vi_sensor_mod.resolution	= VI_RES_640X480;
	vi_sensor_mod.width 		= 640;
	vi_sensor_mod.height		= 480;
	vi_sensor_mod.bpp			= 16;
	vi_sensor_mod.pix_fmt		= YUV_422_INTERLEAVED;

	vi_sensor_mod.contrast		= 3;
	vi_sensor_mod.white_balance = SS_WB_AUTO;
	vi_sensor_mod.focus 		= 3;
	vi_sensor_mod.brightness	= 1;
	vi_sensor_mod.expose_mode	= SS_EXPOSURE_AUTO;
	vi_sensor_mod.special_mode	= SS_EFFECT_NONE_SPECIAL;
	vi_sensor_mod.f_rate		= 12;
	//vi_sensor_mod.f_rate		  = 24;

	// export kernel symbol for snapshotting capture steady usage
	s_need_wait_capture_ready = 0;
	s_pJiffies = kernel_sym("jiffies");
}


/******************************************************************************/
/*!
* \par  Description:
*       sensor module write one register
* \param[in]
*       reg     register address
*       data    data to set
* \retval
*       == 0    success
*       != 0    failed
*******************************************************************************/
static int _vi_ext_mod_set_reg(unsigned int reg, unsigned int data)
{
    int rslt;
    char errmsg[128] = "no error";

#if VI_ALWAYS_OCCUPY_I2C_BUS==0
    rslt = i2c_init(CFG_I2C_CHAN_NUM, CFG_I2C_CLK);
    if ( 0 != rslt )
    {
        sprintf(errmsg, "ERROR(i2c):%d,%s---i2c_init err", __LINE__, __func__);
        goto exit;
    }
#endif

    rslt = i2c_start(SENSOR_I2C_SLAVE_ADDRESS);
    if (rslt < 0)
    {
        sprintf(errmsg, "ERROR(i2c):%d,%s---i2c_start err", __LINE__, __func__);
        goto stop;
    }

    rslt = i2c_write_byte((unsigned char)(reg & 0xffU));
    if (rslt < 0)
    {
        sprintf(errmsg, "ERROR(i2c):%d,%s---i2c_write_byte reg[%#x] err", __LINE__, __func__, reg);
        goto stop;
    }

    rslt = i2c_write_byte((unsigned char)(data & 0xffU));
    if (rslt < 0)
    {
        sprintf(errmsg, "ERROR(i2c):%d,%s---i2c_write_byte data[%#x] err", __LINE__, __func__, data);
        goto stop;
    }

    stop:
    i2c_stop();
    udelay(5);

    exit:
#if VI_ALWAYS_OCCUPY_I2C_BUS==0
    i2c_exit();
#endif
    if (rslt < 0)
    {
        printk("\n%s\n", errmsg);
    }
    return rslt;
}

/******************************************************************************/
/*!
* \par  Description:
*       sensor module read one register
* \param[in]
*       reg     register address
*       data    data to set
* \retval
*       == 0    success
*       != 0    failed
*******************************************************************************/
static int _vi_ext_mod_get_reg(unsigned int reg, unsigned int *data)
{
    int rslt;
    unsigned char reg_val;
    char errmsg[128] = "no error";

#if VI_ALWAYS_OCCUPY_I2C_BUS==0
    rslt = i2c_init(CFG_I2C_CHAN_NUM, CFG_I2C_CLK);
    if ( 0 != rslt )
    {
        sprintf(errmsg, "ERROR(i2c):%d,%s---i2c_init err", __LINE__, __func__);
        goto exit;
    }
#endif

    rslt = i2c_start(SENSOR_I2C_SLAVE_ADDRESS);
    if (rslt < 0)
    {
        sprintf(errmsg, "ERROR(i2c):%d,%s---i2c_start err", __LINE__, __func__);
        goto stop;
    }

    rslt = i2c_write_byte((unsigned char)(reg & 0xffU));
    if (rslt < 0)
    {
        sprintf(errmsg, "ERROR(i2c):%d,%s---i2c_write_byte reg[%#x] err", __LINE__, __func__, reg);
        goto stop;
    }

    udelay(5);

    rslt = i2c_restart(SENSOR_I2C_SLAVE_ADDRESS | 1U);
    if (rslt < 0)
    {
        sprintf(errmsg, "ERROR(i2c):%d,%s---i2c_restart err", __LINE__, __func__);
        goto stop;
    }

    rslt = i2c_read_byte(&reg_val, 0);
    if (rslt < 0)
    {
        sprintf(errmsg, "ERROR(i2c):%d,%s---i2c_read_byte reg[%#x] err", __LINE__, __func__, reg);
        goto stop;
    }
    *data = reg_val;

    stop:
    i2c_stop();
    udelay(5);

    exit:
#if VI_ALWAYS_OCCUPY_I2C_BUS==0
    i2c_exit();
#endif
    if (rslt < 0)
    {
        printk("\n%s\n", errmsg);
    }
    return rslt;
}

/******************************************************************************/
static int _vi_ext_mod_send_batch_cfg(unsigned char const *pCfgData, unsigned int uDataSize)
{
    int ret;
    unsigned int reg_addr, reg_data;

    if(pCfgData == 0)
    {
        return -1;
    }

    while(uDataSize >= (2U * sizeof(unsigned char)))
    {
        reg_addr = *pCfgData;
        pCfgData++;
        reg_data = *pCfgData;
        pCfgData++;
        uDataSize -= 2U * sizeof(unsigned char);
        ret = _vi_ext_mod_set_reg(reg_addr, reg_data);
        if (ret < 0)
        {
            printk_err("sensor module cfg failed, reg_%#x:=%#x", reg_addr, reg_data);
            return -1;
        }
    }

    assert(uDataSize == 0);
    return 0;
}

/******************************************************************************/
static int _vi_ext_mod_start_ag_ae(void)
{
 
    return 0;
}
/******************************************************************************/
static int _vi_ext_mod_set_mirror_flip(unsigned int mirror_en, unsigned int flip_en)
{
	unsigned int reg_0x31;
	int ret = 0;

	//0x31[5] mirror ON
	//0x31[6] flip ON

	ret = _vi_ext_mod_set_reg(0xfd, 0x00);
	if(ret != 0)
	{
		printk_err("switch to page 0 error, ret = %d", ret);
		return ret;
	}

	ret = _vi_ext_mod_get_reg(0x31, &reg_0x31);
	if(ret != 0)
	{
		printk_err("get sensor mirror/flip parameter error, ret = %d", ret);
		return ret;
	}

	
	reg_0x31 &= ~(1U << 5); 				//mirror
	reg_0x31 |= ((mirror_en & 0x1) << 5);
 
	reg_0x31 &= ~(1U << 6); 				//flip
	reg_0x31 |= ((flip_en & 0x1) << 6);

  
	ret = _vi_ext_mod_set_reg(0x31, reg_0x31);
	if(ret != 0)
	{
		printk_err("set sensor mirror/flip parameter error, ret = %d", ret);
		return ret;
	}
	return 0;
}


/******************************************************************************/
static int _vi_ext_mod_set_pclk_polarity(unsigned int polar)
{
    unsigned int reg_0x31;
    int ret;

    ret = _vi_ext_mod_set_reg(0xfd, 0x00);
    if(ret != 0)
    {
        printk_err("switch to page 0 error, ret = %d", ret);
        return ret;
    }

    //0x31[4] Control PCLK polarity
    //0: Data update at Falling-edge
    //1: Data update at Rising-edge

    ret = _vi_ext_mod_get_reg(0x31, &reg_0x31);
    if(ret != 0)
    {
        printk_err("get sensor pclk polarity error, ret = %d", ret);
        return ret;
    }

    reg_0x31 &= ~(1U << 4);
    reg_0x31 |= ((polar & 0x1) << 4);

    ret = _vi_ext_mod_set_reg(0x31, reg_0x31);
    if(ret != 0)
    {
        printk_err("set sensor pclk polarity error, ret = %d", ret);
        return ret;
    }
    return 0;
}

/******************************************************************************/
/*!
 * \par  Description:
 *       set 3640 vsync polar.
 * \param[in]
 *1: Data valid VSYNC High
 *0: Data valid VSYNC low
 * \retval      0 sucess, else failed.
 *******************************************************************************/
static int _vi_ext_mod_set_vsync_polarity(unsigned int polar)
{
	unsigned int reg_0x36;
	int ret;

	ret = _vi_ext_mod_set_reg(0xfd, 0x00);
	if(ret != 0)
	{
		printk_err("switch to page 0 error, ret = %d", ret);
		return ret;
	}

	//0x36[7] Control VSYNC polarity
	//0: Data valid VSYNC High
	//1: Data valid VSYNC low

	ret = _vi_ext_mod_get_reg(0x36, &reg_0x36);
	if(ret != 0)
	{
		printk_err("get sensor vsync polarity error, ret = %d", ret);
		return ret;
	}

	reg_0x36 &= ~(1U << 7);
	reg_0x36 |= ((polar & 0x1) << 7);

	ret = _vi_ext_mod_set_reg(0x36, reg_0x36);
	if(ret != 0)
	{
		printk_err("set sensor vsync polarity error, ret = %d", ret);
		return ret;
	}
	return 0;
}


/******************************************************************************/
/*!
 * \par  Description:
 *       set 3640 href polar.
 * \param[in]
 *0: Data valid HREF High
 *1: Data valid HREF low
 * \retval      0 sucess, else failed.
 *******************************************************************************/
static int _vi_ext_mod_set_href_polarity(unsigned int polar)
{
	unsigned int reg_0x36;
	int ret;

	ret = _vi_ext_mod_set_reg(0xfd, 0x00);
	if(ret != 0)
	{
		printk_err("switch to page 0 error, ret = %d", ret);
		return ret;
	}

	//0x36[6] Control HSYNC polarity
	//0: Data valid HREF high
	//1: Data valid HREF Low

	ret = _vi_ext_mod_get_reg(0x36, &reg_0x36);
	if(ret != 0)
	{
		printk_err("get sensor href polarity error, ret = %d", ret);
		return ret;
	}

	reg_0x36 &= ~(1U << 6);
	reg_0x36 |= ((polar & 0x1) << 6);

	ret = _vi_ext_mod_set_reg(0x36, reg_0x36);
	if(ret != 0)
	{
		printk_err("set sensor href polarity error, ret = %d", ret);
		return ret;
	}
	return 0;
}



