/*====*====*====*====*====*====*====*====*====*====*====*====*====*====*====*

 CAMERA    API    M O D U L E
 
  EDIT HISTORY FOR MODULE
  
   when        who       what, where, why
   --------    ---       -------------------------------------------------------
   10/xx/08  Telechips   Created file.
*====*====*====*====*====*====*====*====*====*====*====*====*====*====*====*/

/*===========================================================================

 INCLUDE FILES FOR MODULE
 telechip 8925

 Attention: this driver was fixed base on SP0838_vga
 
===========================================================================*/
#include <linux/delay.h>
#include <asm/system.h>
#include <mach/hardware.h>
#include <asm/io.h>
#include "sensor_if.h"
#include "cam.h"
#include "cam_reg.h"
#include "tcc_cam_i2c.h"

#define SUPERPIX_SENSOR_ID_REG 0x02  //pzt 2012-6-2

#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX) || defined(CONFIG_ARCH_TCC88XX) || defined(CONFIG_ARCH_TCC892X)
#include <mach/bsp.h>
#elif defined(CONFIG_ARCH_TCC79X)
#include <mach/tcc79x.h>
#endif

#ifdef CONFIG_VIDEO_CAMERA_SENSOR_SP0838


/* Array of image sizes supported by SP0838.  These must be ordered from 
* smallest image size to largest.
*/
#if defined(CONFIG_VIDEO_DUAL_CAMERA_SUPPORT)
struct capture_size sensor_sizes_sp0838[] = {
	{  640,  480 },	/* VGA */
	{  320,  240 },	/* QVGA */
};
#else
struct capture_size sensor_sizes[] = {
	{  640,  480 },	/* VGA */
	{  320,  240 },	/* QVGA */
};
#endif


/* register initialization tables for sensor */
/* common sensor register initialization for all image sizes, pixel formats, 
* and frame rates
*/
static struct sensor_reg sensor_initialize_sp0838[] = {
	//SP0838 ini
{0xfd,0x00},//P0
{0x1B,0x02},
{0x27,0xe8},
{0x28,0x0B},
{0x32,0x00},
{0x22,0xc0},
{0x26,0x10}, 
{0x5f,0x11},//Bayer order
{0xfd,0x01},//P1
{0x25,0x1a},//Awb start
{0x26,0xfb},
{0x28,0x75},
{0x29,0x4e},
{0xfd,0x00},
{0xe7,0x03},
{0xe7,0x00},
{0xfd,0x01},
{0x31,0x60},//64
{0x32,0x18},
{0x4d,0xdc},
{0x4e,0x53},
{0x41,0x8c},
{0x42,0x57},
{0x55,0xff},
{0x56,0x00},
{0x59,0x82},
{0x5a,0x00},
{0x5d,0xff},
{0x5e,0x6f},
{0x57,0xff},
{0x58,0x00},
{0x5b,0xff},
{0x5c,0xa8},
{0x5f,0x75},
{0x60,0x00},
{0x2d,0x00},
{0x2e,0x00},
{0x2f,0x00},
{0x30,0x00},
{0x33,0x00},
{0x34,0x00},
{0x37,0x00},
{0x38,0x00},//awb end
{0xfd,0x00},//P0
{0x33,0x6f},//LSC BPC EN
{0x51,0x3f},//BPC debug start
{0x52,0x09},
{0x53,0x00},
{0x54,0x00},
{0x55,0x10},//BPC debug end
{0x4f,0x08},//blueedge
{0x50,0x08},
{0x57,0x10},//Raw filter debut start
{0x58,0x10},
{0x59,0x10},
{0x56,0x70},
{0x5a,0x02},
{0x5b,0x02},
{0x5c,0x20},//Raw filter debut end 
{0x65,0x03},//Sharpness debug start
{0x66,0x01},
{0x67,0x03},
{0x68,0x46},
{0x69,0x7f},
{0x6a,0x01},
{0x6b,0x04},
{0x6c,0x01},
{0x6d,0x03},//Edge gain normal
{0x6e,0x46},//Edge gain normal
{0x6f,0x7f},
{0x70,0x01},
{0x71,0x05}, //ֵ          
{0x72,0x01}, //ֵ        
{0x73,0x03}, //Եֵ    
{0x74,0x46}, //Եֵ    
{0x75,0x7f},              //ʹλ            
{0x76,0x01},//Sharpness debug end
{0xcb,0x07},//HEQ&Saturation debug start 
{0xcc,0x04},
{0xce,0xff},
{0xcf,0x10},
{0xd0,0x20},
{0xd1,0x00},
{0xd2,0x1c},
{0xd3,0x16},
{0xd4,0x00},
{0xd6,0x1c},
{0xd7,0x16},
{0xdd,0x70},//Contrast
{0xde,0x90},//HEQ&Saturation debug end
{0x7f,0xd7},//Color Correction start
{0x80,0xbc},                        
{0x81,0xed},                        
{0x82,0xd7},                        
{0x83,0xd4},                        
{0x84,0xd6},                        
{0x85,0xff},                        
{0x86,0x89},                        
{0x87,0xf8},                        
{0x88,0x3c},                        
{0x89,0x33},                        
{0x8a,0x0f},//Color Correction end  
{0x8b,0x0 },//gamma start
{0x8c,0x1a},             
{0x8d,0x29},             
{0x8e,0x41},             
{0x8f,0x62},             
{0x90,0x7c},             
{0x91,0x90},             
{0x92,0xa2},             
{0x93,0xaf},             
{0x94,0xbc},             
{0x95,0xc5},             
{0x96,0xcd},             
{0x97,0xd5},             
{0x98,0xdd},             
{0x99,0xe5},             
{0x9a,0xed},             
{0x9b,0xf5},             
{0xfd,0x01},//P1         
{0x8d,0xfd},             
{0x8e,0xff},//gamma end  
{0xfd,0x00},//P0
{0xca,0xcf},
{0xd8,0x48},//UV outdoor
{0xd9,0x48},//UV indoor 
{0xda,0x48},//UV dummy
{0xdb,0x48},//UV lowlight
{0xb9,0x00},//Ygamma start
{0xba,0x04},
{0xbb,0x08},
{0xbc,0x10},
{0xbd,0x20},
{0xbe,0x30},
{0xbf,0x40},
{0xc0,0x50},
{0xc1,0x60},
{0xc2,0x70},
{0xc3,0x80},
{0xc4,0x90},
{0xc5,0xA0},
{0xc6,0xB0},
{0xc7,0xC0},
{0xc8,0xD0},
{0xc9,0xE0},
{0xfd,0x01},//P1
{0x89,0xf0},
{0x8a,0xff},//Ygamma end
{0xfd,0x00},//P0
{0xe8,0x30},//AEdebug start
{0xe9,0x30},
{0xea,0x40},//Alc Window sel
{0xf4,0x1b},//outdoor mode sel
{0xf5,0x80},
{0xf7,0x78},//AE target
{0xf8,0x63},
{0xf9,0x68},//AE target 
{0xfa,0x53},
{0xfd,0x01},//P1
{0x09,0x31},//AE Step 3.0
{0x0a,0x85},
{0x0b,0x0b},//AE Step 3.0
{0x14,0x20},
{0x15,0x0f},
#if 1//caprure preview daylight 24M 50hz 20-8FPS maxgain:0x70	
{0xfd,0x00},
{0x05,0x00},
{0x06,0x00},
{0x09,0x01},
{0x0a,0x76},
{0xf0,0x62},
{0xf1,0x0 },
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},
{0x00,0xb2},
{0x0f,0x60},
{0x16,0x60},
{0x17,0xa2},
{0x18,0xaa},
{0x1b,0x60},
{0x1c,0xaa},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x70},
{0xc6,0x41},
{0xca,0x70},
{0xcb,0xc },
{0xfd,0x00},
#endif	
{0xfd,0x00},  //P0
{0x32,0x15},  //Auto_mode set
{0x34,0x66},  //Isp_mode set
{0x35,0x40},  //out format
	{REG_TERM, VAL_TERM}
};


static struct sensor_reg sensor_set_preview_sp0838[] = {
  //capture preview daylight 24M 50hz 20-8FPS maxgain:0x70   pzt 2012-6-2
        {0xfd,0x00},  
        {0x05,0x0 },  
        {0x06,0x0 },  
        {0x09,0x1 },  
        {0x0a,0x76},  
        {0xf0,0x62},  
        {0xf1,0x0 },  
        {0xf2,0x5f},  
        {0xf5,0x78},  
        {0xfd,0x01},  
        {0x00,0xb2},  
        {0x0f,0x60},  
        {0x16,0x60},  
        {0x17,0xa2},  
        {0x18,0xaa},  
        {0x1b,0x60},  
        {0x1c,0xaa},  
        {0xb4,0x20},  
        {0xb5,0x3a},  
        {0xb6,0x5e},  
        {0xb9,0x40},  
        {0xba,0x4f},  
        {0xbb,0x47},  
        {0xbc,0x45},  
        {0xbd,0x43},  
        {0xbe,0x42},  
        {0xbf,0x42},  
        {0xc0,0x42},  
        {0xc1,0x41},  
        {0xc2,0x41},  
        {0xc3,0x41},  
        {0xc4,0x41},  
        {0xc5,0x70},  
        {0xc6,0x41},  
        {0xca,0x70},  
        {0xcb,0xc },  
        {0x14,0x20},//
        {0x15,0x0f},//
        {0xfd,0x00},  
	{REG_TERM, VAL_TERM}
};                   
                     
static struct sensor_reg sensor_set_capture_sp0838[] = {
	   //capture preview daylight 24M 50hz 20-8FPS maxgain:0x70   same to preview  pzt 2012-6-2
        {0xfd,0x00},   
        {0x05,0x0 },   
        {0x06,0x0 },   
        {0x09,0x1 },   
        {0x0a,0x76},   
        {0xf0,0x62},   
        {0xf1,0x0 },   
        {0xf2,0x5f},   
        {0xf5,0x78},   
        {0xfd,0x01},   
        {0x00,0xb2},   
        {0x0f,0x60},   
        {0x16,0x60},   
        {0x17,0xa2},   
        {0x18,0xaa},   
        {0x1b,0x60},   
        {0x1c,0xaa},   
        {0xb4,0x20},   
        {0xb5,0x3a},   
        {0xb6,0x5e},   
        {0xb9,0x40},   
        {0xba,0x4f},   
        {0xbb,0x47},   
        {0xbc,0x45},   
        {0xbd,0x43},   
        {0xbe,0x42},   
        {0xbf,0x42},   
        {0xc0,0x42},   
        {0xc1,0x41},   
        {0xc2,0x41},   
        {0xc3,0x41},   
        {0xc4,0x41},   
        {0xc5,0x70},   
        {0xc6,0x41},   
        {0xca,0x70},   
        {0xcb,0xc },   
        {0x14,0x20},// 
        {0x15,0x0f},// 
        {0xfd,0x00},   
	{REG_TERM, VAL_TERM}
};      
        
 struct sensor_reg* sensor_reg_common[3] =
{       
	sensor_initialize_sp0838,
	sensor_set_preview_sp0838,
	sensor_set_capture_sp0838
};      
        
static struct sensor_reg sensor_brightness_0_sp0838[] = {
     // Brightness -2
       {0xfd, 0x00},
	{0xdc, 0xe0},
	{REG_TERM, VAL_TERM}
};      
        
static struct sensor_reg sensor_brightness_1_sp0838[] = {
       // Brightness -1  //򷽰û򿪣֪Ĭϵ飬ӵʱԼ
       {0xfd, 0x00},
	{0xdc, 0xf0},
	{REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_brightness_2_sp0838[] = {
   //  Brightness 0
       {0xfd, 0x00},
	{0xdc, 0x00},
	{REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_brightness_3_sp0838[] = {
    // Brightness +1
       {0xfd, 0x00},
	{0xdc, 0x10},
	{REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_brightness_4_sp0838[] = {
     //  Brightness +2
       {0xfd, 0x00},
	{0xdc, 0x20},
	{REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_brightness[5] =
{
	sensor_brightness_0_sp0838,
	sensor_brightness_1_sp0838,
	sensor_brightness_2_sp0838,
	sensor_brightness_3_sp0838,
	sensor_brightness_4_sp0838
};


static struct sensor_reg sensor_awb_auto_sp0838[] = {
    //SP0A18_reg_WB_auto         Զ
	{0xfd,0x01},                                                          
	{0x28,0x75},		                                                       
	{0x29,0x4e},
	{0xfd,0x00},  // AUTO 3000K~7000K   	  
	{0x32,0x15},
	{REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_daylight_sp0838[] = {
	// SP0A18_reg_WB_auto   
	{0xfd,0x00},  //6500K                                     
	{0x32,0x05},                                                          
	{0xfd,0x01},                                                          
	{0x28,0x6b},		                                                       
	{0x29,0x48},		                                                       
	{0xfd,0x00}, 	 
       {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_incandescent_sp0838[] = {
    //Incandescent
  	{0xfd,0x00}, 
	{0x32,0x05},                 
	{0xfd,0x01},                 
	{0x28,0x41},		             
	{0x29,0x71},		             
	{0xfd,0x00}, 
       {REG_TERM, VAL_TERM}

};

static struct sensor_reg sensor_awb_fluorescent_sp0838[] = {
     //fluorescent
	{0xfd,0x00},  
	{0x32,0x05},                  
	{0xfd,0x01},                  
	{0x28,0x5a},		              
	{0x29,0x62},		              
	{0xfd,0x00},		
       {REG_TERM, VAL_TERM}
    
};

static struct sensor_reg sensor_awb_cloudy_sp0838[] = {
    // SP0A18_reg_WB_auto   
	{0xfd,0x00},   //7000K                                     
	{0x32,0x05},                                                          
	{0xfd,0x01},                                                          
	{0x28,0x71},		                                                       
	{0x29,0x41},		                                                       
	{0xfd,0x00},
       {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_sunset_sp0838[] = {
       {REG_TERM, VAL_TERM}
    
};

struct sensor_reg* sensor_reg_awb[5] =  //6
{
	sensor_awb_auto_sp0838,
	sensor_awb_daylight_sp0838,
	sensor_awb_incandescent_sp0838,
	sensor_awb_fluorescent_sp0838,
	sensor_awb_cloudy_sp0838,
	//sensor_awb_sunset_sp0838
	
};

static struct sensor_reg sensor_iso_auto_sp0838[] = {

    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_iso_100_sp0838[] = {

    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_iso_200_sp0838[] = {

    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_iso_400_sp0838[] = {

	{REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_iso[4] =
{
	sensor_iso_auto_sp0838,
	sensor_iso_100_sp0838,
	sensor_iso_200_sp0838,
	sensor_iso_400_sp0838
};

static struct sensor_reg sensor_effect_normal_sp0838[] = {
	{0xfd, 0x00},
	{0x62, 0x00},
	{0x63, 0x80},
	{0x64, 0x80},
       {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_gray_sp0838[] = {	
	{0xfd, 0x00},
	{0x62, 0x40},
	{0x63, 0x80},
	{0x64, 0x80},
       {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_negative_sp0838[] = {
	//Negative
	{0xfd, 0x00},
	{0x62, 0x10},
	{0x63, 0x80},
	{0x64, 0x80},
       {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_sepia_sp0838[] = {
	{0xfd, 0x00},
	{0x62, 0x20},
	{0x63, 0xc0},
	{0x64, 0x20},
       {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_sharpness_sp0838[] = {
	
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_sketch_sp0838[] = {
	
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_effect[6] =
{
	sensor_effect_normal_sp0838,
	sensor_effect_gray_sp0838,
	sensor_effect_negative_sp0838,
	sensor_effect_sepia_sp0838,
	sensor_effect_sharpness_sp0838,
	sensor_effect_sketch_sp0838,
};


static struct sensor_reg sensor_reg_flipnone_sp0838[] = {
       {0xfd, 0x00},
	{0x31, 0x10},
       {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_reg_hflip_sp0838[] = {
       {0xfd, 0x00},
	{0x31, 0x30},
       {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_reg_vflip_sp0838[] = {
       {0xfd, 0x00},
	{0x31, 0x50},
       {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_reg_hvflip_sp0838[] = {
       {0xfd, 0x00},
	{0x31, 0x70},
       {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_flip[4] =
{
	sensor_reg_flipnone_sp0838,
	sensor_reg_hflip_sp0838,
	sensor_reg_vflip_sp0838,
	sensor_reg_hvflip_sp0838,
};


static struct sensor_reg sensor_secne_auto_sp0838[] = {
	//capture preview daylight 24M 50hz 20-8FPS maxgain:0x70    same to preview
	{0xfd,0x00},
        {0x05,0x0 },
        {0x06,0x0 },
        {0x09,0x1 },
        {0x0a,0x76},
        {0xf0,0x62},
        {0xf1,0x0 },
        {0xf2,0x5f},
        {0xf5,0x78},
        {0xfd,0x01},
        {0x00,0xb2},
        {0x0f,0x60},
        {0x16,0x60},
        {0x17,0xa2},
        {0x18,0xaa},
        {0x1b,0x60},
        {0x1c,0xaa},
        {0xb4,0x20},
        {0xb5,0x3a},
        {0xb6,0x5e},
        {0xb9,0x40},
        {0xba,0x4f},
        {0xbb,0x47},
        {0xbc,0x45},
        {0xbd,0x43},
        {0xbe,0x42},
        {0xbf,0x42},
        {0xc0,0x42},
        {0xc1,0x41},
        {0xc2,0x41},
        {0xc3,0x41},
        {0xc4,0x41},
        {0xc5,0x70},
        {0xc6,0x41},
        {0xca,0x70},
        {0xcb,0xc },
        {0x14,0x20},//
        {0x15,0x0f},//
        {0xfd,0x00},
       {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_night_sp0838[] = {
	//capture preview night 24M 50hz 20-6FPS maxgain:0x78	 
	{0xfd,0x00}, 
        {0x05,0x0 }, 
        {0x06,0x0 }, 
        {0x09,0x1 }, 
        {0x0a,0x76}, 
        {0xf0,0x62}, 
        {0xf1,0x0 }, 
        {0xf2,0x5f}, 
        {0xf5,0x78}, 
        {0xfd,0x01}, 
        {0x00,0xc0}, 
        {0x0f,0x60}, 
        {0x16,0x60}, 
        {0x17,0xa8}, 
        {0x18,0xb0}, 
        {0x1b,0x60}, 
        {0x1c,0xb0}, 
        {0xb4,0x20}, 
        {0xb5,0x3a}, 
        {0xb6,0x5e}, 
        {0xb9,0x40}, 
        {0xba,0x4f}, 
        {0xbb,0x47}, 
        {0xbc,0x45}, 
        {0xbd,0x43}, 
        {0xbe,0x42}, 
        {0xbf,0x42}, 
        {0xc0,0x42}, 
        {0xc1,0x41}, 
        {0xc2,0x41}, 
        {0xc3,0x41}, 
        {0xc4,0x41}, 
        {0xc5,0x41}, 
        {0xc6,0x41}, 
        {0xca,0x78}, 
        {0xcb,0x10}, 
        {0x14,0x20},//
        {0x15,0x1f},//
        {0xfd,0x00},
       {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_landscape_sp0838[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_portrait_sp0838[] = {
	{REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_sport_sp0838[] = {
	//capture preview sports 24M 50hz fix20FPS maxgain:0x78	 
	{0xfd,0x00},
        {0x05,0x0 },
        {0x06,0x0 },
        {0x09,0x1 },
        {0x0a,0x76},
        {0xf0,0x62},
        {0xf1,0x0 },
        {0xf2,0x5f},
        {0xf5,0x78},
        {0xfd,0x01},
        {0x00,0xa9},
        {0x0f,0x60},
        {0x16,0x60},
        {0x17,0x91},
        {0x18,0x99},
        {0x1b,0x60},
        {0x1c,0x99},
        {0xb4,0x20},
        {0xb5,0x3a},
        {0xb6,0x5e},
        {0xb9,0x40},
        {0xba,0x4f},
        {0xbb,0x47},
        {0xbc,0x45},
        {0xbd,0x43},
        {0xbe,0x78},
        {0xbf,0x42},
        {0xc0,0x42},
        {0xc1,0x41},
        {0xc2,0x41},
        {0xc3,0x41},
        {0xc4,0x41},
        {0xc5,0x41},
        {0xc6,0x41},
        {0xca,0x78},
        {0xcb,0x5 },
        {0x14,0x20},
        {0x15,0x0f},
        {0xfd,0x00},
	{REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_scene[5] =
{
	sensor_secne_auto_sp0838,
	sensor_secne_night_sp0838,
	sensor_secne_landscape_sp0838,
	sensor_secne_portrait_sp0838,
	sensor_secne_sport_sp0838
};

static struct sensor_reg sensor_me_mtrix_sp0838[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_me_center_weighted_sp0838[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_me_spot_sp0838[] = {
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_metering_exposure[3] =
{
	sensor_me_mtrix_sp0838,
	sensor_me_center_weighted_sp0838,
	sensor_me_spot_sp0838,
};

static struct sensor_reg sensor_af_single_sp0838[] = {

    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_af_manual_sp0838[] = {

    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_af[2] =
{
	sensor_af_single_sp0838,
	sensor_af_manual_sp0838,
};

static int write_regs_sp0838(const struct sensor_reg reglist[])
{
	int err;
	int err_cnt = 0;	
	unsigned char data[132];
	unsigned char bytes;
	struct sensor_reg *next = reglist;
	
	while (!((next->reg == REG_TERM) && (next->val == VAL_TERM)))
	{
		bytes = 0;
		data[bytes++] = (unsigned char)(next->reg) & 0xff; 	
		data[bytes]   = (unsigned char)(next->val) & 0xff;
		
        err = DDI_I2C_Write(data, 1, 1);

        if (err)
        {
			err_cnt++;
			if(err_cnt >= 3)
			{
				printk("ERROR: Sensor I2C !!!! \n"); 
				return err;
			}
        }
        else
        {
        	err_cnt = 0;
            next++;
        }
	}

	return 0;
}


static int sensor_open_sp0838(bool bChangeCamera)
{

	unsigned char id = 0;
	unsigned char id1 = 0;
	int ret = 0;
	unsigned char reg[2] = {0}, val = 0, data[3]={0};
	unsigned short cmd = 0;
	
	sensor_power_disable();
	sensor_delay(10);
	
	sensor_power_enable();
	sensor_delay(10);

	sensor_powerdown_enable();
	sensor_delay(10);
	
	sensor_powerdown_disable();
	sensor_delay(10);
	
	sensor_reset_low();
	sensor_delay(10);
	
	CIF_Open();
	sensor_delay(40);
	
	sensor_reset_high();
	sensor_delay(15);
	printk("2OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO");
	printk("init sensor GT0308 !!!! \n"); 

	DDI_I2C_Read(0x00, 1, &id, 1);
	printk("read sensor ID : %x\n", id); 
	
	//////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////
	ret = write_regs_sp0838(sensor_reg_common[0]);

	DDI_I2C_Read(SUPERPIX_SENSOR_ID_REG, 1, &id1, 1); //superpix pzt 2012-6-2
	printk("read sensor Chip-ID: 0x%02x\n", id1); 

	return ret;
}

static int sensor_close_sp0838(void)
{
	CIF_ONOFF(OFF);

	sensor_reset_low();
	sensor_power_disable();
	sensor_powerdown_enable();

	CIF_Close();   
	msleep(5);    

	return 0;
}

static int sensor_powerdown_sp0838(bool bChangeCamera)
{
	sensor_reset_low();
	sensor_power_disable();
	sensor_powerdown_enable();

	return 0;
}

static int sensor_preview_sp0838(void)
{
	return write_regs_sp0838(sensor_reg_common[1]);
}

static int sensor_capture_sp0838(void)
{
	return write_regs_sp0838(sensor_reg_common[2]);
}

static int sensor_capturecfg_sp0838(int width, int height)
{
	return 0;
}

static int sensor_zoom_sp0838(int val)
{
	return 0;
}

static int sensor_autofocus_sp0838(int val)
{
	return write_regs_sp0838(sensor_reg_af[val]);
}

static int sensor_effect_sp0838(int val)
{
	return write_regs_sp0838(sensor_reg_effect[val]);
}

static int sensor_flip_sp0838(int val)
{
	return write_regs_sp0838(sensor_reg_flip[val]);
}

static int sensor_iso_sp0838(int val)
{
	return write_regs_sp0838(sensor_reg_iso[val]);
}

static int sensor_me_sp0838(int val)
{
	return write_regs_sp0838(sensor_reg_metering_exposure[val]);
}

static int sensor_wb_sp0838(int val)
{
	return write_regs_sp0838(sensor_reg_awb[val]);
}

static int sensor_bright_sp0838(int val)
{
	return write_regs_sp0838(sensor_reg_brightness[val]);
}

static int sensor_scene_sp0838(int val)
{
	return write_regs_sp0838(sensor_reg_scene[val]);
}

static int sensor_check_esd_sp0838(int val)
{
	return 0;
}

static int sensor_check_luma_sp0838(int val)
{
	return 0;
}

/**********************************************************
*    Function  
**********************************************************/
#if defined(CONFIG_VIDEO_DUAL_CAMERA_SUPPORT)
void sensor_info_init_sp0838(TCC_SENSOR_INFO_TYPE *sensor_info)
{
	sensor_info->i2c_addr 				= 0x42;
	sensor_info->reg_term 				= 0x0000;
	sensor_info->val_term 				= 0x0000;
	#if defined(CONFIG_ARCH_TCC92XX)
	sensor_info->m_clock 				= 240000;
	sensor_info->m_clock_source 			= PCDIRECTPLL2;
	sensor_info->s_clock 					= 480000;
	sensor_info->s_clock_source 			= PCDIRECTPLL2;
	#elif defined(CONFIG_ARCH_TCC93XX)
	#if defined(CONFIG_USE_ISP)
	sensor_info->m_clock 				= 240000;
	sensor_info->m_clock_source 			= PCDIRECTPLL1;
	sensor_info->s_clock 					= 480000;
	sensor_info->s_clock_source 			= PCDIRECTPLL2;
	#else // CONFIG_USE_ISP
	sensor_info->m_clock 				= 240000;
	sensor_info->m_clock_source 			= PCDIRECTPLL1;
	sensor_info->s_clock 					= 480000;
	sensor_info->s_clock_source 			= PCDIRECTPLL2;
	#endif // CONFIG_USE_ISP
	#elif defined(CONFIG_ARCH_TCC88XX) || defined(CONFIG_ARCH_TCC892X)
	#if defined(CONFIG_USE_ISP)
	sensor_info->m_clock 				= 240000;
	sensor_info->m_clock_source 			= PCDIRECTPLL2;
	sensor_info->s_clock 					= 480000;
	sensor_info->s_clock_source 			= PCDIRECTPLL2;
	#else // CONFIG_USE_ISP
	sensor_info->m_clock 				= 240000;
	sensor_info->m_clock_source 			= PCDIRECTPLL2;
	sensor_info->s_clock 					= 480000;
	sensor_info->s_clock_source 			= PCDIRECTPLL2;
	#endif // CONFIG_USE_ISP
	#endif
	sensor_info->preview_w 				= 640;
	sensor_info->preview_h 				= 480;
	sensor_info->preview_zoom_offset_x 	= 8;
	sensor_info->preview_zoom_offset_y 	= 6;
	sensor_info->capture_w 				= 640;
	sensor_info->capture_h 				= 480;
	sensor_info->capture_zoom_offset_x 	= 8;
	sensor_info->capture_zoom_offset_y 	= 6;
	sensor_info->max_zoom_step 			= 15;
	sensor_info->cam_capchg_width 		= 640;
	sensor_info->framerate				= 15;
	sensor_info->p_clock_pol 				= NEGATIVE_EDGE;
	sensor_info->v_sync_pol 				= ACT_LOW;
	sensor_info->h_sync_pol                 = ACT_HIGH;
	sensor_info->format 					= M420_ZERO;
	sensor_info->capture_skip_frame 		= 1;
	sensor_info->sensor_sizes 			= sensor_sizes_sp0838;
}

void sensor_init_fnc_sp0838(SENSOR_FUNC_TYPE *sensor_func)
{
	sensor_func->Open 					= sensor_open_sp0838;
	sensor_func->Close 					= sensor_close_sp0838;
	sensor_func->PowerDown				= sensor_powerdown_sp0838;

	sensor_func->Set_Preview 			= sensor_preview_sp0838;
	sensor_func->Set_Capture 			= sensor_capture_sp0838;
	sensor_func->Set_CaptureCfg 			= sensor_capturecfg_sp0838;

	sensor_func->Set_Zoom 				= sensor_zoom_sp0838;
	sensor_func->Set_AF 				= sensor_autofocus_sp0838;
	sensor_func->Set_Effect 				= sensor_effect_sp0838;
	sensor_func->Set_Flip 				= sensor_flip_sp0838;
	sensor_func->Set_ISO 				= sensor_iso_sp0838;
	sensor_func->Set_ME 				= sensor_me_sp0838;
	sensor_func->Set_WB 				= sensor_wb_sp0838;
	sensor_func->Set_Bright 				= sensor_bright_sp0838;
	sensor_func->Set_Scene 				= sensor_scene_sp0838;

	sensor_func->Check_ESD 				= sensor_check_esd_sp0838;
	sensor_func->Check_Luma 			= sensor_check_luma_sp0838;
}
#else
void sensor_init_fnc(SENSOR_FUNC_TYPE *sensor_func)
{
	sensor_func->Open 				= sensor_open_sp0838;
	sensor_func->Close 				= sensor_close_sp0838;
	sensor_func->PowerDown				= sensor_powerdown_sp0838;

	sensor_func->Set_Preview 			= sensor_preview_sp0838;
	sensor_func->Set_Capture 			= sensor_capture_sp0838;
	sensor_func->Set_CaptureCfg 			= sensor_capturecfg_sp0838;

	sensor_func->Set_Zoom 				= sensor_zoom_sp0838;
	sensor_func->Set_AF 				= sensor_autofocus_sp0838;
	sensor_func->Set_Effect 				= sensor_effect_sp0838;
	sensor_func->Set_Flip 				= sensor_flip_sp0838;
	sensor_func->Set_ISO 				= sensor_iso_sp0838;
	sensor_func->Set_ME 				= sensor_me_sp0838;
	sensor_func->Set_WB 				= sensor_wb_sp0838;
	sensor_func->Set_Bright 				= sensor_bright_sp0838;
	sensor_func->Set_Scene 				= sensor_scene_sp0838;

	sensor_func->Check_ESD 				= sensor_check_esd_sp0838;
	sensor_func->Check_Luma 			= sensor_check_luma_sp0838;
}
#endif
#endif

