/*
 * Basic window sizes.  These probably belong somewhere more globally
 * useful.
 */

#define SP0838_ID  0x27 

#define VGA_WIDTH	640
#define VGA_HEIGHT	480
#define QVGA_WIDTH	320
#define QVGA_HEIGHT	240
#define QCIF_WIDTH	176
#define	QCIF_HEIGHT	144

/*
 * Our nominal (default) frame rate.
 */



//#define SP0838_FRAME_RATE 7//15

/* Registers */
#define SP0838_I2C_ADDR 0x30>>1

#define COM7_FMT_VGA 0x00
//#define REG_COM7	0xfe	/* Reset related */
//#define COM7_RESET	  0x80	  /* Register reset */
#define REG_PID		0x02	/* Product ID */
//#define REG_BRIGHT	0x13	/* Brightness */

//ECLK Drv
#define  Pre_Value_P0_0x30  0x00
//Filter en&dis
#define  Pre_Value_P0_0x56  0x70
#define  Pre_Value_P0_0x57  0x10  //filter outdoor
#define  Pre_Value_P0_0x58  0x10  //filter indoor
#define  Pre_Value_P0_0x59  0x10  //filter night
#define  Pre_Value_P0_0x5a  0x02  //smooth outdoor
#define  Pre_Value_P0_0x5b  0x02  //smooth indoor
#define  Pre_Value_P0_0x5c  0x20  //smooht night
//outdoor sharpness
#define  Pre_Value_P0_0x65  0x03
#define  Pre_Value_P0_0x66  0x01
#define  Pre_Value_P0_0x67  0x03
#define  Pre_Value_P0_0x68  0x46
//indoor sharpness
#define  Pre_Value_P0_0x6b  0x04
#define  Pre_Value_P0_0x6c  0x01
#define  Pre_Value_P0_0x6d  0x03
#define  Pre_Value_P0_0x6e  0x46
//night sharpness
#define  Pre_Value_P0_0x71  0x05
#define  Pre_Value_P0_0x72  0x01
#define  Pre_Value_P0_0x73  0x03
#define  Pre_Value_P0_0x74  0x46
//color
#define  Pre_Value_P0_0x7f  0xd7  //R 
#define  Pre_Value_P0_0x87  0xf8  //B
//satutation
#define  Pre_Value_P0_0xd8  0x48
#define  Pre_Value_P0_0xd9  0x48
#define  Pre_Value_P0_0xda  0x48
#define  Pre_Value_P0_0xdb  0x48
//AE target
#define  Pre_Value_P0_0xf7  0x78
#define  Pre_Value_P0_0xf8  0x63
#define  Pre_Value_P0_0xf9  0x68
#define  Pre_Value_P0_0xfa  0x53
//HEQ
#define  Pre_Value_P0_0xdd  0x70
#define  Pre_Value_P0_0xde  0x90
//AWB pre gain
#define  Pre_Value_P1_0x28  0x75
#define  Pre_Value_P1_0x29  0x4e

//VBLANK
#define  Pre_Value_P0_0x05  0x00
#define  Pre_Value_P0_0x06  0x00
//HBLANK
#define  Pre_Value_P0_0x09  0x01
#define  Pre_Value_P0_0x0a  0x76


/* Information we maintain about a known sensor.*/


struct sp0838_format_struct;	/* coming later */
struct sp0838_info {
	struct sp0838_format_struct *fmt;	/* Current format */
	unsigned char sat;	/* Saturation value */
	int hue;		/* Hue value */
};

struct regval_list {
	unsigned char reg_num;
	unsigned char value;
};


/*The default register settings, as obtained from Micron.*/

static struct regval_list sp0838_init_regs[] = {     
//SP0838 ini
{0xfd,0x00},//P0
{0x1B,0x02},
{0x27,0xe8},
{0x28,0x0B},
{0x32,0x00},
{0x22,0xc0},
{0x26,0x10}, 
{0x5f,0x11},//Bayer order
{0xfd,0x01},//P1
{0x25,0x1a},//Awb start
{0x26,0xfb},
{0x28,Pre_Value_P1_0x28},
{0x29,Pre_Value_P1_0x29},
{0xfd,0x00},
{0xe7,0x03},
{0xe7,0x00},
{0xfd,0x01},
{0x31,0x60},//64
{0x32,0x18},
{0x4d,0xdc},
{0x4e,0x53},
{0x41,0x8c},
{0x42,0x57},
{0x55,0xff},
{0x56,0x00},
{0x59,0x82},
{0x5a,0x00},
{0x5d,0xff},
{0x5e,0x6f},
{0x57,0xff},
{0x58,0x00},
{0x5b,0xff},
{0x5c,0xa8},
{0x5f,0x75},
{0x60,0x00},
{0x2d,0x00},
{0x2e,0x00},
{0x2f,0x00},
{0x30,0x00},
{0x33,0x00},
{0x34,0x00},
{0x37,0x00},
{0x38,0x00},//awb end
{0xfd,0x00},//P0
{0x33,0x6f},//LSC BPC EN
{0x51,0x3f},//BPC debug start
{0x52,0x09},
{0x53,0x00},
{0x54,0x00},
{0x55,0x10},//BPC debug end
{0x4f,0x08},//blueedge
{0x50,0x08},
{0x57,Pre_Value_P0_0x57},//Raw filter debut start
{0x58,Pre_Value_P0_0x58},
{0x59,Pre_Value_P0_0x59},
{0x56,Pre_Value_P0_0x56},
{0x5a,Pre_Value_P0_0x5a},
{0x5b,Pre_Value_P0_0x5b},
{0x5c,Pre_Value_P0_0x5c},//Raw filter debut end 
{0x65,Pre_Value_P0_0x65},//Sharpness debug start
{0x66,Pre_Value_P0_0x66},
{0x67,Pre_Value_P0_0x67},
{0x68,Pre_Value_P0_0x68},
{0x69,0x7f},
{0x6a,0x01},
{0x6b,Pre_Value_P0_0x6b},
{0x6c,Pre_Value_P0_0x6c},
{0x6d,Pre_Value_P0_0x6d},//Edge gain normal
{0x6e,Pre_Value_P0_0x6e},//Edge gain normal
{0x6f,0x7f},
{0x70,0x01},
{0x71,Pre_Value_P0_0x71}, //ֵ          
{0x72,Pre_Value_P0_0x72}, //ֵ        
{0x73,Pre_Value_P0_0x73}, //Եֵ    
{0x74,Pre_Value_P0_0x74}, //Եֵ    
{0x75,0x7f},              //ʹλ            
{0x76,0x01},//Sharpness debug end
{0xcb,0x07},//HEQ&Saturation debug start 
{0xcc,0x04},
{0xce,0xff},
{0xcf,0x10},
{0xd0,0x20},
{0xd1,0x00},
{0xd2,0x1c},
{0xd3,0x16},
{0xd4,0x00},
{0xd6,0x1c},
{0xd7,0x16},
{0xdd,Pre_Value_P0_0xdd},//Contrast
{0xde,Pre_Value_P0_0xde},//HEQ&Saturation debug end
{0x7f,Pre_Value_P0_0x7f},//Color Correction start
{0x80,0xbc},                        
{0x81,0xed},                        
{0x82,0xd7},                        
{0x83,0xd4},                        
{0x84,0xd6},                        
{0x85,0xff},                        
{0x86,0x89},                        
{0x87,Pre_Value_P0_0x87},                        
{0x88,0x3c},                        
{0x89,0x33},                        
{0x8a,0x0f},//Color Correction end  
{0x8b,0x0 },//gamma start
{0x8c,0x1a},             
{0x8d,0x29},             
{0x8e,0x41},             
{0x8f,0x62},             
{0x90,0x7c},             
{0x91,0x90},             
{0x92,0xa2},             
{0x93,0xaf},             
{0x94,0xbc},             
{0x95,0xc5},             
{0x96,0xcd},             
{0x97,0xd5},             
{0x98,0xdd},             
{0x99,0xe5},             
{0x9a,0xed},             
{0x9b,0xf5},             
{0xfd,0x01},//P1         
{0x8d,0xfd},             
{0x8e,0xff},//gamma end  
{0xfd,0x00},//P0
{0xca,0xcf},
{0xd8,Pre_Value_P0_0xd8},//UV outdoor
{0xd9,Pre_Value_P0_0xd9},//UV indoor 
{0xda,Pre_Value_P0_0xda},//UV dummy
{0xdb,Pre_Value_P0_0xdb},//UV lowlight
{0xb9,0x00},//Ygamma start
{0xba,0x04},
{0xbb,0x08},
{0xbc,0x10},
{0xbd,0x20},
{0xbe,0x30},
{0xbf,0x40},
{0xc0,0x50},
{0xc1,0x60},
{0xc2,0x70},
{0xc3,0x80},
{0xc4,0x90},
{0xc5,0xA0},
{0xc6,0xB0},
{0xc7,0xC0},
{0xc8,0xD0},
{0xc9,0xE0},
{0xfd,0x01},//P1
{0x89,0xf0},
{0x8a,0xff},//Ygamma end
{0xfd,0x00},//P0
{0xe8,0x30},//AEdebug start
{0xe9,0x30},
{0xea,0x40},//Alc Window sel
{0xf4,0x1b},//outdoor mode sel
{0xf5,0x80},
{0xf7,Pre_Value_P0_0xf7},//AE target
{0xf8,Pre_Value_P0_0xf8},
{0xf9,Pre_Value_P0_0xf9},//AE target 
{0xfa,Pre_Value_P0_0xfa},
{0xfd,0x01},//P1
{0x09,0x31},//AE Step 3.0
{0x0a,0x85},
{0x0b,0x0b},//AE Step 3.0
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},//P0
{0x05,0x00},  
{0x06,0x00},
{0x09,0x01},
{0x0a,0x76},
{0xf0,0x62},
{0xf1,0x00},
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},//P1
{0x00,0xba},
{0x0f,0x60},
{0x16,0x60},
{0x17,0xa2},
{0x18,0xaa},
{0x1b,0x60},
{0x1c,0xaa},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x78},
{0xc6,0x41},
{0xca,0x78},
{0xcb,0x0c},//AEdebug end
#if 1//caprure preview daylight 24M 50hz 20-8FPS maxgain:0x70	
{0xfd,0x00},
{0x05,Pre_Value_P0_0x05 },
{0x06,Pre_Value_P0_0x06 },
{0x09,Pre_Value_P0_0x09 },
{0x0a,Pre_Value_P0_0x0a },
{0xf0,0x62},
{0xf1,0x0 },
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},
{0x00,0xb2},
{0x0f,0x60},
{0x16,0x60},
{0x17,0xa2},
{0x18,0xaa},
{0x1b,0x60},
{0x1c,0xaa},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x70},
{0xc6,0x41},
{0xca,0x70},
{0xcb,0xc },
{0xfd,0x00},
#endif	
{0xfd,0x00},  //P0
{0x32,0x15},  //Auto_mode set
{0x34,0x66},  //Isp_mode set
{0x35,0x40},  //out format
//{0x36,0x00},//zyy add 
{0xFF,0xFF},
};


static struct regval_list sp0838_fmt_yuv422_vga[] = {

    { 0xfd, 0x00 },
    
    
    { 0x31, 0x10 },  
  
    
	{ 0xff, 0xff },

};

static struct regval_list sp0838_fmt_yuv422_qvga[] = {

    { 0xfd, 0x00 },
   
    
    { 0x31, 0x14 },  
  
    
	{ 0xff, 0xff },

};

static struct regval_list sp0838_fmt_yuv422_qcif[] = {


	{ 0xff, 0xff },

};

