// Here is where all the model specific defines should be


#include "hal_i2c.h"
#include "hal_camera.h"
#include "camera_m.h"

typedef struct
{
    UINT8 Addr;            // Register Address
    UINT8 Data;             // Register Data
    UINT8 Wait;            // Time to wait after write
} CAM_REG_T;

typedef struct
{
    UINT16 Len;             // Size of array
    CAM_REG_T *RegVal;      // Array of registers
} CAM_REG_GROUP_T;

typedef enum
{
    CAM_MODE_IDLE,
    CAM_MODE_VIEWFINDER,
    CAM_MODE_PREP_IMG_CAPTURE,
    CAM_MODE_IMG_CAPTURE,
    CAM_MODE_PREP_VID_CAPTURE,
    CAM_MODE_VID_CAPTURE
} CAM_MODE_T;

// On FPGA platform, Pixel Clock must not be too fast or xfer screws up
#define CAM_CLK_CFG 0x81

// use this for 320x240 (VGA) capture
PRIVATE const CAM_REG_T RG_InitPowerOnQvga[] = 
{	
{0xfd , 0x00, 1}, //P0
{0x1B , 0x02, 1},
{0x27 , 0xe8, 1},
{0x28 , 0x0B, 1},
{0x32 , 0x00, 1},
{0x22 , 0xc0, 1},
{0x26 , 0x10, 1}, 
{0x31 , 0x10, 1},   //Upside/mirr/Pclk inv/sub
{0x5f , 0x11, 1},   //Bayer order
{0xfd , 0x01, 1},   //P1
{0x25 , 0x1a, 1},   //Awb start
{0x26 , 0xfb, 1}, 
{0x28 , 0x75, 1}, 
{0x29 , 0x4e, 1},
{0xfd , 0x00, 1},   
{0xe7 , 0x03, 1}, 
{0xe7 , 0x00, 1}, 
{0xfd , 0x01, 1},

{0x31 , 0x60, 1},//64
{0x32 , 0x18, 1},
{0x4d , 0xdc, 1},
{0x4e , 0x53, 1},
{0x41 , 0x8c, 1},
{0x42 , 0x57, 1},
{0x55 , 0xff, 1},
{0x56 , 0x00, 1},
{0x59 , 0x82, 1},
{0x5a , 0x00, 1},
{0x5d , 0xff, 1},
{0x5e , 0x6f, 1},
{0x57 , 0xff, 1},
{0x58 , 0x00, 1},
{0x5b , 0xff, 1},
{0x5c , 0xa8, 1},
{0x5f , 0x75, 1},
{0x60 , 0x00, 1},
{0x2d , 0x00, 1},
{0x2e , 0x00, 1},
{0x2f , 0x00, 1},
{0x30 , 0x00, 1},
{0x33 , 0x00, 1},
{0x34 , 0x00, 1},
{0x37 , 0x00, 1},
{0x38 , 0x00, 1},  //awb end
{0xfd , 0x00, 1},  //P0
{0x33 , 0x6f, 1},  //LSC BPC EN
{0x51 , 0x3f, 1},  //BPC debug start
{0x52 , 0x09, 1},  
{0x53 , 0x00, 1},  
{0x54 , 0x00, 1},
{0x55 , 0x10, 1},  //BPC debug end
{0x4f , 0x08, 1},  //blueedge
{0x50 , 0x08, 1},
{0x57 , 0x10, 1},  //Raw filter debut start
{0x58 , 0x10, 1},
{0x59 , 0x10, 1},
{0x56 , 0x70, 1},
{0x5a , 0x02, 1},
{0x5b , 0x02, 1},
{0x5c , 0x20, 1},  //Raw filter debut end 
{0x65 , 0x03, 1},  //Sharpness debug start
{0x66 , 0x01, 1},
{0x67 , 0x03, 1},
{0x68 , 0x46, 1},
{0x69 , 0x7f, 1},
{0x6a , 0x01, 1},
{0x6b , 0x04, 1},
{0x6c , 0x01, 1},
{0x6d , 0x03, 1},  //Edge gain normal
{0x6e , 0x46, 1},  //Edge gain normal
{0x6f , 0x7f, 1},
{0x70 , 0x01, 1},  
{0x71 , 0x05, 1},  //ֵ            
{0x72 , 0x01, 1},  //ֵ          
{0x73 , 0x03, 1},  //Եֵ      
{0x74 , 0x46, 1},  //Եֵ      
{0x75 , 0x7f, 1},  //ʹλ              
{0x76 , 0x01, 1},  //Sharpness debug end
{0xcb , 0x07, 1},  //HEQ&Saturation debug start 
{0xcc , 0x04, 1},
{0xce , 0xff, 1},
{0xcf , 0x10, 1},
{0xd0 , 0x20, 1},
{0xd1 , 0x00, 1},
{0xd2 , 0x1c, 1},
{0xd3 , 0x16, 1},
{0xd4 , 0x00, 1},
{0xd6 , 0x1c, 1},
{0xd7 , 0x16, 1},
{0xdd , 0x70, 1},  //Contrast
{0xde , 0x90, 1},  //HEQ&Saturation debug end
{0x7f , 0xd7, 1},  //Color Correction start
{0x80 , 0xbc, 1},                          
{0x81 , 0xed, 1},                          
{0x82 , 0xd7, 1},                          
{0x83 , 0xd4, 1},                          
{0x84 , 0xd6, 1},                          
{0x85 , 0xff, 1},                          
{0x86 , 0x89, 1},                          
{0x87 , 0xf8, 1},                          
{0x88 , 0x3c, 1},                          
{0x89 , 0x33, 1},                          
{0x8a , 0x0f, 1},  //Color Correction end 
{0x8b , 0x00, 1},  //gamma start
{0x8c , 0x1a, 1},               
{0x8d , 0x29, 1},               
{0x8e , 0x41, 1},               
{0x8f , 0x62, 1},               
{0x90 , 0x7c, 1},               
{0x91 , 0x90, 1},               
{0x92 , 0xa2, 1},               
{0x93 , 0xaf, 1},               
{0x94 , 0xbc, 1},               
{0x95 , 0xc5, 1},               
{0x96 , 0xcd, 1},               
{0x97 , 0xd5, 1},               
{0x98 , 0xdd, 1},               
{0x99 , 0xe5, 1},               
{0x9a , 0xed, 1},               
{0x9b , 0xf5, 1},               
{0xfd , 0x01, 1},  //P1         
{0x8d , 0xfd, 1},               
{0x8e , 0xff, 1},  //gamma end  
{0xfd , 0x00, 1},  //P0
{0xca , 0xcf, 1},
{0xd8 , 0x48, 1},  //UV outdoor
{0xd9 , 0x48, 1},  //UV indoor 
{0xda , 0x48, 1},  //UV dummy
{0xdb , 0x48, 1},  //UV lowlight
{0xb9 , 0x00, 1},  //Ygamma start
{0xba , 0x04, 1},
{0xbb , 0x08, 1},
{0xbc , 0x10, 1},
{0xbd , 0x20, 1},
{0xbe , 0x30, 1},
{0xbf , 0x40, 1},
{0xc0 , 0x50, 1},
{0xc1 , 0x60, 1},
{0xc2 , 0x70, 1},
{0xc3 , 0x80, 1},
{0xc4 , 0x90, 1},
{0xc5 , 0xA0, 1},
{0xc6 , 0xB0, 1},
{0xc7 , 0xC0, 1},
{0xc8 , 0xD0, 1},
{0xc9 , 0xE0, 1},
{0xfd , 0x01, 1},  //P1
{0x89 , 0xf0, 1},
{0x8a , 0xff, 1},  //Ygamma end
{0xfd , 0x00, 1},  //P0
{0xe8 , 0x30, 1},  //AEdebug start
{0xe9 , 0x30, 1},
{0xea , 0x40, 1},  //Alc Window sel
{0xf4 , 0x1b, 1},  //outdoor mode sel
{0xf5 , 0x80, 1},
{0xf7 , 0x78, 1},  //AE target
{0xf8 , 0x63, 1},  
{0xf9 , 0x68, 1},  //AE target
{0xfa , 0x53, 1},
{0xfd , 0x01, 1},  //P1
{0x09 , 0x31, 1},  //AE Step 3.0
{0x0a , 0x85, 1},
{0x0b , 0x0b, 1},  //AE Step 3.0
{0x14 , 0x20, 1},
{0x15 , 0x0f, 1},
{0xfd , 0x00, 1},  //P0
{0x05 , 0x00, 1},  
{0x06 , 0x00, 1},
{0x09 , 0x01, 1},
{0x0a , 0x76, 1},
{0xf0 , 0x62, 1},
{0xf1 , 0x00, 1},
{0xf2 , 0x5f, 1},
{0xf5 , 0x78, 1},
{0xfd , 0x01, 1},  //P1
{0x00 , 0xba, 1},
{0x0f , 0x60, 1},
{0x16 , 0x60, 1},
{0x17 , 0xa2, 1},
{0x18 , 0xaa, 1},
{0x1b , 0x60, 1},
{0x1c , 0xaa, 1},
{0xb4 , 0x20, 1},
{0xb5 , 0x3a, 1},
{0xb6 , 0x5e, 1},
{0xb9 , 0x40, 1},
{0xba , 0x4f, 1},
{0xbb , 0x47, 1},
{0xbc , 0x45, 1},
{0xbd , 0x43, 1},
{0xbe , 0x42, 1},
{0xbf , 0x42, 1},
{0xc0 , 0x42, 1},
{0xc1 , 0x41, 1},
{0xc2 , 0x41, 1},
{0xc3 , 0x41, 1},
{0xc4 , 0x41, 1},
{0xc5 , 0x78, 1},
{0xc6 , 0x41, 1},
{0xca , 0x78, 1},
{0xcb , 0x0c, 1},  //AEdebug end

#if 1//caprure preview daylight 26M 50hz 20-8FPS maxgain:0x70	
{0xfd , 0x00, 1},
{0x05 , 0x0 , 1},
{0x06 , 0x0 , 1},
{0x09 , 0x1 , 1},
{0x0a , 0xdd, 1},
{0xf0 , 0x62, 1},
{0xf1 , 0x0 , 1},
{0xf2 , 0x5f, 1},
{0xf5 , 0x78, 1},
{0xfd , 0x01, 1},
{0x00 , 0xb2, 1},
{0x0f , 0x60, 1},
{0x16 , 0x60, 1},
{0x17 , 0xa2, 1},
{0x18 , 0xaa, 1},
{0x1b , 0x60, 1},
{0x1c , 0xaa, 1},
{0xb4 , 0x20, 1},
{0xb5 , 0x3a, 1},
{0xb6 , 0x5e, 1},
{0xb9 , 0x40, 1},
{0xba , 0x4f, 1},
{0xbb , 0x47, 1},
{0xbc , 0x45, 1},
{0xbd , 0x43, 1},
{0xbe , 0x42, 1},
{0xbf , 0x42, 1},
{0xc0 , 0x42, 1},
{0xc1 , 0x41, 1},
{0xc2 , 0x41, 1},
{0xc3 , 0x41, 1},
{0xc4 , 0x41, 1},
{0xc5 , 0x70, 1},
{0xc6 , 0x41, 1},
{0xca , 0x70, 1},
{0xcb , 0xc , 1},
{0xfd , 0x00, 1},
#endif	

{0xfd , 0x00, 1},  //P0
{0x32 , 0x15, 1},  //Auto_mode set
{0x34 , 0x66, 1},  //Isp_mode set
{0x35 , 0x00, 1},  //out format
};

// use this for 640x480 (VGA) capture
PRIVATE const CAM_REG_T RG_InitPowerOnVga[] =  
{
{0xfd , 0x00, 1}, //P0
{0x1B , 0x02, 1},
{0x27 , 0xe8, 1},
{0x28 , 0x0B, 1},
{0x32 , 0x00, 1},
{0x22 , 0xc0, 1},
{0x26 , 0x10, 1}, 
{0x31 , 0x10, 1},   //Upside/mirr/Pclk inv/sub
{0x5f , 0x11, 1},   //Bayer order
{0xfd , 0x01, 1},   //P1
{0x25 , 0x1a, 1},   //Awb start
{0x26 , 0xfb, 1}, 
{0x28 , 0x75, 1}, 
{0x29 , 0x4e, 1},
{0xfd , 0x00, 1},   
{0xe7 , 0x03, 1}, 
{0xe7 , 0x00, 1}, 
{0xfd , 0x01, 1},

{0x31 , 0x60, 1},//64
{0x32 , 0x18, 1},
{0x4d , 0xdc, 1},
{0x4e , 0x53, 1},
{0x41 , 0x8c, 1},
{0x42 , 0x57, 1},
{0x55 , 0xff, 1},
{0x56 , 0x00, 1},
{0x59 , 0x82, 1},
{0x5a , 0x00, 1},
{0x5d , 0xff, 1},
{0x5e , 0x6f, 1},
{0x57 , 0xff, 1},
{0x58 , 0x00, 1},
{0x5b , 0xff, 1},
{0x5c , 0xa8, 1},
{0x5f , 0x75, 1},
{0x60 , 0x00, 1},
{0x2d , 0x00, 1},
{0x2e , 0x00, 1},
{0x2f , 0x00, 1},
{0x30 , 0x00, 1},
{0x33 , 0x00, 1},
{0x34 , 0x00, 1},
{0x37 , 0x00, 1},
{0x38 , 0x00, 1},  //awb end
{0xfd , 0x00, 1},  //P0
{0x33 , 0x6f, 1},  //LSC BPC EN
{0x51 , 0x3f, 1},  //BPC debug start
{0x52 , 0x09, 1},  
{0x53 , 0x00, 1},  
{0x54 , 0x00, 1},
{0x55 , 0x10, 1},  //BPC debug end
{0x4f , 0x08, 1},  //blueedge
{0x50 , 0x08, 1},
{0x57 , 0x10, 1},  //Raw filter debut start
{0x58 , 0x10, 1},
{0x59 , 0x10, 1},
{0x56 , 0x70, 1},
{0x5a , 0x02, 1},
{0x5b , 0x02, 1},
{0x5c , 0x20, 1},  //Raw filter debut end 
{0x65 , 0x03, 1},  //Sharpness debug start
{0x66 , 0x01, 1},
{0x67 , 0x03, 1},
{0x68 , 0x46, 1},
{0x69 , 0x7f, 1},
{0x6a , 0x01, 1},
{0x6b , 0x04, 1},
{0x6c , 0x01, 1},
{0x6d , 0x03, 1},  //Edge gain normal
{0x6e , 0x46, 1},  //Edge gain normal
{0x6f , 0x7f, 1},
{0x70 , 0x01, 1},
{0x71 , 0x05, 1},
{0x72 , 0x01, 1},
{0x73 , 0x03, 1},
{0x74 , 0x46, 1},
{0x75 , 0x7f, 1},
{0x76 , 0x01, 1},  //Sharpness debug end
{0xcb , 0x07, 1},  //HEQ&Saturation debug start 
{0xcc , 0x04, 1},
{0xce , 0xff, 1},
{0xcf , 0x10, 1},
{0xd0 , 0x20, 1},
{0xd1 , 0x00, 1},
{0xd2 , 0x1c, 1},
{0xd3 , 0x16, 1},
{0xd4 , 0x00, 1},
{0xd6 , 0x1c, 1},
{0xd7 , 0x16, 1},
{0xdd , 0x70, 1},  //Contrast
{0xde , 0x90, 1},  //HEQ&Saturation debug end
{0x7f , 0xd7, 1},  //Color Correction start
{0x80 , 0xbc, 1},                          
{0x81 , 0xed, 1},                          
{0x82 , 0xd7, 1},                          
{0x83 , 0xd4, 1},                          
{0x84 , 0xd6, 1},                          
{0x85 , 0xff, 1},                          
{0x86 , 0x89, 1},                          
{0x87 , 0xf8, 1},                          
{0x88 , 0x3c, 1},                          
{0x89 , 0x33, 1},                          
{0x8a , 0x0f, 1},  //Color Correction end 
{0x8b , 0x00, 1},  //gamma start
{0x8c , 0x1a, 1},               
{0x8d , 0x29, 1},               
{0x8e , 0x41, 1},               
{0x8f , 0x62, 1},               
{0x90 , 0x7c, 1},               
{0x91 , 0x90, 1},               
{0x92 , 0xa2, 1},               
{0x93 , 0xaf, 1},               
{0x94 , 0xbc, 1},               
{0x95 , 0xc5, 1},               
{0x96 , 0xcd, 1},               
{0x97 , 0xd5, 1},               
{0x98 , 0xdd, 1},               
{0x99 , 0xe5, 1},               
{0x9a , 0xed, 1},               
{0x9b , 0xf5, 1},               
{0xfd , 0x01, 1},  //P1         
{0x8d , 0xfd, 1},               
{0x8e , 0xff, 1},  //gamma end  
{0xfd , 0x00, 1},  //P0
{0xca , 0xcf, 1},
{0xd8 , 0x48, 1},  //UV outdoor
{0xd9 , 0x48, 1},  //UV indoor 
{0xda , 0x48, 1},  //UV dummy
{0xdb , 0x48, 1},  //UV lowlight
{0xb9 , 0x00, 1},  //Ygamma start
{0xba , 0x04, 1},
{0xbb , 0x08, 1},
{0xbc , 0x10, 1},
{0xbd , 0x20, 1},
{0xbe , 0x30, 1},
{0xbf , 0x40, 1},
{0xc0 , 0x50, 1},
{0xc1 , 0x60, 1},
{0xc2 , 0x70, 1},
{0xc3 , 0x80, 1},
{0xc4 , 0x90, 1},
{0xc5 , 0xA0, 1},
{0xc6 , 0xB0, 1},
{0xc7 , 0xC0, 1},
{0xc8 , 0xD0, 1},
{0xc9 , 0xE0, 1},
{0xfd , 0x01, 1},  //P1
{0x89 , 0xf0, 1},
{0x8a , 0xff, 1},  //Ygamma end
{0xfd , 0x00, 1},  //P0
{0xe8 , 0x30, 1},  //AEdebug start
{0xe9 , 0x30, 1},
{0xea , 0x40, 1},  //Alc Window sel
{0xf4 , 0x1b, 1},  //outdoor mode sel
{0xf5 , 0x80, 1},
{0xf7 , 0x78, 1},  //AE target
{0xf8 , 0x63, 1},  
{0xf9 , 0x68, 1},  //AE target
{0xfa , 0x53, 1},
{0xfd , 0x01, 1},  //P1
{0x09 , 0x31, 1},  //AE Step 3.0
{0x0a , 0x85, 1},
{0x0b , 0x0b, 1},  //AE Step 3.0
{0x14 , 0x20, 1},
{0x15 , 0x0f, 1},
{0xfd , 0x00, 1},  //P0
{0x05 , 0x00, 1},  
{0x06 , 0x00, 1},
{0x09 , 0x01, 1},
{0x0a , 0x76, 1},
{0xf0 , 0x62, 1},
{0xf1 , 0x00, 1},
{0xf2 , 0x5f, 1},
{0xf5 , 0x78, 1},
{0xfd , 0x01, 1},  //P1
{0x00 , 0xba, 1},
{0x0f , 0x60, 1},
{0x16 , 0x60, 1},
{0x17 , 0xa2, 1},
{0x18 , 0xaa, 1},
{0x1b , 0x60, 1},
{0x1c , 0xaa, 1},
{0xb4 , 0x20, 1},
{0xb5 , 0x3a, 1},
{0xb6 , 0x5e, 1},
{0xb9 , 0x40, 1},
{0xba , 0x4f, 1},
{0xbb , 0x47, 1},
{0xbc , 0x45, 1},
{0xbd , 0x43, 1},
{0xbe , 0x42, 1},
{0xbf , 0x42, 1},
{0xc0 , 0x42, 1},
{0xc1 , 0x41, 1},
{0xc2 , 0x41, 1},
{0xc3 , 0x41, 1},
{0xc4 , 0x41, 1},
{0xc5 , 0x78, 1},
{0xc6 , 0x41, 1},
{0xca , 0x78, 1},
{0xcb , 0x0c, 1},  //AEdebug end

#if 1//caprure preview daylight 26M 50hz 20-8FPS maxgain:0x70	
{0xfd , 0x00, 1},
{0x05 , 0x0 , 1},
{0x06 , 0x0 , 1},
{0x09 , 0x1 , 1},
{0x0a , 0xdd, 1},
{0xf0 , 0x62, 1},
{0xf1 , 0x0 , 1},
{0xf2 , 0x5f, 1},
{0xf5 , 0x78, 1},
{0xfd , 0x01, 1},
{0x00 , 0xb2, 1},
{0x0f , 0x60, 1},
{0x16 , 0x60, 1},
{0x17 , 0xa2, 1},
{0x18 , 0xaa, 1},
{0x1b , 0x60, 1},
{0x1c , 0xaa, 1},
{0xb4 , 0x20, 1},
{0xb5 , 0x3a, 1},
{0xb6 , 0x5e, 1},
{0xb9 , 0x40, 1},
{0xba , 0x4f, 1},
{0xbb , 0x47, 1},
{0xbc , 0x45, 1},
{0xbd , 0x43, 1},
{0xbe , 0x42, 1},
{0xbf , 0x42, 1},
{0xc0 , 0x42, 1},
{0xc1 , 0x41, 1},
{0xc2 , 0x41, 1},
{0xc3 , 0x41, 1},
{0xc4 , 0x41, 1},
{0xc5 , 0x70, 1},
{0xc6 , 0x41, 1},
{0xca , 0x70, 1},
{0xcb , 0xc , 1},
{0xfd , 0x00, 1},
#endif	

{0xfd , 0x00, 1},  //P0
{0x32 , 0x15, 1},  //Auto_mode set
{0x34 , 0x66, 1},  //Isp_mode set
{0x35 , 0x00, 1},  //out format
};

// use this for 160*120 (QQVGA) capture
PRIVATE const CAM_REG_T RG_InitPowerOnQqvga[] = 
{
{0xfd , 0x00, 1}, //P0
{0x1B , 0x02, 1},
{0x27 , 0xe8, 1},
{0x28 , 0x0B, 1},
{0x32 , 0x00, 1},
{0x22 , 0xc0, 1},
{0x26 , 0x10, 1}, 
{0x31 , 0x10, 1},   //Upside/mirr/Pclk inv/sub
{0x5f , 0x11, 1},   //Bayer order
{0xfd , 0x01, 1},   //P1
{0x25 , 0x1a, 1},   //Awb start
{0x26 , 0xfb, 1}, 
{0x28 , 0x75, 1}, 
{0x29 , 0x4e, 1},
{0xfd , 0x00, 1},   
{0xe7 , 0x03, 1}, 
{0xe7 , 0x00, 1}, 
{0xfd , 0x01, 1},
{0x31 , 0x60, 1},//64
{0x32 , 0x18, 1},
{0x4d , 0xdc, 1},
{0x4e , 0x53, 1},
{0x41 , 0x8c, 1},
{0x42 , 0x57, 1},
{0x55 , 0xff, 1},
{0x56 , 0x00, 1},
{0x59 , 0x82, 1},
{0x5a , 0x00, 1},
{0x5d , 0xff, 1},
{0x5e , 0x6f, 1},
{0x57 , 0xff, 1},
{0x58 , 0x00, 1},
{0x5b , 0xff, 1},
{0x5c , 0xa8, 1},
{0x5f , 0x75, 1},
{0x60 , 0x00, 1},
{0x2d , 0x00, 1},
{0x2e , 0x00, 1},
{0x2f , 0x00, 1},
{0x30 , 0x00, 1},
{0x33 , 0x00, 1},
{0x34 , 0x00, 1},
{0x37 , 0x00, 1},
{0x38 , 0x00, 1},  //awb end
{0xfd , 0x00, 1},  //P0
{0x33 , 0x6f, 1},  //LSC BPC EN
{0x51 , 0x3f, 1},  //BPC debug start
{0x52 , 0x09, 1},  
{0x53 , 0x00, 1},  
{0x54 , 0x00, 1},
{0x55 , 0x10, 1},  //BPC debug end
{0x4f , 0x08, 1},  //blueedge
{0x50 , 0x08, 1},
{0x57 , 0x10, 1},  //Raw filter debut start
{0x58 , 0x10, 1},
{0x59 , 0x10, 1},
{0x56 , 0x70, 1},
{0x5a , 0x02, 1},
{0x5b , 0x02, 1},
{0x5c , 0x20, 1},  //Raw filter debut end 
{0x65 , 0x03, 1},  //Sharpness debug start
{0x66 , 0x01, 1},
{0x67 , 0x03, 1},
{0x68 , 0x46, 1},
{0x69 , 0x7f, 1},
{0x6a , 0x01, 1},
{0x6b , 0x04, 1},
{0x6c , 0x01, 1},
{0x6d , 0x03, 1},  //Edge gain normal
{0x6e , 0x46, 1},  //Edge gain normal
{0x6f , 0x7f, 1},
{0x70 , 0x01, 1},
{0x71 , 0x05, 1},
{0x72 , 0x01, 1},
{0x73 , 0x03, 1},
{0x74 , 0x46, 1},
{0x75 , 0x7f, 1},
{0x76 , 0x01, 1},  //Sharpness debug end
{0xcb , 0x07, 1},  //HEQ&Saturation debug start 
{0xcc , 0x04, 1},
{0xce , 0xff, 1},
{0xcf , 0x10, 1},
{0xd0 , 0x20, 1},
{0xd1 , 0x00, 1},
{0xd2 , 0x1c, 1},
{0xd3 , 0x16, 1},
{0xd4 , 0x00, 1},
{0xd6 , 0x1c, 1},
{0xd7 , 0x16, 1},
{0xdd , 0x70, 1},  //Contrast
{0xde , 0x90, 1},  //HEQ&Saturation debug end
{0x7f , 0xd7, 1},  //Color Correction start
{0x80 , 0xbc, 1},                          
{0x81 , 0xed, 1},                          
{0x82 , 0xd7, 1},                          
{0x83 , 0xd4, 1},                          
{0x84 , 0xd6, 1},                          
{0x85 , 0xff, 1},                          
{0x86 , 0x89, 1},                          
{0x87 , 0xf8, 1},                          
{0x88 , 0x3c, 1},                          
{0x89 , 0x33, 1},                          
{0x8a , 0x0f, 1},  //Color Correction end 
{0x8b , 0x00, 1},  //gamma start
{0x8c , 0x1a, 1},               
{0x8d , 0x29, 1},               
{0x8e , 0x41, 1},               
{0x8f , 0x62, 1},               
{0x90 , 0x7c, 1},               
{0x91 , 0x90, 1},               
{0x92 , 0xa2, 1},               
{0x93 , 0xaf, 1},               
{0x94 , 0xbc, 1},               
{0x95 , 0xc5, 1},               
{0x96 , 0xcd, 1},               
{0x97 , 0xd5, 1},               
{0x98 , 0xdd, 1},               
{0x99 , 0xe5, 1},               
{0x9a , 0xed, 1},               
{0x9b , 0xf5, 1},               
{0xfd , 0x01, 1},  //P1         
{0x8d , 0xfd, 1},               
{0x8e , 0xff, 1},  //gamma end  
{0xfd , 0x00, 1},  //P0
{0xca , 0xcf, 1},
{0xd8 , 0x48, 1},  //UV outdoor
{0xd9 , 0x48, 1},  //UV indoor 
{0xda , 0x48, 1},  //UV dummy
{0xdb , 0x48, 1},  //UV lowlight
{0xb9 , 0x00, 1},  //Ygamma start
{0xba , 0x04, 1},
{0xbb , 0x08, 1},
{0xbc , 0x10, 1},
{0xbd , 0x20, 1},
{0xbe , 0x30, 1},
{0xbf , 0x40, 1},
{0xc0 , 0x50, 1},
{0xc1 , 0x60, 1},
{0xc2 , 0x70, 1},
{0xc3 , 0x80, 1},
{0xc4 , 0x90, 1},
{0xc5 , 0xA0, 1},
{0xc6 , 0xB0, 1},
{0xc7 , 0xC0, 1},
{0xc8 , 0xD0, 1},
{0xc9 , 0xE0, 1},
{0xfd , 0x01, 1},  //P1
{0x89 , 0xf0, 1},
{0x8a , 0xff, 1},  //Ygamma end
{0xfd , 0x00, 1},  //P0
{0xe8 , 0x30, 1},  //AEdebug start
{0xe9 , 0x30, 1},
{0xea , 0x40, 1},  //Alc Window sel
{0xf4 , 0x1b, 1},  //outdoor mode sel
{0xf5 , 0x80, 1},
{0xf7 , 0x78, 1},  //AE target
{0xf8 , 0x63, 1},  
{0xf9 , 0x68, 1},  //AE target
{0xfa , 0x53, 1},
{0xfd , 0x01, 1},  //P1
{0x09 , 0x31, 1},  //AE Step 3.0
{0x0a , 0x85, 1},
{0x0b , 0x0b, 1},  //AE Step 3.0
{0x14 , 0x20, 1},
{0x15 , 0x0f, 1},
{0xfd , 0x00, 1},  //P0
{0x05 , 0x00, 1},  
{0x06 , 0x00, 1},
{0x09 , 0x01, 1},
{0x0a , 0x76, 1},
{0xf0 , 0x62, 1},
{0xf1 , 0x00, 1},
{0xf2 , 0x5f, 1},
{0xf5 , 0x78, 1},
{0xfd , 0x01, 1},  //P1
{0x00 , 0xba, 1},
{0x0f , 0x60, 1},
{0x16 , 0x60, 1},
{0x17 , 0xa2, 1},
{0x18 , 0xaa, 1},
{0x1b , 0x60, 1},
{0x1c , 0xaa, 1},
{0xb4 , 0x20, 1},
{0xb5 , 0x3a, 1},
{0xb6 , 0x5e, 1},
{0xb9 , 0x40, 1},
{0xba , 0x4f, 1},
{0xbb , 0x47, 1},
{0xbc , 0x45, 1},
{0xbd , 0x43, 1},
{0xbe , 0x42, 1},
{0xbf , 0x42, 1},
{0xc0 , 0x42, 1},
{0xc1 , 0x41, 1},
{0xc2 , 0x41, 1},
{0xc3 , 0x41, 1},
{0xc4 , 0x41, 1},
{0xc5 , 0x78, 1},
{0xc6 , 0x41, 1},
{0xca , 0x78, 1},
{0xcb , 0x0c, 1},  //AEdebug end

#if 1//caprure preview daylight 26M 50hz 20-8FPS maxgain:0x70	
{0xfd , 0x00, 1},
{0x05 , 0x0 , 1},
{0x06 , 0x0 , 1},
{0x09 , 0x1 , 1},
{0x0a , 0xdd, 1},
{0xf0 , 0x62, 1},
{0xf1 , 0x0 , 1},
{0xf2 , 0x5f, 1},
{0xf5 , 0x78, 1},
{0xfd , 0x01, 1},
{0x00 , 0xb2, 1},
{0x0f , 0x60, 1},
{0x16 , 0x60, 1},
{0x17 , 0xa2, 1},
{0x18 , 0xaa, 1},
{0x1b , 0x60, 1},
{0x1c , 0xaa, 1},
{0xb4 , 0x20, 1},
{0xb5 , 0x3a, 1},
{0xb6 , 0x5e, 1},
{0xb9 , 0x40, 1},
{0xba , 0x4f, 1},
{0xbb , 0x47, 1},
{0xbc , 0x45, 1},
{0xbd , 0x43, 1},
{0xbe , 0x42, 1},
{0xbf , 0x42, 1},
{0xc0 , 0x42, 1},
{0xc1 , 0x41, 1},
{0xc2 , 0x41, 1},
{0xc3 , 0x41, 1},
{0xc4 , 0x41, 1},
{0xc5 , 0x70, 1},
{0xc6 , 0x41, 1},
{0xca , 0x70, 1},
{0xcb , 0xc , 1},
{0xfd , 0x00, 1},
#endif	

{0xfd , 0x00, 1},  //P0
{0x32 , 0x15, 1},  //Auto_mode set
{0x34 , 0x66, 1},  //Isp_mode set
{0x35 , 0x00, 1},  //out format
};

PRIVATE const CAM_REG_T RG_VidFormatVga[] = 
{
};

PRIVATE const CAM_REG_T RG_VidFormatQvga[] = 
{
};

PRIVATE const CAM_REG_T RG_VidFormatQqvga[] = 
{
};

typedef struct
{
    // Camera information structure
    char                desc[40];	

    UINT8               snrtype;            //sensor type(RGB or YUV)
    UINT8               pclk;               //use PCLK of sensor
    UINT8               clkcfg;             //[0]: VSync delay enable; [1]: Fifo VSync delay enable; [2]: Use PCLK Negtive Edge.
    HAL_I2C_BPS_T       i2crate;            //I2C rate : KHz				
    UINT8               i2caddress;         //I2C address
    UINT8               i2cispaddress;      //ISP I2C address
    UINT8               pwrcfg;             //sensor power initial configure(SIF REG801 BIT[0]~BIT[2])
    BOOL                rstactiveh;         //sensor reset active hi/low polarity (TRUE = active hi)
    BOOL                snrrst;             //Reset sensor enable

    CAM_MODE_T          cammode;


    BOOL                capturevideo;
    BOOL                captureframe;
    UINT8               numactivebuffers;
    UINT8               currentbuf;

    UINT8               brightness;         //brightness
    UINT8               contrast;           //contrast

    CAM_FORMAT_T        format;
    UINT32              npixels;
    CAM_IMAGE_T         vidbuffer;

    CAM_REG_GROUP_T     standby;            //sensor standby register
    CAM_REG_GROUP_T     Initpoweron;        //SENSOR ISP initial configure

    CAM_REG_GROUP_T     sifpwronseq;        //Sif config sequence(Reg.800 bit0~2) when sensor power on
    CAM_REG_GROUP_T     sifstdbyseq;        //Sif config sequence(Reg.800 bit0~2) when sensor standby
    CAM_REG_GROUP_T     dsif;               //SIF initial configure
    //    TSnrSizeCfgGroup    snrSizeCfg;         //sensor size configure information

    // This 'callback' mechanism is not necessary because the encapsulation is taken
    // care of by the environment
    //    PSensorSetReg       snrSetRegCall;      //set reg callback
    //    PSensorGetReg       snrGetRegCall;
    //    PSnrIdChk           snrIdChkCall;
    //    PSnrMirrorFlip      snrMirrorFlipCall;
    //    PSnrContrast        snrContrastCall;
    //    PSnrBrightness      snrBrightnessCall;
    //    PSnrMode            snrModeCall;
    //    PSnrGetEt           snrGetEt;
    //    PSnrSetEt           snrSetEt;
    //    TFlashParm          flashparm;
    CAM_IRQ_HANDLER_T   camirqhandler;
} CAM_INFO_T;

PRIVATE volatile CAM_INFO_T gSensorInfo =
{
    "SP0838",      // char desc[40];	

    0,                      // UINT8 snrtype; (rgb)
    1,                      // UINT8 pclk; use or don't use PCLK of sensor???
    0,                      // UINT8 clkcfg; ???
    HAL_I2C_BPS_100K,       // UINT16 i2crate;
    0x18                  // UINT8 i2caddress; this is the _REAL_ slave address.  0x40 ronlus
    //  The value written in phase 1 of a write cycle will be 0x42
    //  while the value just before the read will be 0x43
    0x18,                   // UINT8 i2cispaddress;
    0x02,                   // UINT8 pwrcfg;???
    TRUE,                   // BOOL rstactiveh;  // active high
    FALSE,                  // BOOL snrrst;

    CAM_MODE_IDLE,          // cammode

    FALSE,                  // Capture video mode
    FALSE,                  // Capturing single frame
    0,                      // Number of active buffers reserved by the upper layer
    0,                      // current frame buffer

    3,                      // UINT8 brightness;
    4,                      // UINT8 contrast;
    CAM_FORMAT_RGB565,      // Format
    CAM_NPIX_VGA,           // npixels(VGA, QVGA, QQVGA...)
    NULL,                   // Video Buffer.  First frame buffer for video or image buffer

    {0, NULL},              // CAM_REG_GROUP_T standby;
    // Should start with QQVGA, but for testing, start with VGA
    //    {sizeof(RG_InitPowerOnQqvga) / sizeof(CAM_REG_T), (CAM_REG_T*)&RG_InitPowerOnQqvga}, // CAM_REG_GROUP_T Initpoweron;
    {sizeof(RG_InitPowerOnVga) / sizeof(CAM_REG_T), (CAM_REG_T*)&RG_InitPowerOnVga}, // CAM_REG_GROUP_T Initpoweron;

    {0, NULL},              // CAM_REG_GROUP_T sifpwronseq;
    {0, NULL},              // CAM_REG_GROUP_T sifstdbyseq;
    {0, NULL},              // CAM_REG_GROUP_T dsif; initial value
    NULL                    // IRQ Callback
};

// ############
// Private
// ############

// These are the commands specific to the SCCB
#define PHASE3_WR_PH1   0x11000     // Start & Write    Slave ID
#define PHASE3_WR_PH2   0x1000      // Write            Slave Register Address
#define PHASE3_WR_PH3   0x1100      // Stop & Write     Slave Register Data

#define PHASE2_WR_PH1   0x11000     // Start & Write    Slave ID
#define PHASE2_WR_PH2   0x1100      // Stop & Write     Slave Register Address (to read)

#define PHASE2_RD_PH1   0x11000     // Start & Write    Slave ID
#define PHASE2_RD_PH2   0x111       // Stop & Read & NACK   Slave Register Read Data

PRIVATE VOID camerap_WriteOneReg( UINT8 Addr, UINT8 Data);
PRIVATE VOID camerap_Delay(UINT16 Wait_mS);
PRIVATE VOID camerap_WriteReg(CAM_REG_T RegWrite);
PRIVATE UINT8 camerap_ReadReg(UINT8 Addr);
PRIVATE VOID camerap_SendRgSeq(const CAM_REG_T *RegList, UINT16 Len);
PRIVATE VOID camerap_InteruptHandler(HAL_CAMERA_IRQ_CAUSE_T cause);
PRIVATE UINT8 camerap_GetId(VOID);

