/******************************************************************************
 ** Copyright (c) 
 ** File Name:		sensor_SP0838.c 										  *
 ** Author: 													  *
 ** DATE:															  *
 ** Description:   This file contains driver for sensor SP0838. 
 ** 														 
 ******************************************************************************

 ******************************************************************************
 ** 					   Edit History 									  *
 ** ------------------------------------------------------------------------- *
 ** DATE		   NAME 			DESCRIPTION 							  *
 ** 	  
 ******************************************************************************/

/**---------------------------------------------------------------------------*
 ** 						Dependencies									  *
 **---------------------------------------------------------------------------*/
#include "ms_customize_trc.h"
#include "sensor_cfg.h"
#include "sensor_drv.h"
#include "os_api.h"
#include "chip.h"
#include "dal_dcamera.h"

/**---------------------------------------------------------------------------*
 ** 						Compiler Flag									  *
 **---------------------------------------------------------------------------*/
#ifdef	 __cplusplus
	extern	 "C" 
	{
#endif
/**---------------------------------------------------------------------------*
 ** 					Extern Function Declaration 						  *
 **---------------------------------------------------------------------------*/
extern uint32 OS_TickDelay(uint32 ticks);
/*
extern void Sensor_SetMCLK(uint32 mclk);
*/
//extern ERR_I2C_E I2C_WriteCmdArrNoStop(uint8 addr, uint8 *pCmd, uint32 len, BOOLEAN ack_en);




 /**---------------------------------------------------------------------------*
 **                         Const variables                                   *
 **---------------------------------------------------------------------------*/
#define SP0838_I2C_ADDR_W    			0x30
#define SP0838_I2C_ADDR_R    			0x31
#define SP0838_I2C_ACK				       0x0
#define SENSOR_GAIN_SCALE		              16
LOCAL uint32			s_sensor_mclk		 = 0;
LOCAL uint32			Antiflicker		 = DCAMERA_FLICKER_50HZ;

//ECLK Drv
#define  SP0838_P0_0x30  0x00
//Filter en&dis
#define  SP0838_P0_0x56  0x70
#define  SP0838_P0_0x57  0x10  //filter outdoor
#define  SP0838_P0_0x58  0x10  //filter indoor
#define  SP0838_P0_0x59  0x10  //filter night
#define  SP0838_P0_0x5a  0x02  //smooth outdoor
#define  SP0838_P0_0x5b  0x02  //smooth indoor
#define  SP0838_P0_0x5c  0x20  //smooht night
//outdoor sharpness
#define  SP0838_P0_0x65  0x03
#define  SP0838_P0_0x66  0x01
#define  SP0838_P0_0x67  0x03
#define  SP0838_P0_0x68  0x46
//indoor sharpness
#define  SP0838_P0_0x6b  0x04
#define  SP0838_P0_0x6c  0x01
#define  SP0838_P0_0x6d  0x03
#define  SP0838_P0_0x6e  0x46
//night sharpness
#define  SP0838_P0_0x71  0x05
#define  SP0838_P0_0x72  0x01
#define  SP0838_P0_0x73  0x03
#define  SP0838_P0_0x74  0x46
//color
#define  SP0838_P0_0x7f   0xd7  //R 
#define  SP0838_P0_0x87  0xf8  //B
//satutation
#define  SP0838_P0_0xd8  0x48
#define  SP0838_P0_0xd9  0x48
#define  SP0838_P0_0xda  0x48
#define  SP0838_P0_0xdb  0x48
//AE target
#define  SP0838_P0_0xf7  0x78
#define  SP0838_P0_0xf8  0x63
#define  SP0838_P0_0xf9  0x68
#define  SP0838_P0_0xfa  0x53
//HEQ
#define  SP0838_P0_0xdd  0x70
#define  SP0838_P0_0xde  0x90
//AWB pre gain
#define  SP0838_P1_0x28  0x75
#define  SP0838_P1_0x29  0x4e

//VBLANK
#define  SP0838_P0_0x05  0x00
#define  SP0838_P0_0x06  0x00
//HBLANK
#define  SP0838_P0_0x09  0x01
#define  SP0838_P0_0x0a  0x76
/**---------------------------------------------------------------------------*
 ** 					Local Function Prototypes							  *
 **---------------------------------------------------------------------------*/
LOCAL uint32 set_SP0838_ae_enable(uint32 enable);
LOCAL uint32 set_hmirror_enable(uint32 enable);
LOCAL uint32 set_vmirror_enable(uint32 enable);
LOCAL uint32 set_preview_mode(uint32 preview_mode);
LOCAL uint32 SP0838_Identify(uint32 param);
LOCAL uint32 SP0838_Power_On(uint32 power_on);
LOCAL uint32 set_brightness(uint32 level);
LOCAL uint32 set_contrast(uint32 level);
LOCAL uint32 set_sharpness(uint32 level);
LOCAL uint32 set_saturation(uint32 level);
LOCAL uint32 set_image_effect(uint32 effect_type);

LOCAL uint32 read_ev_value(uint32 value);
LOCAL uint32 write_ev_value(uint32 exposure_value);
LOCAL uint32 read_gain_value(uint32 value);
LOCAL uint32 write_gain_value(uint32 gain_value);
LOCAL uint32 read_gain_scale(uint32 value);
LOCAL uint32 set_frame_rate(uint32 param);
LOCAL uint32 SP0838_set_work_mode(uint32 mode);

LOCAL uint32 set_SP0838_ev(uint32 level);
LOCAL uint32 set_SP0838_awb(uint32 mode);
LOCAL uint32 set_SP0838_anti_flicker(uint32 mode);
LOCAL uint32 set_SP0838_video_mode(uint32 mode);

LOCAL BOOLEAN gc_enter_effect = SCI_FALSE;

/**---------------------------------------------------------------------------*
 ** 						Local Variables 								 *
 **---------------------------------------------------------------------------*/
LOCAL void SP0838_Delayms (uint32 ms)	
{
	uint32 t1, t2;
	
	t1 = t2 = SCI_GetTickCount ();
	
	do{
		t2 = SCI_GetTickCount ();
	}while (t2 < (t1+ms));
}
__align(4) const SENSOR_REG_T SP0838_YUV_640X480[]=
{     
//SP0838 ini
{0xfd,0x00},//P0
{0x1B,0x02},
{0x27,0xe8},
{0x28,0x0B},
{0x31,0x10},
{0x32,0x00},
{0x22,0xc0},
{0x26,0x10}, 
{0x5f,0x11},//Bayer order
{0xfd,0x01},//P1
{0x25,0x1a},//Awb start
{0x26,0xfb},
{0x28,SP0838_P1_0x28},
{0x29,SP0838_P1_0x29},
{0xfd,0x00},
{0xe7,0x03},
{0xe7,0x00},
{0xfd,0x01},
{0x31,0x60},//64
{0x32,0x18},
{0x4d,0xdc},
{0x4e,0x53},
{0x41,0x8c},
{0x42,0x57},
{0x55,0xff},
{0x56,0x00},
{0x59,0x82},
{0x5a,0x00},
{0x5d,0xff},
{0x5e,0x6f},
{0x57,0xff},
{0x58,0x00},
{0x5b,0xff},
{0x5c,0xa8},
{0x5f,0x75},
{0x60,0x00},
{0x2d,0x00},
{0x2e,0x00},
{0x2f,0x00},
{0x30,0x00},
{0x33,0x00},
{0x34,0x00},
{0x37,0x00},
{0x38,0x00},//awb end
{0xfd,0x00},//P0
{0x33,0x6f},//LSC BPC EN
{0x51,0x3f},//BPC debug start
{0x52,0x09},
{0x53,0x00},
{0x54,0x00},
{0x55,0x10},//BPC debug end
{0x4f,0x08},//blueedge
{0x50,0x08},
{0x57,SP0838_P0_0x57},//Raw filter debut start
{0x58,SP0838_P0_0x58},
{0x59,SP0838_P0_0x59},
{0x56,SP0838_P0_0x56},
{0x5a,SP0838_P0_0x5a},
{0x5b,SP0838_P0_0x5b},
{0x5c,SP0838_P0_0x5c},//Raw filter debut end 
{0x65,SP0838_P0_0x65},//Sharpness debug start
{0x66,SP0838_P0_0x66},
{0x67,SP0838_P0_0x67},
{0x68,SP0838_P0_0x68},
{0x69,0x7f},
{0x6a,0x01},
{0x6b,SP0838_P0_0x6b},
{0x6c,SP0838_P0_0x6c},
{0x6d,SP0838_P0_0x6d},//Edge gain normal
{0x6e,SP0838_P0_0x6e},//Edge gain normal
{0x6f,0x7f},
{0x70,0x01},
{0x71,SP0838_P0_0x71}, //ֵ          
{0x72,SP0838_P0_0x72}, //ֵ        
{0x73,SP0838_P0_0x73}, //Եֵ    
{0x74,SP0838_P0_0x74}, //Եֵ    
{0x75,0x7f},              //ʹλ            
{0x76,0x01},//Sharpness debug end
{0xcb,0x07},//HEQ&Saturation debug start 
{0xcc,0x04},
{0xce,0xff},
{0xcf,0x10},
{0xd0,0x20},
{0xd1,0x00},
{0xd2,0x1c},
{0xd3,0x16},
{0xd4,0x00},
{0xd6,0x1c},
{0xd7,0x16},
{0xdd,SP0838_P0_0xdd},//Contrast
{0xde,SP0838_P0_0xde},//HEQ&Saturation debug end
{0x7f,SP0838_P0_0x7f},//Color Correction start
{0x80,0xbc},                        
{0x81,0xed},                        
{0x82,0xd7},                        
{0x83,0xd4},                        
{0x84,0xd6},                        
{0x85,0xff},                        
{0x86,0x89},                        
{0x87,SP0838_P0_0x87},                        
{0x88,0x3c},                        
{0x89,0x33},                        
{0x8a,0x0f},//Color Correction end  
{0x8b,0x0 },//gamma start
{0x8c,0x1a},             
{0x8d,0x29},             
{0x8e,0x41},             
{0x8f,0x62},             
{0x90,0x7c},             
{0x91,0x90},             
{0x92,0xa2},             
{0x93,0xaf},             
{0x94,0xbc},             
{0x95,0xc5},             
{0x96,0xcd},             
{0x97,0xd5},             
{0x98,0xdd},             
{0x99,0xe5},             
{0x9a,0xed},             
{0x9b,0xf5},             
{0xfd,0x01},//P1         
{0x8d,0xfd},             
{0x8e,0xff},//gamma end  
{0xfd,0x00},//P0
{0xca,0xcf},
{0xd8,SP0838_P0_0xd8},//UV outdoor
{0xd9,SP0838_P0_0xd9},//UV indoor 
{0xda,SP0838_P0_0xda},//UV dummy
{0xdb,SP0838_P0_0xdb},//UV lowlight
{0xb9,0x00},//Ygamma start
{0xba,0x04},
{0xbb,0x08},
{0xbc,0x10},
{0xbd,0x20},
{0xbe,0x30},
{0xbf,0x40},
{0xc0,0x50},
{0xc1,0x60},
{0xc2,0x70},
{0xc3,0x80},
{0xc4,0x90},
{0xc5,0xA0},
{0xc6,0xB0},
{0xc7,0xC0},
{0xc8,0xD0},
{0xc9,0xE0},
{0xfd,0x01},//P1
{0x89,0xf0},
{0x8a,0xff},//Ygamma end
{0xfd,0x00},//P0
{0xe8,0x30},//AEdebug start
{0xe9,0x30},
{0xea,0x40},//Alc Window sel
{0xf4,0x1b},//outdoor mode sel
{0xf5,0x80},
{0xf7,SP0838_P0_0xf7},//AE target
{0xf8,SP0838_P0_0xf8},
{0xf9,SP0838_P0_0xf9},//AE target 
{0xfa,SP0838_P0_0xfa},
{0xfd,0x01},//P1
{0x09,0x31},//AE Step 3.0
{0x0a,0x85},
{0x0b,0x0b},//AE Step 3.0
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},//P0
{0x05,0x00},  
{0x06,0x00},
{0x09,0x01},
{0x0a,0x76},
{0xf0,0x62},
{0xf1,0x00},
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},//P1
{0x00,0xba},
{0x0f,0x60},
{0x16,0x60},
{0x17,0xa2},
{0x18,0xaa},
{0x1b,0x60},
{0x1c,0xaa},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x78},
{0xc6,0x41},
{0xca,0x78},
{0xcb,0x0c},//AEdebug end
#if 1//caprure preview daylight 24M 50hz 20-8FPS maxgain:0x70	
{0xfd,0x00},
{0x05,SP0838_P0_0x05 },
{0x06,SP0838_P0_0x06 },
{0x09,SP0838_P0_0x09 },
{0x0a,SP0838_P0_0x0a },
{0xf0,0x62},
{0xf1,0x0 },
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},
{0x00,0xb2},
{0x0f,0x60},
{0x16,0x60},
{0x17,0xa2},
{0x18,0xaa},
{0x1b,0x60},
{0x1c,0xaa},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x70},
{0xc6,0x41},
{0xca,0x70},
{0xcb,0xc },
{0xfd,0x00},
#endif	
{0xfd,0x00},  //P0
{0x32,0x15},  //Auto_mode set
{0x34,0x66},  //Isp_mode set
{0x35,0x40},  //out format
{0xFF,0xFF},
};


LOCAL SENSOR_REG_TAB_INFO_T s_SP0838_resolution_Tab_YUV[]=
{
	// COMMON INIT
	{ADDR_AND_LEN_OF_ARRAY(SP0838_YUV_640X480), 640, 480, 24, SENSOR_IMAGE_FORMAT_YUV422},
	
	// YUV422 PREVIEW 1	
	{PNULL, 0, 640, 480,24, SENSOR_IMAGE_FORMAT_YUV422},
	{PNULL, 0, 0, 0, 0, 0},
	{PNULL, 0, 0, 0, 0, 0},
	{PNULL, 0, 0, 0, 0, 0},
	
	// YUV422 PREVIEW 2 
	{PNULL, 0, 0, 0, 0, 0},
	{PNULL, 0, 0, 0, 0, 0},
	{PNULL, 0, 0, 0, 0, 0},
	{PNULL, 0, 0, 0, 0, 0}

};

LOCAL SENSOR_IOCTL_FUNC_TAB_T s_SP0838_ioctl_func_tab = 
{
    // Internal 
    PNULL,
    SP0838_Power_On,///PNULL,
    PNULL,
    SP0838_Identify,

    PNULL,			// write register
    PNULL,			// read  register	
    PNULL,
    PNULL,

    // External
    set_SP0838_ae_enable,
    set_hmirror_enable,
    set_vmirror_enable,

    set_brightness,
    set_contrast,
    set_sharpness,
    set_saturation,

    set_preview_mode,	
    set_image_effect,

	PNULL,	//	SP0838_BeforeSnapshot,
    PNULL,		//SP0838_After_Snapshot,

    PNULL,

    read_ev_value,
    write_ev_value,
    read_gain_value,
    write_gain_value,
    read_gain_scale,
    set_frame_rate,	
    PNULL,
    PNULL,
    set_SP0838_awb,
    PNULL,
    PNULL,
    set_SP0838_ev,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    set_SP0838_anti_flicker,
    set_SP0838_video_mode,
    PNULL,
    PNULL
};

/**---------------------------------------------------------------------------*
 ** 						Global Variables								  *
 **---------------------------------------------------------------------------*/
 PUBLIC SENSOR_INFO_T g_SP0838_yuv_info =
{
	SP0838_I2C_ADDR_W,				// salve i2c write address
	SP0838_I2C_ADDR_R, 				// salve i2c read address
	
	0,								// bit0: 0: i2c register value is 8 bit, 1: i2c register value is 16 bit
									// bit2: 0: i2c register addr  is 8 bit, 1: i2c register addr  is 16 bit
									// other bit: reseved
	SENSOR_HW_SIGNAL_PCLK_P|\
	SENSOR_HW_SIGNAL_VSYNC_P|\
	SENSOR_HW_SIGNAL_HSYNC_P,		// bit0: 0:negative; 1:positive -> polarily of pixel clock
									// bit2: 0:negative; 1:positive -> polarily of horizontal synchronization signal
									// bit4: 0:negative; 1:positive -> polarily of vertical synchronization signal
									// other bit: reseved											
											
	// preview mode
	SENSOR_ENVIROMENT_NORMAL|\
	SENSOR_ENVIROMENT_NIGHT|\
	SENSOR_ENVIROMENT_SUNNY,		

	// image effect
	SENSOR_IMAGE_EFFECT_NORMAL|\
	SENSOR_IMAGE_EFFECT_BLACKWHITE|\
	SENSOR_IMAGE_EFFECT_RED|\
	SENSOR_IMAGE_EFFECT_GREEN|\
	SENSOR_IMAGE_EFFECT_BLUE|\
	SENSOR_IMAGE_EFFECT_YELLOW|\
	SENSOR_IMAGE_EFFECT_NEGATIVE|\
	SENSOR_IMAGE_EFFECT_CANVAS,
	// while balance mode
	0,
		
	7,								// bit[0:7]: count of step in brightness, contrast, sharpness, saturation
									// bit[8:31] reseved
	
	SENSOR_LOW_PULSE_RESET,			// reset pulse level
	100,								// reset pulse width(ms)
	
	SENSOR_HIGH_LEVEL_PWDN,			// 1: high level valid; 0: low level valid
	
	2,								// count of identify code
	0x02, 0x27,						// supply two code to identify sensor.
	0x02, 0x27,						// for Example: index = 0-> Device id, index = 1 -> version id	
									
	SENSOR_AVDD_2800MV,				// voltage of avdd	

	640,							// max width of source image
	480,							// max height of source image
	"SP0838",						// name of sensor												

	SENSOR_IMAGE_FORMAT_YUV422,		// define in SENSOR_IMAGE_FORMAT_E enum,
									// if set to SENSOR_IMAGE_FORMAT_MAX here, image format depent on SENSOR_REG_TAB_INFO_T
	SENSOR_IMAGE_PATTERN_YUV422_YUYV,	// pattern of input image form sensor;			

	s_SP0838_resolution_Tab_YUV,	// point to resolution table information structure
	&s_SP0838_ioctl_func_tab,		// point to ioctl function table
			
	PNULL,							// information and table about Rawrgb sensor
	PNULL,							// extend information about sensor	
       SENSOR_AVDD_2800MV,                     // iovdd
	SENSOR_AVDD_1800MV,                      // dvdd
	3,
       3,
	0,
	2,
	0,
	0,
	0,
	0,
       0,
};
/**---------------------------------------------------------------------------*
 ** 							Function  Definitions
 **---------------------------------------------------------------------------*/
LOCAL void SP0838_WriteReg( uint8  subaddr, uint8 data )
{
	
	#ifndef	_USE_DSP_I2C_
		//uint8 cmd[2];
		//cmd[0]	=	subaddr;
		//cmd[1]	=	data;		
		//I2C_WriteCmdArr(SP0838_I2C_ADDR_W, cmd, 2, SCI_TRUE);
		Sensor_WriteReg(subaddr, data);
	#else
		DSENSOR_IICWrite((uint16)subaddr, (uint16)data);
	#endif

	//SCI_TRACE_LOW:"SENSOR: SP0838_WriteReg reg/value(%x,%x) !!"
	SCI_TRACE_ID(TRACE_TOOL_CONVERT,SENSOR_SP0838_469_112_2_18_0_32_40_1375,(uint8*)"dd", subaddr, data);

}

LOCAL uint8 SP0838_ReadReg( uint8  subaddr)
{
	uint8 value = 0;
	
	#ifndef	_USE_DSP_I2C_
	//I2C_WriteCmdArrNoStop(SP0838_I2C_ADDR_W, &subaddr, 1,SCI_TRUE);
	//I2C_ReadCmd(SP0838_I2C_ADDR_R, &value, SCI_TRUE);
	value =Sensor_ReadReg( subaddr);
	#else
		value = (uint16)DSENSOR_IICRead((uint16)subaddr);
	#endif

    //SCI_TRACE_LOW:"SENSOR: SP0838_ReadReg reg/value(%x,%x) !!"
    SCI_TRACE_ID(TRACE_TOOL_CONVERT,SENSOR_SP0838_485_112_2_18_0_32_40_1376,(uint8*)"dd", subaddr, value);
    
	return value;
}


#if 1
LOCAL uint32 SP0838_Power_On(uint32 power_on)
{
	SENSOR_AVDD_VAL_E		dvdd_val=g_SP0838_yuv_info.dvdd_val;
	SENSOR_AVDD_VAL_E		avdd_val=g_SP0838_yuv_info.avdd_val;
	SENSOR_AVDD_VAL_E		iovdd_val=g_SP0838_yuv_info.iovdd_val;  
	BOOLEAN 				power_down=g_SP0838_yuv_info.power_down_level;	    
	BOOLEAN 				reset_level=g_SP0838_yuv_info.reset_pulse_level;
	uint32 				reset_width=g_SP0838_yuv_info.reset_pulse_width;	    

	if(SCI_TRUE==power_on)
	{
		Sensor_SetVoltage(dvdd_val, avdd_val, iovdd_val);
		SCI_Sleep(10);        
#if 1//def DOUBLE_CAMERA_SUPPORT
		if(SENSOR_MAIN == Sensor_GetCurId()) //(Sensor_GetFrontCameraState())
		{
			GPIO_SetFrontSensorPwdn(power_down);
			GPIO_SetSensorPwdn(!power_down);						
		}
		else
		{
			GPIO_SetSensorPwdn(power_down);
			GPIO_SetFrontSensorPwdn(!power_down);			
		}
#else      
		GPIO_SetSensorPwdn(!power_down);
#endif

		//Open Mclk in default frequency
		Sensor_SetMCLK(24);   
		SCI_Sleep(50);  

		// reset pin on SP0838
		Sensor_SetResetLevel(!reset_level);
		SCI_Sleep(50);
		Sensor_SetResetLevel(reset_level);
		SCI_Sleep(reset_width);
		Sensor_SetResetLevel(!reset_level);
		SCI_Sleep(50);

	}
	else
	{
#if 1//def DOUBLE_CAMERA_SUPPORT
		GPIO_SetFrontSensorPwdn(!power_down);
		GPIO_SetSensorPwdn(!power_down);
#else	
		GPIO_SetSensorPwdn(!power_down);
#endif

		Sensor_SetMCLK(SENSOR_DISABLE_MCLK);           

		Sensor_SetVoltage(SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED);   
		SCI_Sleep(20);
	}


	return SCI_SUCCESS;
}
#endif


LOCAL uint32 SP0838_Identify(uint32 param)
{
    uint16  iden_reg_val = 0;
    uint32  ret = SENSOR_OP_ERR;

    iden_reg_val = SP0838_ReadReg(0x02);
    if(iden_reg_val == 0x27)
    {
            ret = SENSOR_OP_SUCCESS;
    }
    
    //SCI_TRACE_LOW:"SP0838_Identify: ret = %d"
    SCI_TRACE_ID(TRACE_TOOL_CONVERT,SENSOR_SP0838_544_112_2_18_0_32_40_1380,(uint8*)"d", ret);
    return ret;

}

LOCAL uint32 set_SP0838_ae_enable(uint32 enable)
{
   #define AE_ENABLE  (0x32)
    unsigned char ae_value;
		SP0838_WriteReg(0xfd,0x00);
        ae_value=SP0838_ReadReg(AE_ENABLE);

        if(0x00==enable)
        {
            ae_value &= 0xf0;
            SP0838_WriteReg(AE_ENABLE,ae_value);
        }
        else if(0x01==enable)
        {
            ae_value|=0x05;
            SP0838_WriteReg(AE_ENABLE,ae_value);
        }

	return 0;
}


LOCAL uint32 set_hmirror_enable(uint32 enable)
{
#ifdef DOUBLE_CAMERA_SUPPORT
uint32 i;
BOOLEAN is_page0 = FALSE;
for(i = 0; i < s_SP0838_resolution_Tab_YUV->reg_count; i++)
{
if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0xfd)
{
if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x00)
is_page0 = TRUE;
else if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x01)
is_page0 = FALSE;
}
if ((s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0x31) && is_page0)
{
if(enable)
{
s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value |= 0x20;
}
else
{
s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value &= ~0x20;
}
break;
}
}
return SCI_SUCCESS; 
#endif
}


LOCAL uint32 set_vmirror_enable(uint32 enable)
{
#ifdef DOUBLE_CAMERA_SUPPORT
uint32 i;
BOOLEAN is_page0 = FALSE;
for(i = 0; i < s_SP0838_resolution_Tab_YUV->reg_count; i++)
{
if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0xfd)
{
if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x00)
is_page0 = TRUE;
else if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x01)
is_page0 = FALSE;
}
if ((s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0x31) && is_page0)
{
if(enable)
{
s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value |= 0x40;
}
else
{
s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value &= ~0x40;
}
break;
}
}
return SCI_SUCCESS; 
#endif
}
/******************************************************************************/
// Description: set brightness 
// Global resource dependence: 
// Author:
// Note:
//		level  must smaller than 8
/******************************************************************************/
__align(4) const SENSOR_REG_T SP0838_brightness_tab[][3]=
{
	{
		{0xfd,0x00},
		{0xdc,0xd0},//level -3
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0xe0},//level -2
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0xf0},//level -1
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0x00},//level 0
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0x10},//level +1
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0x20},//level +2
		{0xff,0xff},
	},
    {		
		{0xfd,0x00},
		{0xdc,0x30},//level +3
		{0xff,0xff},
	},

};


LOCAL uint32 set_brightness(uint32 level)
{
	uint16 i;
	SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0838_brightness_tab[level];

	SCI_ASSERT(level < 7);/*assert verified*/
	SCI_ASSERT(PNULL != sensor_reg_ptr);/*assert verified*/
	
	for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) && (0xFF != sensor_reg_ptr[i].reg_value); i++)
	{
		SP0838_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
	}
	OS_TickDelay(100); 
	//SCI_TRACE_LOW:"set_brightness: level = %d"
	SCI_TRACE_ID(TRACE_TOOL_CONVERT,SENSOR_SP0838_644_112_2_18_0_32_41_1384,(uint8*)"d", level);
	
	return 0;
}

__align(4) const SENSOR_REG_T SP0838_ev_tab[][6]=
{   
    {
		{0xfd,0x00},   
		{0xf7,SP0838_P0_0xf7-0x18},//level -3
		{0xf8,SP0838_P0_0xf8-0x18},	
		{0xf9,SP0838_P0_0xf9-0x18},
		{0xfa,SP0838_P0_0xfa-0x18},		
		{0xff,0xff},     
	},
    {
		{0xfd,0x00},   
		{0xf7,SP0838_P0_0xf7-0x10},//level -2
		{0xf8,SP0838_P0_0xf8-0x10},	
		{0xf9,SP0838_P0_0xf9-0x10},
		{0xff,0xff},     
	},
    {
		{0xfd,0x00},   
		{0xf7,SP0838_P0_0xf7-0x08},//level -1
		{0xf8,SP0838_P0_0xf8-0x08},	
		{0xf9,SP0838_P0_0xf9-0x08},
		{0xfa,SP0838_P0_0xfa-0x08},		
		{0xff,0xff},     
	},
    {
		{0xfd,0x00},   
		{0xf7,SP0838_P0_0xf7},//level 0
		{0xf8,SP0838_P0_0xf8},	
		{0xf9,SP0838_P0_0xf9},
		{0xfa,SP0838_P0_0xfa},		
		{0xff,0xff},    
	},
    {
		{0xfd,0x00},   
		{0xf7,SP0838_P0_0xf7+0x08},//level +1
		{0xf8,SP0838_P0_0xf8+0x08},	
		{0xf9,SP0838_P0_0xf9+0x08},
		{0xfa,SP0838_P0_0xfa+0x08},		
		{0xff,0xff},  
	},
    {
		{0xfd,0x00},   
		{0xf7,SP0838_P0_0xf7+0x10},//level +2
		{0xf8,SP0838_P0_0xf8+0x10},	
		{0xf9,SP0838_P0_0xf9+0x10},
		{0xfa,SP0838_P0_0xfa+0x10},		
		{0xff,0xff},     
	},
    {
		{0xfd,0x00},   
		{0xf7,SP0838_P0_0xf7+0x18},//level +3
		{0xf8,SP0838_P0_0xf8+0x18},	
		{0xf9,SP0838_P0_0xf9+0x18},
		{0xfa,SP0838_P0_0xfa+0x18},		
		{0xff,0xff},    
	},
};


LOCAL uint32 set_SP0838_ev(uint32 level)
{
    uint16 i; 
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0838_ev_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);/*assert verified*/
    SCI_ASSERT(level < 7);/*assert verified*/
 
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) ||(0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        SP0838_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }

    
    //SCI_TRACE_LOW:"SENSOR: set_ev: level = %d"
    SCI_TRACE_ID(TRACE_TOOL_CONVERT,SENSOR_SP0838_724_112_2_18_0_32_41_1385,(uint8*)"d", level);

    return 0;
}

/******************************************************************************/
// Description: anti 50/60 hz banding flicker
// Global resource dependence: 
// Author:
// Note:
//		level  must smaller than 8
/******************************************************************************/
LOCAL uint32 set_SP0838_anti_flicker(uint32 param )
{ 
 #if 1
     switch(param)
    {
        case DCAMERA_FLICKER_50HZ:			
		Antiflicker = DCAMERA_FLICKER_50HZ;
		SCI_TRACE_LOW( " set_SP0838_anti_flicker  50hz \r\n" );
		break;
            
        case DCAMERA_FLICKER_60HZ:
		Antiflicker = DCAMERA_FLICKER_60HZ;
		SCI_TRACE_LOW( " set_SP0838_anti_flicker  60hz \r\n" );
		break;
            
        default:
            break;
    }
    
    SP0838_Delayms(100); 
#endif
    return 0;
}

/******************************************************************************/
// Description: set video mode
// Global resource dependence: 
// Author:
// Note:
//		 
/******************************************************************************/
__align(4) const SENSOR_REG_T SP0838_video_mode_nand_tab[][9]=
{
#if 1
    {		
		{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xff,0xff},
	},

#else
    {
        {0x01,0x6a},{0x02,0x32},{0x0f,0x20},{0xe2,0x00},{0xe3,0x96},{0xe8,0x04},{0xe9,0x1a}, {0xec,0x20},{0xff, 0xff} 
    },    
    //vodeo mode
    {
        {0x01,0x6a},{0x02,0x70},{0x0f,0x00},{0xe2,0x00},{0xe3,0x96},{0xe8,0x02},{0xe9,0x58}, {0xec,0x20},{0xff, 0xff}      
    }
#endif
};
__align(4) const SENSOR_REG_T SP0838_video_mode_nor_tab_50hz[][40]=
{
	//caprure preview daylight 24M 50hz 20-8FPS maxgain:0x70
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x62},
{0xf1,0x0 },
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},
{0x00,0xb2},
{0x0f,0x60},
{0x16,0x60},
{0x17,0xa2},
{0x18,0xaa},
{0x1b,0x60},
{0x1c,0xaa},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x70},
{0xc6,0x41},
{0xca,0x70},
{0xcb,0xc },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},	//־	
	},    
    //Video record daylight 24M 50hz fix 20FPS maxgain:0x78
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x62},
{0xf1,0x0 },
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},
{0x00,0xa9},
{0x0f,0x60},
{0x16,0x60},
{0x17,0x91},
{0x18,0x99},
{0x1b,0x60},
{0x1c,0x99},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x78},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x41},
{0xc6,0x41},
{0xca,0x78},
{0xcb,0x5 },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},//־	
    },
   // UPCC  mode  daylight 24M 50hz fix 20FPS maxgain:0x78
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x62},
{0xf1,0x0 },
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},
{0x00,0xa9},
{0x0f,0x60},
{0x16,0x60},
{0x17,0x91},
{0x18,0x99},
{0x1b,0x60},
{0x1c,0x99},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x78},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x41},
{0xc6,0x41},
{0xca,0x78},
{0xcb,0x5 },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},//־	
    }  
};
__align(4) const SENSOR_REG_T SP0838_video_mode_nor_tab_60hz[][40]=
{
#if 0
    {		
		{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},
		{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xff,0xff},
	},

#else
	//caprure preview daylight 24M 60hz 20-8FPS maxgain:0x70
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x51},
{0xf1,0x0 },
{0xf2,0x5c},
{0xf5,0x75},
{0xfd,0x01},
{0x00,0xb4},
{0x0f,0x5d},
{0x16,0x5d},
{0x17,0xa4},
{0x18,0xac},
{0x1b,0x5d},
{0x1c,0xac},
{0xb4,0x21},
{0xb5,0x3d},
{0xb6,0x4d},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x41},
{0xc6,0x41},
{0xca,0x70},
{0xcb,0xf },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},//־  
	},    
    //Video record daylight 24M 60hz fix 20FPS maxgain:0x78
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x51},
{0xf1,0x0 },
{0xf2,0x5c},
{0xf5,0x75},
{0xfd,0x01},
{0x00,0xa9},
{0x0f,0x5d},
{0x16,0x5d},
{0x17,0x91},
{0x18,0x99},
{0x1b,0x5d},
{0x1c,0x99},
{0xb4,0x21},
{0xb5,0x3d},
{0xb6,0x4d},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x78},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x41},
{0xc6,0x41},
{0xca,0x78},
{0xcb,0x6 },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},//־     
    },
   // UPCC  mode daylight 24M 60hz fix 20FPS maxgain:0x78
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x51},
{0xf1,0x0 },
{0xf2,0x5c},
{0xf5,0x75},
{0xfd,0x01},
{0x00,0xa9},
{0x0f,0x5d},
{0x16,0x5d},
{0x17,0x91},
{0x18,0x99},
{0x1b,0x5d},
{0x1c,0x99},
{0xb4,0x21},
{0xb5,0x3d},
{0xb6,0x4d},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x78},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x41},
{0xc6,0x41},
{0xca,0x78},
{0xcb,0x6 },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},//־    
    }  
#endif	
};

LOCAL uint32 set_SP0838_video_mode(uint32 mode)
{
#if 1
    uint8 data=0x00;
    uint16 i;
    SENSOR_REG_T* sensor_reg_ptr = PNULL;
    uint8 tempregval = 0;
    SCI_TRACE_LOW(" xg:set_SP0838_video_mode ,%d,%d\r\n",mode,Antiflicker);
    SCI_ASSERT(mode <=DCAMERA_MODE_MAX);

	if(Antiflicker ==DCAMERA_FLICKER_50HZ )
	{
	    if(CHIP_DetectMemType())
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0838_video_mode_nor_tab_50hz[mode];

	    else
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0838_video_mode_nor_tab_50hz[mode];
	}
	else
	{
	    if(CHIP_DetectMemType())
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0838_video_mode_nor_tab_60hz[mode];

	    else
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0838_video_mode_nor_tab_60hz[mode];

	}

    SCI_ASSERT(PNULL != sensor_reg_ptr);

    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value); i++)
    {
    	SP0838_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
#endif	
    return 0;
}


/******************************************************************************/
// Description: set wb mode 
// Global resource dependence: 
// Author:
// Note:
//		
/******************************************************************************/

/*
// enum: preview while balance mode
typedef enum
{
	DCAMERA_WB_MODE_AUTO = 0x00,			//Զ
	DCAMERA_WB_MODE_INCANDESCENCE,		//׳
	DCAMERA_WB_MODE_U30,				//ùԴ
	DCAMERA_WB_MODE_CWF,				//ӫ
	DCAMERA_WB_MODE_FLUORESCENT,		//չ
	DCAMERA_WB_MODE_SUN,				//
	DCAMERA_WB_MODE_CLOUD,				//
	DCAMERA_WB_MODE_MAX
}DCAMERA_PARAM_WB_MODE_E;
*/
__align(4) const SENSOR_REG_T SP0838_awb_tab[][7]=
	{
	   	//AUTO  // AUTO 3000K~7000K 
	   	{
			{0xfd,0x01},                      
			{0x28,SP0838_P1_0x28},		                  
			{0x29,SP0838_P1_0x29},                      
			{0xfd,0x00}, 
			{0x32,0x15},   
			{0xfd,0x00},  
		{0xff,0xff} 
		},    
		//INCANDESCENCE:  //2800K~3000K  
		{
			{0xfd,0x00}, 
			{0x32,0x05},                 
			{0xfd,0x01},                 
			{0x28,0x41},		             
			{0x29,0x71},		             
			{0xfd,0x00}, 
		{0xff,0xff} 
		},
		//U30 //3000K
		{
			{0xfd,0x00},
			{0x32,0x05},       
			{0xfd,0x01},       
			{0x28,0x41},	     
			{0x29,0x71},	     
			{0xfd,0x00},     
		{0xff,0xff} 
		},  
		//CWF  //4150K
		{
			{0xfd,0x00},
			{0x32,0x05},       
			{0xfd,0x01},       
			{0x28,0x57},	     
			{0x29,0x66},	     
			{0xfd,0x00},     
		{0xff,0xff} 
		},    
		//FLUORESCENT://4200K~5000K   
		{
			{0xfd,0x00},  
			{0x32,0x05},                  
			{0xfd,0x01},                  
			{0x28,0x5a},		              
			{0x29,0x62},		              
			{0xfd,0x00},
		    {0xff,0xff} 
		},
		//SUN: //DAYLIGHT//6500K   
		{
			{0xfd,0x00}, 
			{0x32,0x05},           
			{0xfd,0x01},           
			{0x28,0x6b},	         
			{0x29,0x48},	         
			{0xfd,0x00},  
		    {0xff,0xff} 
		},
                //CLOUD: //7000K 
		{
			{0xfd,0x00}, 
			{0x32,0x05},          
			{0xfd,0x01},          
			{0x28,0x71},	        
			{0x29,0x41},	        
			{0xfd,0x00},
		    {0xff,0xff} 
		},
};
	
	LOCAL uint32 set_SP0838_awb(uint32 mode)
	{
		uint16 i;
		
		SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0838_awb_tab[mode];
	
		SCI_ASSERT(mode < 7);/*assert verified*/
		SCI_ASSERT(PNULL != sensor_reg_ptr);/*assert verified*/
		// hill 0408
		for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) && (0xFF != sensor_reg_ptr[i].reg_value); i++)
		{
			SP0838_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
		}
		OS_TickDelay(100); 
		//SCI_TRACE_LOW:"SENSOR: set_awb_mode: mode = %d"
		SCI_TRACE_ID(TRACE_TOOL_CONVERT,SENSOR_SP0838_1119_112_2_18_0_32_42_1386,(uint8*)"d", mode);
		
		return 0;
}

__align(4) const SENSOR_REG_T SP0838_contrast_tab[][4]=
{
	{
		{0xfd, 0x00},
		{0xdd,  SP0838_P0_0xdd-0x30},	//level -3
		{0xde,  SP0838_P0_0xde-0x30},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  SP0838_P0_0xdd-0x20},	//level -2
		{0xde,  SP0838_P0_0xde-0x20},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd, SP0838_P0_0xdd-0x10},	//level -1
		{0xde, SP0838_P0_0xde-0x10},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd, SP0838_P0_0xdd},	//level 0
		{0xde, SP0838_P0_0xde},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  SP0838_P0_0xdd+0x10},	//level +1
		{0xde,  SP0838_P0_0xde+0x10},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  SP0838_P0_0xdd+0x20},	//level +2
		{0xde,  SP0838_P0_0xde+0x20},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  SP0838_P0_0xdd+0x30},	//level +3
		{0xde,  SP0838_P0_0xde+0x30},
		{0xff, 0xff}
	}, 
};

LOCAL uint32 set_contrast(uint32 level)
{
    uint16 i;
    SENSOR_REG_T* sensor_reg_ptr;

    sensor_reg_ptr = (SENSOR_REG_T*)SP0838_contrast_tab[level];

    SCI_ASSERT(level < 7 );/*assert verified*/
    SCI_ASSERT(PNULL != sensor_reg_ptr);/*assert verified*/

    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) && (0xFF != sensor_reg_ptr[i].reg_value); i++)
    {
        SP0838_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
    OS_TickDelay(20);
    //SCI_TRACE_LOW:"set_contrast: level = %d"
    SCI_TRACE_ID(TRACE_TOOL_CONVERT,SENSOR_SP0838_1193_112_2_18_0_32_42_1387,(uint8*)"d", level);
    return 0;
}


LOCAL uint32 set_sharpness(uint32 level)
{
	
	return 0;
}


LOCAL uint32 set_saturation(uint32 level)
{

	
	return 0;
}

/******************************************************************************/
// Description: set brightness 
// Global resource dependence: 
// Author:
// Note:
//		level  must smaller than 8
/******************************************************************************/

LOCAL uint32 set_preview_mode(uint32 preview_mode)
{
#if 1
	switch (preview_mode)
	{
		case DCAMERA_ENVIRONMENT_NORMAL: 
		case DCAMERA_ENVIRONMENT_SUNNY:
		{
			if(Antiflicker== DCAMERA_FLICKER_50HZ)
			{
			SCI_TRACE_LOW("normal mode 50hz\r\n");
			//caprure preview daylight 24M 50hz 20-8FPS maxgain:0x70   
			SP0838_WriteReg(0xfd,0x00);
			SP0838_WriteReg(0x05,0x0 );
			SP0838_WriteReg(0x06,0x0 );
			SP0838_WriteReg(0x09,0x1 );
			SP0838_WriteReg(0x0a,0x76);
			SP0838_WriteReg(0xf0,0x62);
			SP0838_WriteReg(0xf1,0x0 );
			SP0838_WriteReg(0xf2,0x5f);
			SP0838_WriteReg(0xf5,0x78);
			SP0838_WriteReg(0xfd,0x01);
			SP0838_WriteReg(0x00,0xb2);
			SP0838_WriteReg(0x0f,0x60);
			SP0838_WriteReg(0x16,0x60);
			SP0838_WriteReg(0x17,0xa2);
			SP0838_WriteReg(0x18,0xaa);
			SP0838_WriteReg(0x1b,0x60);
			SP0838_WriteReg(0x1c,0xaa);
			SP0838_WriteReg(0xb4,0x20);
			SP0838_WriteReg(0xb5,0x3a);
			SP0838_WriteReg(0xb6,0x5e);
			SP0838_WriteReg(0xb9,0x40);
			SP0838_WriteReg(0xba,0x4f);
			SP0838_WriteReg(0xbb,0x47);
			SP0838_WriteReg(0xbc,0x45);
			SP0838_WriteReg(0xbd,0x43);
			SP0838_WriteReg(0xbe,0x42);
			SP0838_WriteReg(0xbf,0x42);
			SP0838_WriteReg(0xc0,0x42);
			SP0838_WriteReg(0xc1,0x41);
			SP0838_WriteReg(0xc2,0x41);
			SP0838_WriteReg(0xc3,0x41);
			SP0838_WriteReg(0xc4,0x41);
			SP0838_WriteReg(0xc5,0x70);
			SP0838_WriteReg(0xc6,0x41);
			SP0838_WriteReg(0xca,0x70);
			SP0838_WriteReg(0xcb,0xc );
			SP0838_WriteReg(0x14,0x20);
			SP0838_WriteReg(0x15,0x0f);
			SP0838_WriteReg(0xfd,0x00);
			}
			else	
			{
			SCI_TRACE_LOW("normal mode 60hz\r\n");
			//caprure preview daylight 24M 60hz 20-8FPS maxgain:0x70	       
			SP0838_WriteReg(0xfd,0x00);
			SP0838_WriteReg(0x05,0x0 );
			SP0838_WriteReg(0x06,0x0 );
			SP0838_WriteReg(0x09,0x1 );
			SP0838_WriteReg(0x0a,0x76);
			SP0838_WriteReg(0xf0,0x51);
			SP0838_WriteReg(0xf1,0x0 );
			SP0838_WriteReg(0xf2,0x5c);
			SP0838_WriteReg(0xf5,0x75);
			SP0838_WriteReg(0xfd,0x01);
			SP0838_WriteReg(0x00,0xb4);
			SP0838_WriteReg(0x0f,0x5d);
			SP0838_WriteReg(0x16,0x5d);
			SP0838_WriteReg(0x17,0xa4);
			SP0838_WriteReg(0x18,0xac);
			SP0838_WriteReg(0x1b,0x5d);
			SP0838_WriteReg(0x1c,0xac);
			SP0838_WriteReg(0xb4,0x21);
			SP0838_WriteReg(0xb5,0x3d);
			SP0838_WriteReg(0xb6,0x4d);
			SP0838_WriteReg(0xb9,0x40);
			SP0838_WriteReg(0xba,0x4f);
			SP0838_WriteReg(0xbb,0x47);
			SP0838_WriteReg(0xbc,0x45);
			SP0838_WriteReg(0xbd,0x43);
			SP0838_WriteReg(0xbe,0x42);
			SP0838_WriteReg(0xbf,0x42);
			SP0838_WriteReg(0xc0,0x42);
			SP0838_WriteReg(0xc1,0x41);
			SP0838_WriteReg(0xc2,0x41);
			SP0838_WriteReg(0xc3,0x41);
			SP0838_WriteReg(0xc4,0x41);
			SP0838_WriteReg(0xc5,0x41);
			SP0838_WriteReg(0xc6,0x41);
			SP0838_WriteReg(0xca,0x70);
			SP0838_WriteReg(0xcb,0xf );
			SP0838_WriteReg(0x14,0x20);
			SP0838_WriteReg(0x15,0x0f);
			SP0838_WriteReg(0xfd,0x00);
			}	
			break;
		}
		case DCAMERA_ENVIRONMENT_NIGHT:
		{

			if(Antiflicker== DCAMERA_FLICKER_50HZ)
			{
			SCI_TRACE_LOW("night mode 50hz\r\n");
			//caprure preview night 24M 50hz 20-6FPS maxgain:0x78	       
			SP0838_WriteReg(0xfd,0x00);
			SP0838_WriteReg(0x05,0x0 );
			SP0838_WriteReg(0x06,0x0 );
			SP0838_WriteReg(0x09,0x1 );
			SP0838_WriteReg(0x0a,0x76);
			SP0838_WriteReg(0xf0,0x62);
			SP0838_WriteReg(0xf1,0x0 );
			SP0838_WriteReg(0xf2,0x5f);
			SP0838_WriteReg(0xf5,0x78);
			SP0838_WriteReg(0xfd,0x01);
			SP0838_WriteReg(0x00,0xc0);
			SP0838_WriteReg(0x0f,0x60);
			SP0838_WriteReg(0x16,0x60);
			SP0838_WriteReg(0x17,0xa8);
			SP0838_WriteReg(0x18,0xb0);
			SP0838_WriteReg(0x1b,0x60);
			SP0838_WriteReg(0x1c,0xb0);
			SP0838_WriteReg(0xb4,0x20);
			SP0838_WriteReg(0xb5,0x3a);
			SP0838_WriteReg(0xb6,0x5e);
			SP0838_WriteReg(0xb9,0x40);
			SP0838_WriteReg(0xba,0x4f);
			SP0838_WriteReg(0xbb,0x47);
			SP0838_WriteReg(0xbc,0x45);
			SP0838_WriteReg(0xbd,0x43);
			SP0838_WriteReg(0xbe,0x42);
			SP0838_WriteReg(0xbf,0x42);
			SP0838_WriteReg(0xc0,0x42);
			SP0838_WriteReg(0xc1,0x41);
			SP0838_WriteReg(0xc2,0x41);
			SP0838_WriteReg(0xc3,0x41);
			SP0838_WriteReg(0xc4,0x41);
			SP0838_WriteReg(0xc5,0x41);
			SP0838_WriteReg(0xc6,0x41);
			SP0838_WriteReg(0xca,0x78);
			SP0838_WriteReg(0xcb,0x10);
			SP0838_WriteReg(0x14,0x20);
			SP0838_WriteReg(0x15,0x1f);
			SP0838_WriteReg(0xfd,0x00);
			}
			else
			{
			SCI_TRACE_LOW("night mode 60hz\r\n");
			//caprure preview night 24M 60hz 20-6FPS maxgain:0x78
			SP0838_WriteReg(0xfd,0x00);
			SP0838_WriteReg(0x05,0x0 );
			SP0838_WriteReg(0x06,0x0 );
			SP0838_WriteReg(0x09,0x1 );
			SP0838_WriteReg(0x0a,0x76);
			SP0838_WriteReg(0xf0,0x51);
			SP0838_WriteReg(0xf1,0x0 );
			SP0838_WriteReg(0xf2,0x5c);
			SP0838_WriteReg(0xf5,0x75);
			SP0838_WriteReg(0xfd,0x01);
			SP0838_WriteReg(0x00,0xc1);
			SP0838_WriteReg(0x0f,0x5d);
			SP0838_WriteReg(0x16,0x5d);
			SP0838_WriteReg(0x17,0xa9);
			SP0838_WriteReg(0x18,0xb1);
			SP0838_WriteReg(0x1b,0x5d);
			SP0838_WriteReg(0x1c,0xb1);
			SP0838_WriteReg(0xb4,0x21);
			SP0838_WriteReg(0xb5,0x3d);
			SP0838_WriteReg(0xb6,0x4d);
			SP0838_WriteReg(0xb9,0x40);
			SP0838_WriteReg(0xba,0x4f);
			SP0838_WriteReg(0xbb,0x47);
			SP0838_WriteReg(0xbc,0x45);
			SP0838_WriteReg(0xbd,0x43);
			SP0838_WriteReg(0xbe,0x42);
			SP0838_WriteReg(0xbf,0x42);
			SP0838_WriteReg(0xc0,0x42);
			SP0838_WriteReg(0xc1,0x41);
			SP0838_WriteReg(0xc2,0x41);
			SP0838_WriteReg(0xc3,0x41);
			SP0838_WriteReg(0xc4,0x41);
			SP0838_WriteReg(0xc5,0x41);
			SP0838_WriteReg(0xc6,0x41);
			SP0838_WriteReg(0xca,0x78);
			SP0838_WriteReg(0xcb,0x14);
			SP0838_WriteReg(0x14,0x20);
			SP0838_WriteReg(0x15,0x1f);
			SP0838_WriteReg(0xfd,0x00);
			}
			break;
		}
		
		default:
		{
			break;
		}

	SCI_Sleep(100);
return 0;
	} 
#endif
}
/*
typedef enum
{
	DCAMERA_EFFECT_NORMAL = 0x00,
	DCAMERA_EFFECT_BLACKWHITE,			// ڰ
	DCAMERA_EFFECT_RED,					// ƫ
	DCAMERA_EFFECT_GREEN,				// ƫ
	DCAMERA_EFFECT_BLUE,				// ƫ
	DCAMERA_EFFECT_YELLOW,				// ƫ
	DCAMERA_EFFECT_NEGATIVE,			// Ƭ
	DCAMERA_EFFECT_CANVAS,				// 
	DCAMERA_EFFECT_RELIEVOS,			// 	
	DCAMERA_EFFECT_MAX
}DCAMERA_PARAM_EFFECT_E;
*/

__align(4) const SENSOR_REG_T SP0838_image_effect_tab[][5]=	
{
    // effect normal
    {
		{0xfd, 0x00},
		{0x62, 0x00},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },
    //effect BLACKWHITE//GRAYSCALE
    {
		{0xfd, 0x00},
		{0x62, 0x40},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },
    // effect RED pink
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0xb0},
		{0x64, 0x90},
		{0xff, 0xff}
    },
    // effect GREEN//SEPIAGREEN
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0x20},
		{0x64, 0x20},
		{0xff, 0xff}
    },
    // effect  BLUE//SEPIABLUE
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0x20},
		{0x64, 0xf0},
		{0xff, 0xff}
    },
    // effect  YELLOW//SEPIAYELLOW
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0xb0},
		{0x64, 0x00},
		{0xff, 0xff}

    },  
    // effect NEGATIVE//COLORINV
    {	     
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },    
    //effect ANTIQUE//SEPIA
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0xc0},
		{0x64, 0x20},
		{0xff, 0xff}
    },
};
LOCAL uint32 set_image_effect(uint32 effect_type)
{
    uint16 i;
    
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0838_image_effect_tab[effect_type];

    SCI_ASSERT(PNULL != sensor_reg_ptr);/*assert verified*/

    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
    //SCI_TRACE_LOW:"-----------set_image_effect: effect_type = %d------------"
    SCI_TRACE_ID(TRACE_TOOL_CONVERT,SENSOR_SP0838_1477_112_2_18_0_32_42_1389,(uint8*)"d", effect_type);
    
    return 0;
}

LOCAL uint32 read_ev_value(uint32 value)
{
	return 0;
}

LOCAL uint32 write_ev_value(uint32 exposure_value)
{
	
	return 0;	
}

LOCAL uint32 read_gain_value(uint32 value)
{

	
	return 0;
}

LOCAL uint32 write_gain_value(uint32 gain_value)
{

	
	return 0;
}

LOCAL uint32 read_gain_scale(uint32 value)
{
	return SENSOR_GAIN_SCALE;
	
}


LOCAL uint32 set_frame_rate(uint32 param)
    
{
	//SP0838_WriteReg( 0xd8, uint8 data );
	return 0;
}

/******************************************************************************/
// Description:
// Global resource dependence: 
// Author:
// Note:
//		mode 0:normal;	 1:night 
/******************************************************************************/
__align(4) const SENSOR_REG_T SP0838_mode_tab[][5]=
{
#if  0
	//LCDGAMMAֵҪϸȻһȦȦĹ
	//Fps 12.5 YUV open auto frame function, չѶjpeg벻̫̫죬˽֡12.5fps
	{/*{0xa0, 0x50},*/{0xec, 0x20},{0xFF, 0xFF},},
	//Fps 12.5->3.125 YUV open auto frame function
	{/*{0xa0, 0x40},*/{0xd8, 0x30},{0xFF, 0xFF},},
	
	{0xff,0xff}
#endif

	 {    
        {0xfd,0x01},	 
        {0x14,0x20},
        {0x15,0x0f},
        {0xfd,0x00},	 
        {0xff,0xff},
        
    },
	//night
    {    
        {0xfd,0x01},	 
        {0x14,0x20},
        {0x15,0x1f},
        {0xfd,0x00},	 
        {0xff,0xff},
    }
};

LOCAL uint32 SP0838_set_work_mode(uint32 mode)
{
	uint16 i;
	SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0838_mode_tab[mode];

	SCI_ASSERT(mode <= 1);/*assert verified*/
	SCI_ASSERT(PNULL != sensor_reg_ptr);/*assert verified*/
	
	for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value); i++)
	{
		SP0838_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
	}

	//SCI_TRACE_LOW:"set_work_mode: mode = %d"
	SCI_TRACE_ID(TRACE_TOOL_CONVERT,SENSOR_SP0838_1632_112_2_18_0_32_43_1391,(uint8*)"d", mode);
	return 0;
}
