/******************************************************************************
 ** File Name:    Sensor_SP0838.c                                         *
 ** Author:        superpix                                      	  *
 ** Date:           03/02/2011                                          *
 ** Copyright:    Spreadtrum All Rights Reserved.        *
 ** Description:   implementation of digital camera register interface       *
 ******************************************************************************

 ******************************************************************************
 **                        Edit History                                       *
 ** ------------------------------------------------------------------------- *
 ** DATE           NAME             DESCRIPTION                               *
 **                                  *
 ******************************************************************************/

#ifndef _SP0838_C_
#define _SP0838_C_

#include "sensor_cfg.h"
#include "sensor_drv.h"
#include "i2c_drv.h"
#include "os_api.h"
#include "chip.h"
#include "dal_dcamera.h"

/**---------------------------------------------------------------------------*
 **                         Const variables                                   *
 **---------------------------------------------------------------------------*/
#define SP0838_I2C_ADDR_W    			0x30
#define SP0838_I2C_ADDR_R    			0x31
#define SP0838_I2C_ACK				       0x0
LOCAL uint32			s_sensor_mclk		 = 0;
LOCAL uint32			Antiflicker		 = DCAMERA_FLICKER_50HZ;

//ECLK Drv
#define  Pre_Value_P0_0x30  0x00
//Filter en&dis
#define  Pre_Value_P0_0x56  0x70
#define  Pre_Value_P0_0x57  0x10  //filter outdoor
#define  Pre_Value_P0_0x58  0x10  //filter indoor
#define  Pre_Value_P0_0x59  0x10  //filter night
#define  Pre_Value_P0_0x5a  0x02  //smooth outdoor
#define  Pre_Value_P0_0x5b  0x02  //smooth indoor
#define  Pre_Value_P0_0x5c  0x20  //smooht night
//outdoor sharpness
#define  Pre_Value_P0_0x65  0x03
#define  Pre_Value_P0_0x66  0x01
#define  Pre_Value_P0_0x67  0x03
#define  Pre_Value_P0_0x68  0x46
//indoor sharpness
#define  Pre_Value_P0_0x6b  0x04
#define  Pre_Value_P0_0x6c  0x01
#define  Pre_Value_P0_0x6d  0x03
#define  Pre_Value_P0_0x6e  0x46
//night sharpness
#define  Pre_Value_P0_0x71  0x05
#define  Pre_Value_P0_0x72  0x01
#define  Pre_Value_P0_0x73  0x03
#define  Pre_Value_P0_0x74  0x46
//color
#define  Pre_Value_P0_0x7f  0xd7  //R 
#define  Pre_Value_P0_0x87  0xf8  //B
//satutation
#define  Pre_Value_P0_0xd8  0x48
#define  Pre_Value_P0_0xd9  0x48
#define  Pre_Value_P0_0xda  0x48
#define  Pre_Value_P0_0xdb  0x48
//AE target
#define  Pre_Value_P0_0xf7  0x78
#define  Pre_Value_P0_0xf8  0x63
#define  Pre_Value_P0_0xf9  0x68
#define  Pre_Value_P0_0xfa  0x53
//HEQ
#define  Pre_Value_P0_0xdd  0x70
#define  Pre_Value_P0_0xde  0x90
//AWB pre gain
#define  Pre_Value_P1_0x28  0x75
#define  Pre_Value_P1_0x29  0x4e

//VBLANK
#define  Pre_Value_P0_0x05  0x00
#define  Pre_Value_P0_0x06  0x00
//HBLANK
#define  Pre_Value_P0_0x09  0x01
#define  Pre_Value_P0_0x0a  0x76
/**---------------------------------------------------------------------------*
 **                         Macro define                                      *
 **---------------------------------------------------------------------------*/
#define    SP0838_DELAY_MS(ms)            {\
                                              uint32 end = SCI_GetTickCount() + ms;\
                                              while (SCI_GetTickCount() < end)\
                                              ;\
                                          }
                                           
 


/**---------------------------------------------------------------------------*
 **                         Const variables                                   *
 **---------------------------------------------------------------------------*/
//  give the parameters

/**---------------------------------------------------------------------------*
 **                     Local Function Prototypes                             *
 **---------------------------------------------------------------------------*/
LOCAL uint32 SP0838_Power_On(uint32 power_on);


#ifdef DOUBLE_CAMERA_SUPPORT
__align(4) SENSOR_REG_T SP0838_YVU_640X480[] = 
#else
__align(4) const SENSOR_REG_T SP0838_YVU_640X480[] = 
#endif
{     
//SP0838 ini
{0xfd,0x00},//P0
{0x1B,0x02},
{0x27,0xe8},
{0x28,0x0B},
{0x32,0x00},
{0x22,0xc0},
{0x26,0x10}, 
{0x5f,0x11},//Bayer order
{0xfd,0x01},//P1
{0x25,0x1a},//Awb start
{0x26,0xfb},
{0x28,Pre_Value_P1_0x28},
{0x29,Pre_Value_P1_0x29},
{0xfd,0x00},
{0xe7,0x03},
{0xe7,0x00},
{0xfd,0x01},
{0x31,0x60},//64
{0x32,0x18},
{0x4d,0xdc},
{0x4e,0x53},
{0x41,0x8c},
{0x42,0x57},
{0x55,0xff},
{0x56,0x00},
{0x59,0x82},
{0x5a,0x00},
{0x5d,0xff},
{0x5e,0x6f},
{0x57,0xff},
{0x58,0x00},
{0x5b,0xff},
{0x5c,0xa8},
{0x5f,0x75},
{0x60,0x00},
{0x2d,0x00},
{0x2e,0x00},
{0x2f,0x00},
{0x30,0x00},
{0x33,0x00},
{0x34,0x00},
{0x37,0x00},
{0x38,0x00},//awb end
{0xfd,0x00},//P0
{0x33,0x6f},//LSC BPC EN
{0x51,0x3f},//BPC debug start
{0x52,0x09},
{0x53,0x00},
{0x54,0x00},
{0x55,0x10},//BPC debug end
{0x4f,0x08},//blueedge
{0x50,0x08},
{0x57,Pre_Value_P0_0x57},//Raw filter debut start
{0x58,Pre_Value_P0_0x58},
{0x59,Pre_Value_P0_0x59},
{0x56,Pre_Value_P0_0x56},
{0x5a,Pre_Value_P0_0x5a},
{0x5b,Pre_Value_P0_0x5b},
{0x5c,Pre_Value_P0_0x5c},//Raw filter debut end 
{0x65,Pre_Value_P0_0x65},//Sharpness debug start
{0x66,Pre_Value_P0_0x66},
{0x67,Pre_Value_P0_0x67},
{0x68,Pre_Value_P0_0x68},
{0x69,0x7f},
{0x6a,0x01},
{0x6b,Pre_Value_P0_0x6b},
{0x6c,Pre_Value_P0_0x6c},
{0x6d,Pre_Value_P0_0x6d},//Edge gain normal
{0x6e,Pre_Value_P0_0x6e},//Edge gain normal
{0x6f,0x7f},
{0x70,0x01},
{0x71,Pre_Value_P0_0x71}, //ֵ          
{0x72,Pre_Value_P0_0x72}, //ֵ        
{0x73,Pre_Value_P0_0x73}, //Եֵ    
{0x74,Pre_Value_P0_0x74}, //Եֵ    
{0x75,0x7f},              //ʹλ            
{0x76,0x01},//Sharpness debug end
{0xcb,0x07},//HEQ&Saturation debug start 
{0xcc,0x04},
{0xce,0xff},
{0xcf,0x10},
{0xd0,0x20},
{0xd1,0x00},
{0xd2,0x1c},
{0xd3,0x16},
{0xd4,0x00},
{0xd6,0x1c},
{0xd7,0x16},
{0xdd,Pre_Value_P0_0xdd},//Contrast
{0xde,Pre_Value_P0_0xde},//HEQ&Saturation debug end
{0x7f,Pre_Value_P0_0x7f},//Color Correction start
{0x80,0xbc},                        
{0x81,0xed},                        
{0x82,0xd7},                        
{0x83,0xd4},                        
{0x84,0xd6},                        
{0x85,0xff},                        
{0x86,0x89},                        
{0x87,Pre_Value_P0_0x87},                        
{0x88,0x3c},                        
{0x89,0x33},                        
{0x8a,0x0f},//Color Correction end  
{0x8b,0x0 },//gamma start
{0x8c,0x1a},             
{0x8d,0x29},             
{0x8e,0x41},             
{0x8f,0x62},             
{0x90,0x7c},             
{0x91,0x90},             
{0x92,0xa2},             
{0x93,0xaf},             
{0x94,0xbc},             
{0x95,0xc5},             
{0x96,0xcd},             
{0x97,0xd5},             
{0x98,0xdd},             
{0x99,0xe5},             
{0x9a,0xed},             
{0x9b,0xf5},             
{0xfd,0x01},//P1         
{0x8d,0xfd},             
{0x8e,0xff},//gamma end  
{0xfd,0x00},//P0
{0xca,0xcf},
{0xd8,Pre_Value_P0_0xd8},//UV outdoor
{0xd9,Pre_Value_P0_0xd9},//UV indoor 
{0xda,Pre_Value_P0_0xda},//UV dummy
{0xdb,Pre_Value_P0_0xdb},//UV lowlight
{0xb9,0x00},//Ygamma start
{0xba,0x04},
{0xbb,0x08},
{0xbc,0x10},
{0xbd,0x20},
{0xbe,0x30},
{0xbf,0x40},
{0xc0,0x50},
{0xc1,0x60},
{0xc2,0x70},
{0xc3,0x80},
{0xc4,0x90},
{0xc5,0xA0},
{0xc6,0xB0},
{0xc7,0xC0},
{0xc8,0xD0},
{0xc9,0xE0},
{0xfd,0x01},//P1
{0x89,0xf0},
{0x8a,0xff},//Ygamma end
{0xfd,0x00},//P0
{0xe8,0x30},//AEdebug start
{0xe9,0x30},
{0xea,0x40},//Alc Window sel
{0xf4,0x1b},//outdoor mode sel
{0xf5,0x80},
{0xf7,Pre_Value_P0_0xf7},//AE target
{0xf8,Pre_Value_P0_0xf8},
{0xf9,Pre_Value_P0_0xf9},//AE target 
{0xfa,Pre_Value_P0_0xfa},
{0xfd,0x01},//P1
{0x09,0x31},//AE Step 3.0
{0x0a,0x85},
{0x0b,0x0b},//AE Step 3.0
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},//P0
{0x05,0x00},  
{0x06,0x00},
{0x09,0x01},
{0x0a,0x76},
{0xf0,0x62},
{0xf1,0x00},
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},//P1
{0x00,0xba},
{0x0f,0x60},
{0x16,0x60},
{0x17,0xa2},
{0x18,0xaa},
{0x1b,0x60},
{0x1c,0xaa},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x78},
{0xc6,0x41},
{0xca,0x78},
{0xcb,0x0c},//AEdebug end
#if 1//caprure preview daylight 24M 50hz 20-8FPS maxgain:0x70	
{0xfd,0x00},
{0x05,Pre_Value_P0_0x05 },
{0x06,Pre_Value_P0_0x06 },
{0x09,Pre_Value_P0_0x09 },
{0x0a,Pre_Value_P0_0x0a },
{0xf0,0x62},
{0xf1,0x0 },
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},
{0x00,0xb2},
{0x0f,0x60},
{0x16,0x60},
{0x17,0xa2},
{0x18,0xaa},
{0x1b,0x60},
{0x1c,0xaa},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x70},
{0xc6,0x41},
{0xca,0x70},
{0xcb,0xc },
{0xfd,0x00},
#endif	
{0xfd,0x00},  //P0
{0x32,0x15},  //Auto_mode set
{0x34,0x66},  //Isp_mode set
{0x35,0x40},  //out format
{0xFF,0xFF},
};


/**---------------------------------------------------------------------------*
 **                         Function Definitions                              *
 **---------------------------------------------------------------------------*/
LOCAL SP0838_WriteReg( uint8  subaddr, uint8 data )
{
	
	#ifndef	_USE_DSP_I2C_
		uint8 cmd[2];
		cmd[0]	=	subaddr;
		cmd[1]	=	data;
		
		I2C_WriteCmdArr(SP0838_I2C_ADDR_W, cmd, 2, SCI_TRUE);
	#else
		DSENSOR_IICWrite((uint16)subaddr, (uint16)data);
	#endif

	SCI_TRACE_LOW("SENSOR: SP0838_WriteReg reg/value(%x,%x) !!", subaddr, data);

}

LOCAL uint8 SP0838_ReadReg( uint8  subaddr)
{
	uint8 value = 0;
	
	#ifndef	_USE_DSP_I2C_
		I2C_WriteCmdArr(SP0838_I2C_ADDR_W, &subaddr, 1, SCI_TRUE);
		I2C_ReadCmd(SP0838_I2C_ADDR_R, &value, SCI_TRUE);
		SCI_TRACE_LOW( "SP0838 read ret is %X=%X",subaddr,value );
	#else
		value = (uint16)DSENSOR_IICRead((uint16)subaddr);
	#endif
	
	return value;
}

/******************************************************************************/
// Description: sensor probe function
// Author:     benny.zou
// Input:      none
// Output:     result
// Return:     0           successful
//             others      failed
// Note:       this function only to check whether sensor is work, not identify 
//              whitch it is!!
/******************************************************************************/
LOCAL uint32 SP0838_Identify(void)
{
#if 1
    uint8 reg[2]    = {0x02, 0x02};
    uint8 value[2]  = {0x27, 0x27};
    uint8 ret       = 0;
    uint32 i;
    uint8 err_cnt = 0;
    uint32 nLoop = 1000;

//	SP0838_DELAY_MS(10);

    for(i = 0; i<2; )
    {
        nLoop = 1000;
        //SP0838_WriteReg(0xfd,0x00);  
        ret = SP0838_ReadReg(reg[i]);    
		
		SCI_TRACE_LOW( "SP0838 read ret is %X",ret );
		
        if( ret != value[i])
        {
            err_cnt++;
            if(err_cnt>3)
            {
				SCI_TRACE_LOW( "SP0838 Fail" );
				return 0xFF;
            }
            else
            {
                //Masked by frank.yang,SCI_Sleep() will cause a  Assert when called in boot precedure
                //SCI_Sleep(10);
                while(nLoop--);
                continue;
            }
        }
        err_cnt = 0;
        i++;
    }

     SCI_TRACE_LOW( "SP0838 succ" );
#endif

return (uint32)SCI_SUCCESS;

}



LOCAL uint32 hmirror_enable(uint32 enable);
LOCAL uint32 vmirror_enable(uint32 enable);

__align(4) const SENSOR_REG_T SP0838_brightness_tab[][3]=
{
#if 0
        {		
        	{0xfd, 0x00},	{0xff,0xff},
        },

#else//default level 0
	{
		{0xfd,0x00},
		{0xdc,0xd0},//level -3
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0xe0},//level -2
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0xf0},//level -1
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0x00},//level 0
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0x10},//level +1
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0x20},//level +2
		{0xff,0xff},
	},
    {		
		{0xfd,0x00},
		{0xdc,0x30},//level +3
		{0xff,0xff},
	},

#endif	
};
LOCAL uint32 set_brightness(uint32 level)
{
    
#if 1  
	uint16 i;
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0838_brightness_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
#endif    
    return 0;
}

__align(4) const SENSOR_REG_T SP0838_contrast_tab[][4]=
{
#if 0
    {		
		{0xfd, 0x00},	{0xff,0xff},
	},

#else//default level 0
	{
		{0xfd, 0x00},
		{0xdd,  Pre_Value_P0_0xdd-0x30},	//level -3
		{0xde,  Pre_Value_P0_0xde-0x30},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  Pre_Value_P0_0xdd-0x20},	//level -2
		{0xde,  Pre_Value_P0_0xde-0x20},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd, Pre_Value_P0_0xdd-0x10},	//level -1
		{0xde, Pre_Value_P0_0xde-0x10},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd, Pre_Value_P0_0xdd},	//level 0
		{0xde, Pre_Value_P0_0xde},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  Pre_Value_P0_0xdd+0x10},	//level +1
		{0xde,  Pre_Value_P0_0xde+0x10},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  Pre_Value_P0_0xdd+0x20},	//level +2
		{0xde,  Pre_Value_P0_0xde+0x20},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  Pre_Value_P0_0xdd+0x30},	//level +3
		{0xde,  Pre_Value_P0_0xde+0x30},
		{0xff, 0xff}
	}, 
#endif
};

LOCAL uint32 set_contrast(uint32 level)
{
#if 1
    uint16 i;    
    
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0838_contrast_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
#endif    
    return 0;
}


LOCAL uint32 Sensor_Write_Regs( SENSOR_REG_T* sensor_reg_ptr )
{
#if 1

    uint32 i;
    
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) && (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
#endif
    return 0;
}


__align(4) const SENSOR_REG_T SP0838_mode_tab[][40]=
{
#if 0
    {		
		{0xfd, 0x00},	{0xff,0xff},
	},

#else
	//caprure preview daylight 24M 50hz 20-8FPS maxgain:0x70
{     
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x62},
{0xf1,0x0 },
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},
{0x00,0xb2},
{0x0f,0x60},
{0x16,0x60},
{0x17,0xa2},
{0x18,0xaa},
{0x1b,0x60},
{0x1c,0xaa},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x70},
{0xc6,0x41},
{0xca,0x70},
{0xcb,0xc },
{0x14,0x20},//
{0x15,0x0f},//
{0xfd,0x00},
{0xFF,0xFF},	//־
},
//caprure preview night 24M 50hz 20-6FPS maxgain:0x78
    {    
{0xfd,0x00}, 
{0x05,0x0 }, 
{0x06,0x0 }, 
{0x09,0x1 }, 
{0x0a,0x76}, 
{0xf0,0x62}, 
{0xf1,0x0 }, 
{0xf2,0x5f}, 
{0xf5,0x78}, 
{0xfd,0x01}, 
{0x00,0xc0}, 
{0x0f,0x60}, 
{0x16,0x60}, 
{0x17,0xa8}, 
{0x18,0xb0}, 
{0x1b,0x60}, 
{0x1c,0xb0}, 
{0xb4,0x20}, 
{0xb5,0x3a}, 
{0xb6,0x5e}, 
{0xb9,0x40}, 
{0xba,0x4f}, 
{0xbb,0x47}, 
{0xbc,0x45}, 
{0xbd,0x43}, 
{0xbe,0x42}, 
{0xbf,0x42}, 
{0xc0,0x42}, 
{0xc1,0x41}, 
{0xc2,0x41}, 
{0xc3,0x41}, 
{0xc4,0x41}, 
{0xc5,0x41}, 
{0xc6,0x41}, 
{0xca,0x78}, 
{0xcb,0x10}, 
{0x14,0x20},//
{0x15,0x1f},//
{0xfd,0x00},
{0xFF,0xFF},	//־
    }
#endif	
};  
    
LOCAL uint32 set_preview_mode(uint32 preview_mode)
{
#if 1
	switch (preview_mode)
	{
		case DCAMERA_ENVIRONMENT_NORMAL: 
		case DCAMERA_ENVIRONMENT_SUNNY:
		{
			if(Antiflicker== DCAMERA_FLICKER_50HZ)
			{
			SCI_TRACE_LOW("normal mode 50hz\r\n");
			//caprure preview daylight 24M 50hz 20-8FPS maxgain:0x70   
			SP0838_WriteReg(0xfd,0x00);
			SP0838_WriteReg(0x05,0x0 );
			SP0838_WriteReg(0x06,0x0 );
			SP0838_WriteReg(0x09,0x1 );
			SP0838_WriteReg(0x0a,0x76);
			SP0838_WriteReg(0xf0,0x62);
			SP0838_WriteReg(0xf1,0x0 );
			SP0838_WriteReg(0xf2,0x5f);
			SP0838_WriteReg(0xf5,0x78);
			SP0838_WriteReg(0xfd,0x01);
			SP0838_WriteReg(0x00,0xb2);
			SP0838_WriteReg(0x0f,0x60);
			SP0838_WriteReg(0x16,0x60);
			SP0838_WriteReg(0x17,0xa2);
			SP0838_WriteReg(0x18,0xaa);
			SP0838_WriteReg(0x1b,0x60);
			SP0838_WriteReg(0x1c,0xaa);
			SP0838_WriteReg(0xb4,0x20);
			SP0838_WriteReg(0xb5,0x3a);
			SP0838_WriteReg(0xb6,0x5e);
			SP0838_WriteReg(0xb9,0x40);
			SP0838_WriteReg(0xba,0x4f);
			SP0838_WriteReg(0xbb,0x47);
			SP0838_WriteReg(0xbc,0x45);
			SP0838_WriteReg(0xbd,0x43);
			SP0838_WriteReg(0xbe,0x42);
			SP0838_WriteReg(0xbf,0x42);
			SP0838_WriteReg(0xc0,0x42);
			SP0838_WriteReg(0xc1,0x41);
			SP0838_WriteReg(0xc2,0x41);
			SP0838_WriteReg(0xc3,0x41);
			SP0838_WriteReg(0xc4,0x41);
			SP0838_WriteReg(0xc5,0x70);
			SP0838_WriteReg(0xc6,0x41);
			SP0838_WriteReg(0xca,0x70);
			SP0838_WriteReg(0xcb,0xc );
			SP0838_WriteReg(0x14,0x20);
			SP0838_WriteReg(0x15,0x0f);
			SP0838_WriteReg(0xfd,0x00);
			}
			else	
			{
			SCI_TRACE_LOW("normal mode 60hz\r\n");
			//caprure preview daylight 24M 60hz 20-8FPS maxgain:0x70	       
			SP0838_WriteReg(0xfd,0x00);
			SP0838_WriteReg(0x05,0x0 );
			SP0838_WriteReg(0x06,0x0 );
			SP0838_WriteReg(0x09,0x1 );
			SP0838_WriteReg(0x0a,0x76);
			SP0838_WriteReg(0xf0,0x51);
			SP0838_WriteReg(0xf1,0x0 );
			SP0838_WriteReg(0xf2,0x5c);
			SP0838_WriteReg(0xf5,0x75);
			SP0838_WriteReg(0xfd,0x01);
			SP0838_WriteReg(0x00,0xb4);
			SP0838_WriteReg(0x0f,0x5d);
			SP0838_WriteReg(0x16,0x5d);
			SP0838_WriteReg(0x17,0xa4);
			SP0838_WriteReg(0x18,0xac);
			SP0838_WriteReg(0x1b,0x5d);
			SP0838_WriteReg(0x1c,0xac);
			SP0838_WriteReg(0xb4,0x21);
			SP0838_WriteReg(0xb5,0x3d);
			SP0838_WriteReg(0xb6,0x4d);
			SP0838_WriteReg(0xb9,0x40);
			SP0838_WriteReg(0xba,0x4f);
			SP0838_WriteReg(0xbb,0x47);
			SP0838_WriteReg(0xbc,0x45);
			SP0838_WriteReg(0xbd,0x43);
			SP0838_WriteReg(0xbe,0x42);
			SP0838_WriteReg(0xbf,0x42);
			SP0838_WriteReg(0xc0,0x42);
			SP0838_WriteReg(0xc1,0x41);
			SP0838_WriteReg(0xc2,0x41);
			SP0838_WriteReg(0xc3,0x41);
			SP0838_WriteReg(0xc4,0x41);
			SP0838_WriteReg(0xc5,0x41);
			SP0838_WriteReg(0xc6,0x41);
			SP0838_WriteReg(0xca,0x70);
			SP0838_WriteReg(0xcb,0xf );
			SP0838_WriteReg(0x14,0x20);
			SP0838_WriteReg(0x15,0x0f);
			SP0838_WriteReg(0xfd,0x00);
			}	
			break;
		}
		case DCAMERA_ENVIRONMENT_NIGHT:
		{

			if(Antiflicker== DCAMERA_FLICKER_50HZ)
			{
			SCI_TRACE_LOW("night mode 50hz\r\n");
			//caprure preview night 24M 50hz 20-6FPS maxgain:0x78	       
			SP0838_WriteReg(0xfd,0x00);
			SP0838_WriteReg(0x05,0x0 );
			SP0838_WriteReg(0x06,0x0 );
			SP0838_WriteReg(0x09,0x1 );
			SP0838_WriteReg(0x0a,0x76);
			SP0838_WriteReg(0xf0,0x62);
			SP0838_WriteReg(0xf1,0x0 );
			SP0838_WriteReg(0xf2,0x5f);
			SP0838_WriteReg(0xf5,0x78);
			SP0838_WriteReg(0xfd,0x01);
			SP0838_WriteReg(0x00,0xc0);
			SP0838_WriteReg(0x0f,0x60);
			SP0838_WriteReg(0x16,0x60);
			SP0838_WriteReg(0x17,0xa8);
			SP0838_WriteReg(0x18,0xb0);
			SP0838_WriteReg(0x1b,0x60);
			SP0838_WriteReg(0x1c,0xb0);
			SP0838_WriteReg(0xb4,0x20);
			SP0838_WriteReg(0xb5,0x3a);
			SP0838_WriteReg(0xb6,0x5e);
			SP0838_WriteReg(0xb9,0x40);
			SP0838_WriteReg(0xba,0x4f);
			SP0838_WriteReg(0xbb,0x47);
			SP0838_WriteReg(0xbc,0x45);
			SP0838_WriteReg(0xbd,0x43);
			SP0838_WriteReg(0xbe,0x42);
			SP0838_WriteReg(0xbf,0x42);
			SP0838_WriteReg(0xc0,0x42);
			SP0838_WriteReg(0xc1,0x41);
			SP0838_WriteReg(0xc2,0x41);
			SP0838_WriteReg(0xc3,0x41);
			SP0838_WriteReg(0xc4,0x41);
			SP0838_WriteReg(0xc5,0x41);
			SP0838_WriteReg(0xc6,0x41);
			SP0838_WriteReg(0xca,0x78);
			SP0838_WriteReg(0xcb,0x10);
			SP0838_WriteReg(0x14,0x20);
			SP0838_WriteReg(0x15,0x1f);
			SP0838_WriteReg(0xfd,0x00);
			}
			else
			{
			SCI_TRACE_LOW("night mode 60hz\r\n");
			//caprure preview night 24M 60hz 20-6FPS maxgain:0x78
			SP0838_WriteReg(0xfd,0x00);
			SP0838_WriteReg(0x05,0x0 );
			SP0838_WriteReg(0x06,0x0 );
			SP0838_WriteReg(0x09,0x1 );
			SP0838_WriteReg(0x0a,0x76);
			SP0838_WriteReg(0xf0,0x51);
			SP0838_WriteReg(0xf1,0x0 );
			SP0838_WriteReg(0xf2,0x5c);
			SP0838_WriteReg(0xf5,0x75);
			SP0838_WriteReg(0xfd,0x01);
			SP0838_WriteReg(0x00,0xc1);
			SP0838_WriteReg(0x0f,0x5d);
			SP0838_WriteReg(0x16,0x5d);
			SP0838_WriteReg(0x17,0xa9);
			SP0838_WriteReg(0x18,0xb1);
			SP0838_WriteReg(0x1b,0x5d);
			SP0838_WriteReg(0x1c,0xb1);
			SP0838_WriteReg(0xb4,0x21);
			SP0838_WriteReg(0xb5,0x3d);
			SP0838_WriteReg(0xb6,0x4d);
			SP0838_WriteReg(0xb9,0x40);
			SP0838_WriteReg(0xba,0x4f);
			SP0838_WriteReg(0xbb,0x47);
			SP0838_WriteReg(0xbc,0x45);
			SP0838_WriteReg(0xbd,0x43);
			SP0838_WriteReg(0xbe,0x42);
			SP0838_WriteReg(0xbf,0x42);
			SP0838_WriteReg(0xc0,0x42);
			SP0838_WriteReg(0xc1,0x41);
			SP0838_WriteReg(0xc2,0x41);
			SP0838_WriteReg(0xc3,0x41);
			SP0838_WriteReg(0xc4,0x41);
			SP0838_WriteReg(0xc5,0x41);
			SP0838_WriteReg(0xc6,0x41);
			SP0838_WriteReg(0xca,0x78);
			SP0838_WriteReg(0xcb,0x14);
			SP0838_WriteReg(0x14,0x20);
			SP0838_WriteReg(0x15,0x1f);
			SP0838_WriteReg(0xfd,0x00);
			}
			break;
		}
		
		default:
		{
			break;
		}

	SCI_Sleep(100);
return 0;
	} 
#endif
}


__align(4) const SENSOR_REG_T SP0838_image_effect_tab[][5]=	
{
#if 0
    {		
		{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},
		{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xff,0xff},
	},

#else
    // effect normal
    {
		{0xfd, 0x00},
		{0x62, 0x00},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },
    //effect BLACKWHITE//GRAYSCALE
    {
		{0xfd, 0x00},
		{0x62, 0x40},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },
    // effect RED pink
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0xb0},
		{0x64, 0x90},
		{0xff, 0xff}
    },
    // effect GREEN//SEPIAGREEN
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0x20},
		{0x64, 0x20},
		{0xff, 0xff}
    },
    // effect  BLUE//SEPIABLUE
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0x20},
		{0x64, 0xf0},
		{0xff, 0xff}
    },
    // effect  YELLOW//SEPIAYELLOW
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0xb0},
		{0x64, 0x00},
		{0xff, 0xff}

    },  
    // effect NEGATIVE//COLORINV
    {	     
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },    
    //effect ANTIQUE//SEPIA
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0xc0},
		{0x64, 0x20},
		{0xff, 0xff}
    },
#endif    
};
LOCAL uint32 set_image_effect(uint32 effect_type)
{
#if 1
    uint16 i;
    
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0838_image_effect_tab[effect_type];

    SCI_ASSERT(PNULL != sensor_reg_ptr);

    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
#endif    
    return 0;
}

LOCAL uint32 SP0838_ae_awb_enable(uint32 ae_enable, uint32 awb_enable)
{
#if 1
	uint16 ae_value = 0 , awb_value=0;

	SP0838_WriteReg(0xfd,0x00);
	ae_value = SP0838_ReadReg(0x32);
	awb_value = SP0838_ReadReg(0x32);

	if(0x01==ae_enable)
	{
		ae_value |= 0x05;       
	}
	else if(0x00==ae_enable)
	{
		ae_value &= 0xf0;
	}

	if(0x01==awb_enable)
	{
		awb_value |= 0x10;
	}
	else if(0x00==awb_enable)
	{
		awb_value &= 0xef;
	}	
	
	SP0838_WriteReg(0x32, ae_value);
	SP0838_WriteReg(0x32, awb_value);
#endif
	return 0;
}

LOCAL void Delayms (uint32 ms)	
{
	uint32 t1, t2;
	
	t1 = t2 = SCI_GetTickCount ();
	
	do{
		t2 = SCI_GetTickCount ();
	}while (t2 < (t1+ms));
}

LOCAL uint32 SP0838_before_snapshot(uint32 para)
{
	#if 0                   // for FAE when have to reconfig the HB/VB.
	uint8 reg_val = 0;
	uint16 exposal_time=0x00;


	SP0838_ae_awb_enable(0x00,0x00);   // close aec awb

	reg_val = SP0838_ReadReg(0x04);
	exposal_time=reg_val&0x00ff;
	reg_val = SP0838_ReadReg(0x03);
	exposal_time|=((reg_val&0x00ff)<<0x08);
	
	//exposal_time=exposal_time*2/3;

	if(exposal_time<2)
	{
		exposal_time=2;
	}

	reg_val=exposal_time&0x00ff;
	SP0838_WriteReg(0x04, reg_val);
	reg_val=(exposal_time&0xff00)>>0x08;
	SP0838_WriteReg(0x03, reg_val);

	
	SP0838_WriteReg(0x01,0xfa);
	SP0838_WriteReg(0x02,0x70);  
	SP0838_WriteReg(0x0f,0x01);
	SP0838_WriteReg(0xe2,0x00);
	SP0838_WriteReg(0xe3,0x64);
	
	Delayms(400);	
	#endif

	return 0;
}

LOCAL uint32 SP0838_after_snapshot(uint32 para)
{

	#if 0                          // for FAE when have to reconfig the HB/VB.
	SP0838_WriteReg(0x01,0xfa);  
	SP0838_WriteReg(0x02,0x70);  
	SP0838_WriteReg(0x0f,0x01);
	SP0838_WriteReg(0xe2,0x00);
	SP0838_WriteReg(0xe3,0x64);  


	SP0838_ae_awb_enable(0x01,0x01);   // Open aec awb
	
	Delayms(400);
	#endif

	return 0;
}

//@ Chenfeng for adding AWB & AE functions
__align(4) const SENSOR_REG_T SP0838_awb_tab[][7]=
{
#if 0
    {		
		{0xfd, 0x00},	{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xff,0xff},
	},

#else
	   	//AUTO  // AUTO 3000K~7000K 
	   	{
			{0xfd,0x01},                      
			{0x28,Pre_Value_P1_0x28},		                  
			{0x29,Pre_Value_P1_0x29},                      
			{0xfd,0x00}, 
			{0x32,0x15},   
			{0xfd,0x00},  
		{0xff,0xff} 
		},    
		//INCANDESCENCE:  //2800K~3000K  
		{
			{0xfd,0x00}, 
			{0x32,0x05},                 
			{0xfd,0x01},                 
			{0x28,0x41},		             
			{0x29,0x71},		             
			{0xfd,0x00}, 
		{0xff,0xff} 
		},
		//U30 //3000K
		{
			{0xfd,0x00},
			{0x32,0x05},       
			{0xfd,0x01},       
			{0x28,0x41},	     
			{0x29,0x71},	     
			{0xfd,0x00},     
		{0xff,0xff} 
		},  
		//CWF  //4150K
		{
			{0xfd,0x00},
			{0x32,0x05},       
			{0xfd,0x01},       
			{0x28,0x57},	     
			{0x29,0x66},	     
			{0xfd,0x00},     
		{0xff,0xff} 
		},    
		//FLUORESCENT://4200K~5000K   
		{
			{0xfd,0x00},  
			{0x32,0x05},                  
			{0xfd,0x01},                  
			{0x28,0x5a},		              
			{0x29,0x62},		              
			{0xfd,0x00},
		    {0xff,0xff} 
		},
		//SUN: //DAYLIGHT//6500K   
		{
			{0xfd,0x00}, 
			{0x32,0x05},           
			{0xfd,0x01},           
			{0x28,0x6b},	         
			{0x29,0x48},	         
			{0xfd,0x00},  
		    {0xff,0xff} 
		},
                //CLOUD: //7000K 
		{
			{0xfd,0x00}, 
			{0x32,0x05},          
			{0xfd,0x01},          
			{0x28,0x71},	        
			{0x29,0x41},	        
			{0xfd,0x00},
		    {0xff,0xff} 
		},
#endif		
};

LOCAL uint32 set_SP0838_awb(uint32 mode)
{
#if 1
	uint8 awb_en_value;
	uint16 i;
	
	SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0838_awb_tab[mode];

	SP0838_WriteReg(0xfd,0x00);
	awb_en_value = SP0838_ReadReg(0x32);

	SCI_ASSERT(mode < 7);
	SCI_ASSERT(PNULL != sensor_reg_ptr);
	
	for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value); i++)
	{
	       if(0x32 == sensor_reg_ptr[i].reg_addr)
       	{
			if(mode == 0)
		   		SP0838_WriteReg(0x32, awb_en_value |0x10 );
			else
               	SP0838_WriteReg(0x32, awb_en_value &0xef );
       	}
      else
       {
           SP0838_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
	   }
	}
	Delayms(100); 
#endif	
	return 0;
}

LOCAL uint32 set_SP0838_ae_enable(uint32 enable)
{
#if 1
   #define AE_ENABLE  (0x32)
    unsigned char ae_value;
		SP0838_WriteReg(0xfd,0x00);
        ae_value=SP0838_ReadReg(AE_ENABLE);

        if(0x00==enable)
        {
            ae_value &= 0xf0;
            SP0838_WriteReg(AE_ENABLE,ae_value);
        }
        else if(0x01==enable)
        {
            ae_value|=0x05;
            SP0838_WriteReg(AE_ENABLE,ae_value);
        }

#endif
	return 0;
}

/******************************************************************************/
// Description: anti 50/60 hz banding flicker
// Global resource dependence: 
// Author:
// Note:
//		
/******************************************************************************/
LOCAL uint32 set_SP0838_anti_flicker(uint32 mode)
{ 
 #if 1
     switch(mode)
    {
        case DCAMERA_FLICKER_50HZ:			
		Antiflicker = DCAMERA_FLICKER_50HZ;
		SCI_TRACE_LOW( " set_SP0838_anti_flicker  50hz \r\n" );
		break;
            
        case DCAMERA_FLICKER_60HZ:
		Antiflicker = DCAMERA_FLICKER_60HZ;
		SCI_TRACE_LOW( " set_SP0838_anti_flicker  60hz \r\n" );
		break;
            
        default:
            break;
    }
    
    Delayms(100); 
#endif
    return 0;
}
/******************************************************************************/
// Description: set video mode
// Global resource dependence: 
// Author:
// Note:
//		 
/******************************************************************************/
__align(4) const SENSOR_REG_T SP0838_video_mode_nand_tab[][9]=
{
#if 1
    {		
		{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xff,0xff},
	},

#else
    {
        {0x01,0x6a},{0x02,0x32},{0x0f,0x20},{0xe2,0x00},{0xe3,0x96},{0xe8,0x04},{0xe9,0x1a}, {0xec,0x20},{0xff, 0xff} 
    },    
    //vodeo mode
    {
        {0x01,0x6a},{0x02,0x70},{0x0f,0x00},{0xe2,0x00},{0xe3,0x96},{0xe8,0x02},{0xe9,0x58}, {0xec,0x20},{0xff, 0xff}      
    }
#endif
};
/******************************************************************************/
// Description: set video mode
// Global resource dependence: 
// Author:
// Note:
//		 
/******************************************************************************/
__align(4) const SENSOR_REG_T SP0838_video_mode_nor_tab_50hz[][40]=
{
#if 0
    {		
		{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},
		{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xff,0xff},
	},

#else
	//caprure preview daylight 24M 50hz 20-8FPS maxgain:0x70
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x62},
{0xf1,0x0 },
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},
{0x00,0xb2},
{0x0f,0x60},
{0x16,0x60},
{0x17,0xa2},
{0x18,0xaa},
{0x1b,0x60},
{0x1c,0xaa},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x70},
{0xc6,0x41},
{0xca,0x70},
{0xcb,0xc },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},	//־	
	},    
    //Video record daylight 24M 50hz fix 20FPS maxgain:0x78
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x62},
{0xf1,0x0 },
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},
{0x00,0xa9},
{0x0f,0x60},
{0x16,0x60},
{0x17,0x91},
{0x18,0x99},
{0x1b,0x60},
{0x1c,0x99},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x78},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x41},
{0xc6,0x41},
{0xca,0x78},
{0xcb,0x5 },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},//־	
    },
   // UPCC  mode  daylight 24M 50hz fix 20FPS maxgain:0x78
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x62},
{0xf1,0x0 },
{0xf2,0x5f},
{0xf5,0x78},
{0xfd,0x01},
{0x00,0xa9},
{0x0f,0x60},
{0x16,0x60},
{0x17,0x91},
{0x18,0x99},
{0x1b,0x60},
{0x1c,0x99},
{0xb4,0x20},
{0xb5,0x3a},
{0xb6,0x5e},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x78},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x41},
{0xc6,0x41},
{0xca,0x78},
{0xcb,0x5 },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},//־	
    }  
#endif	
};    


__align(4) const SENSOR_REG_T SP0838_video_mode_nor_tab_60hz[][40]=
{
#if 0
    {		
		{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},
		{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xff,0xff},
	},

#else
	//caprure preview daylight 24M 60hz 20-8FPS maxgain:0x70
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x51},
{0xf1,0x0 },
{0xf2,0x5c},
{0xf5,0x75},
{0xfd,0x01},
{0x00,0xb4},
{0x0f,0x5d},
{0x16,0x5d},
{0x17,0xa4},
{0x18,0xac},
{0x1b,0x5d},
{0x1c,0xac},
{0xb4,0x21},
{0xb5,0x3d},
{0xb6,0x4d},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x42},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x41},
{0xc6,0x41},
{0xca,0x70},
{0xcb,0xf },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},//־  
	},    
    //Video record daylight 24M 60hz fix 20FPS maxgain:0x78
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x51},
{0xf1,0x0 },
{0xf2,0x5c},
{0xf5,0x75},
{0xfd,0x01},
{0x00,0xa9},
{0x0f,0x5d},
{0x16,0x5d},
{0x17,0x91},
{0x18,0x99},
{0x1b,0x5d},
{0x1c,0x99},
{0xb4,0x21},
{0xb5,0x3d},
{0xb6,0x4d},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x78},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x41},
{0xc6,0x41},
{0xca,0x78},
{0xcb,0x6 },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},//־     
    },
   // UPCC  mode daylight 24M 60hz fix 20FPS maxgain:0x78
    {
{0xfd,0x00},
{0x05,0x0 },
{0x06,0x0 },
{0x09,0x1 },
{0x0a,0x76},
{0xf0,0x51},
{0xf1,0x0 },
{0xf2,0x5c},
{0xf5,0x75},
{0xfd,0x01},
{0x00,0xa9},
{0x0f,0x5d},
{0x16,0x5d},
{0x17,0x91},
{0x18,0x99},
{0x1b,0x5d},
{0x1c,0x99},
{0xb4,0x21},
{0xb5,0x3d},
{0xb6,0x4d},
{0xb9,0x40},
{0xba,0x4f},
{0xbb,0x47},
{0xbc,0x45},
{0xbd,0x43},
{0xbe,0x42},
{0xbf,0x78},
{0xc0,0x42},
{0xc1,0x41},
{0xc2,0x41},
{0xc3,0x41},
{0xc4,0x41},
{0xc5,0x41},
{0xc6,0x41},
{0xca,0x78},
{0xcb,0x6 },
{0x14,0x20},
{0x15,0x0f},
{0xfd,0x00},
{0xFF,0xFF},//־    
    }  
#endif	
};


LOCAL uint32 set_SP0838_video_mode(uint32 mode)
{
#if 1
    uint8 data=0x00;
    uint16 i;
    SENSOR_REG_T* sensor_reg_ptr = PNULL;
    uint8 tempregval = 0;
    SCI_TRACE_LOW(" xg:set_SP0838_video_mode ,%d,%d\r\n",mode,Antiflicker);
    SCI_ASSERT(mode <=DCAMERA_MODE_MAX);

	if(Antiflicker ==DCAMERA_FLICKER_50HZ )
	{
	    if(CHIP_DetectMemType())
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0838_video_mode_nor_tab_50hz[mode];

	    else
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0838_video_mode_nor_tab_50hz[mode];
	}
	else
	{
	    if(CHIP_DetectMemType())
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0838_video_mode_nor_tab_60hz[mode];

	    else
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0838_video_mode_nor_tab_60hz[mode];

	}

    SCI_ASSERT(PNULL != sensor_reg_ptr);

    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value); i++)
    {
    	SP0838_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
#endif	
    return 0;
}

__align(4) const SENSOR_REG_T SP0838_ev_tab[][6]=
{   
#if 0
    {		
		{0xfd, 0x00},{0xfd, 0x00},{0xff,0xff},
	},
#else	
    {
		{0xfd,0x00},   
		{0xf7,Pre_Value_P0_0xf7-0x18},//level -3
		{0xf8,Pre_Value_P0_0xf8-0x18},	
		{0xf9,Pre_Value_P0_0xf9-0x18},
		{0xfa,Pre_Value_P0_0xfa-0x18},		
		{0xff,0xff},     
	},
    {
		{0xfd,0x00},   
		{0xf7,Pre_Value_P0_0xf7-0x10},//level -2
		{0xf8,Pre_Value_P0_0xf8-0x10},	
		{0xf9,Pre_Value_P0_0xf9-0x10},
		{0xff,0xff},     
	},
    {
		{0xfd,0x00},   
		{0xf7,Pre_Value_P0_0xf7-0x08},//level -1
		{0xf8,Pre_Value_P0_0xf8-0x08},	
		{0xf9,Pre_Value_P0_0xf9-0x08},
		{0xfa,Pre_Value_P0_0xfa-0x08},		
		{0xff,0xff},     
	},
    {
		{0xfd,0x00},   
		{0xf7,Pre_Value_P0_0xf7},//level 0
		{0xf8,Pre_Value_P0_0xf8},	
		{0xf9,Pre_Value_P0_0xf9},
		{0xfa,Pre_Value_P0_0xfa},		
		{0xff,0xff},    
	},
    {
		{0xfd,0x00},   
		{0xf7,Pre_Value_P0_0xf7+0x08},//level +1
		{0xf8,Pre_Value_P0_0xf8+0x08},	
		{0xf9,Pre_Value_P0_0xf9+0x08},
		{0xfa,Pre_Value_P0_0xfa+0x08},		
		{0xff,0xff},  
	},
    {
		{0xfd,0x00},   
		{0xf7,Pre_Value_P0_0xf7+0x10},//level +2
		{0xf8,Pre_Value_P0_0xf8+0x10},	
		{0xf9,Pre_Value_P0_0xf9+0x10},
		{0xfa,Pre_Value_P0_0xfa+0x10},		
		{0xff,0xff},     
	},
    {
		{0xfd,0x00},   
		{0xf7,Pre_Value_P0_0xf7+0x18},//level +3
		{0xf8,Pre_Value_P0_0xf8+0x18},	
		{0xf9,Pre_Value_P0_0xf9+0x18},
		{0xfa,Pre_Value_P0_0xfa+0x18},		
		{0xff,0xff},    
	},
#endif    
};

LOCAL uint32 set_SP0838_ev(uint32 level)
{
#if 1
    uint16 i; 
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0838_ev_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    SCI_ASSERT(level < 7);
 
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) ||(0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        SP0838_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }

    
#endif
    return 0;
}

LOCAL SENSOR_REG_TAB_INFO_T s_SP0838_resolution_Tab_YUV[]=
{   
	// COMMON INIT
	{ ADDR_AND_LEN_OF_ARRAY(SP0838_YVU_640X480),    640,    480, 24,  SENSOR_IMAGE_FORMAT_YUV422 },

	// YUV422 PREVIEW 1
	{ PNULL,                    0,   640,    480,    24   ,        SENSOR_IMAGE_FORMAT_YUV422 },
	{ PNULL,                    0,      0,  0  ,        0,        0      },   
	{ PNULL,                    0,      0,  0  ,        0,        0      },   
	{ PNULL,                    0,      0,  0  ,        0,        0      },   


	// YUV422 PREVIEW 2 
	{ PNULL,                    0,      0,  0  ,        0,        0      },   
	{ PNULL,                    0,      0,  0  ,        0,        0      },   
	{ PNULL,                    0,      0,  0  ,        0,        0      },   
	{ PNULL,                    0,      0,  0  ,        0,        0      }

};

LOCAL SENSOR_IOCTL_FUNC_TAB_T s_SP0838_ioctl_func_tab = 
{
// Internal 
    PNULL,
    SP0838_Power_On,  // PNULL 
    PNULL,
    SP0838_Identify,
    
    PNULL,          // write register
    PNULL,          // read  register   
    PNULL,
    PNULL,

// External
    set_SP0838_ae_enable, //set_ae_enable,
    hmirror_enable, //set_hmirror_enable,
    vmirror_enable, //set_vmirror_enable,
    
    set_brightness,
    set_contrast, //write_ev_value,
    PNULL, //read_gain_value,
    PNULL, //write_gain_value,
    set_preview_mode, //read_gain_scale,
    set_image_effect, //set_frame_rate,
    SP0838_before_snapshot,
    SP0838_after_snapshot,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    set_SP0838_awb,    
    PNULL, //read_ev_value,
    PNULL, //write_ev_value,
    set_SP0838_ev, //read_gain_value,
    PNULL, //write_gain_value,
    PNULL, //read_gain_scale,
    PNULL, //set_frame_rate,
    
    PNULL,
    PNULL,
    set_SP0838_anti_flicker, //PNULL,
    set_SP0838_video_mode,    
};

PUBLIC SENSOR_INFO_T g_SP0838_yuv_info =
{
	SP0838_I2C_ADDR_W,				// salve i2c write address
	SP0838_I2C_ADDR_R, 				// salve i2c read address
	
	0,								// bit0: 0: i2c register value is 8 bit, 1: i2c register value is 16 bit
									// bit2: 0: i2c register addr  is 8 bit, 1: i2c register addr  is 16 bit
									// other bit: reseved
	SENSOR_HW_SIGNAL_PCLK_P|\
	SENSOR_HW_SIGNAL_VSYNC_P|\
	SENSOR_HW_SIGNAL_HSYNC_P,		// bit0: 0:negative; 1:positive -> polarily of pixel clock
									// bit2: 0:negative; 1:positive -> polarily of horizontal synchronization signal
									// bit4: 0:negative; 1:positive -> polarily of vertical synchronization signal
									// other bit: reseved											

	// preview mode
	SENSOR_ENVIROMENT_NORMAL|\
	SENSOR_ENVIROMENT_NIGHT|\
	SENSOR_ENVIROMENT_SUNNY,		
	
	// image effect
	SENSOR_IMAGE_EFFECT_NORMAL|\
	SENSOR_IMAGE_EFFECT_BLACKWHITE|\
	SENSOR_IMAGE_EFFECT_RED|\
	SENSOR_IMAGE_EFFECT_GREEN|\
	SENSOR_IMAGE_EFFECT_BLUE|\
	SENSOR_IMAGE_EFFECT_YELLOW|\
	SENSOR_IMAGE_EFFECT_NEGATIVE|\
	SENSOR_IMAGE_EFFECT_CANVAS,
	
	// while balance mode
	0,
		
	7,								// bit[0:7]: count of step in brightness, contrast, sharpness, saturation
									// bit[8:31] reseved
	
	SENSOR_LOW_PULSE_RESET,		// reset pulse level
	100,								// reset pulse width(ms)
	
	SENSOR_HIGH_LEVEL_PWDN,			// 1: high level valid; 0: low level valid
	2,
	0x02, 0x27,
	0x02, 0x27,
	
	SENSOR_AVDD_2800MV,				// voltage of avdd	
	
	640,							// max width of source image
	480,							// max height of source image
	"SP0838",						// name of sensor												

	SENSOR_IMAGE_FORMAT_YUV422,		// define in SENSOR_IMAGE_FORMAT_E enum,
									// if set to SENSOR_IMAGE_FORMAT_MAX here, image format depent on SENSOR_REG_TAB_INFO_T
	SENSOR_IMAGE_PATTERN_YUV422_YUYV,	// pattern of input image form sensor;			

	s_SP0838_resolution_Tab_YUV,	// point to resolution table information structure
	&s_SP0838_ioctl_func_tab,		// point to ioctl function table
			
	PNULL,							// information and table about Rawrgb sensor
	PNULL,							// extend information about sensor	
	SENSOR_AVDD_2800MV,                     // iovdd
	SENSOR_AVDD_1800MV,                      // dvdd
	3,								//skip frame num before preview
	0,								//skip frame num before capture
	0,			//skip frame num during preview
	2			//skip frame num during video preview
};

LOCAL uint32 SP0838_Power_On(uint32 power_on)
{
    SENSOR_AVDD_VAL_E		dvdd_val=g_SP0838_yuv_info.dvdd_val;
    SENSOR_AVDD_VAL_E		avdd_val=g_SP0838_yuv_info.avdd_val;
    SENSOR_AVDD_VAL_E		iovdd_val=g_SP0838_yuv_info.iovdd_val;  
    BOOLEAN 				power_down=g_SP0838_yuv_info.power_down_level;	    
    BOOLEAN 				reset_level=g_SP0838_yuv_info.reset_pulse_level;
    uint32 				reset_width=g_SP0838_yuv_info.reset_pulse_width;	    
    
    if(SCI_TRUE==power_on)
	{
		Sensor_SetVoltage(dvdd_val, avdd_val, iovdd_val);
		        SCI_Sleep(10);        
#ifdef DOUBLE_CAMERA_SUPPORT
		if (Sensor_GetFrontCameraState())
		{
			GPIO_SetFrontSensorPwdn(!power_down);
			GPIO_SetSensorPwdn(power_down);
		}
		else
		{
			GPIO_SetFrontSensorPwdn(power_down);
			GPIO_SetSensorPwdn(!power_down);
		}
#else      
	        GPIO_SetSensorPwdn(!power_down);
#endif

		//Open Mclk in default frequency
		Sensor_SetMCLK(24);   
             SCI_Sleep(50);  

		// reset pin on SP0838
		 Sensor_SetResetLevel(!reset_level);
	        SCI_Sleep(50);
	        Sensor_SetResetLevel(reset_level);
	        SCI_Sleep(reset_width);
	        Sensor_SetResetLevel(!reset_level);
	        SCI_Sleep(100);
	       
	}
    else
    {
#ifdef DOUBLE_CAMERA_SUPPORT
		GPIO_SetFrontSensorPwdn(power_down);
		GPIO_SetSensorPwdn(power_down);
#else	
		GPIO_SetSensorPwdn(power_down);
#endif

		Sensor_SetMCLK(SENSOR_DISABLE_MCLK);           

		Sensor_SetVoltage(SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED);   
				SCI_Sleep(20);
    }
    

    return SCI_SUCCESS;
}

LOCAL uint32 hmirror_enable(uint32 enable)
{
#ifdef DOUBLE_CAMERA_SUPPORT
uint32 i;
BOOLEAN is_page0 = FALSE;
for(i = 0; i < s_SP0838_resolution_Tab_YUV->reg_count; i++)
{
if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0xfd)
{
if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x00)
is_page0 = TRUE;
else if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x01)
is_page0 = FALSE;
}
if ((s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0x31) && is_page0)
{
if(enable)
{
s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value |= 0x20;
}
else
{
s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value &= ~0x20;
}
break;
}
}
return SCI_SUCCESS; 
#endif
}

LOCAL uint32 vmirror_enable(uint32 enable)
{
#ifdef DOUBLE_CAMERA_SUPPORT
uint32 i;
BOOLEAN is_page0 = FALSE;
for(i = 0; i < s_SP0838_resolution_Tab_YUV->reg_count; i++)
{
if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0xfd)
{
if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x00)
is_page0 = TRUE;
else if (s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x01)
is_page0 = FALSE;
}
if ((s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0x31) && is_page0)
{
if(enable)
{
s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value |= 0x40;
}
else
{
s_SP0838_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value &= ~0x40;
}
break;
}
}
return SCI_SUCCESS; 
#endif
}

#endif




