//2012-1113 QVGA subsample  ջ

/*------------------------------------------------------------------------------
 * (c) Copyright, Augusta Technology, Inc., 2006-present.
 * (c) Copyright, Augusta Technology USA, Inc., 2006-present.
 *
 * This software, document, web pages, or material (the "Work") is copyrighted
 * by its respective copyright owners.  The Work may be confidential and
 * proprietary.  The Work may be further protected by one or more patents and
 * be protected as a part of a trade secret package.
 *
 * No part of the Work may be copied, photocopied, reproduced, translated, or
 * reduced to any electronic medium or machine-readable form, in whole or in
 * part, without prior written consent of the copyright owner. Any other
 * reproduction in any form without the permission of the copyright owner is
 * prohibited.
 *
 * All Work are protected by the copyright laws of all relevant jurisdictions,
 * including protection under the United States copyright laws, and may not be
 * reproduced, distributed, transmitted, displayed, published, or broadcast
 * without the prior written permission of the copyright owner.
 *
 ------------------------------------------------------------------------------*/

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/i2c.h>
#include <linux/time.h>
#include <linux/atxx/camera.h>
#include <mach/gpio.h>
#include <mach/pmu.h>
#include <linux/clk.h>
#include <linux/cam.h>
#include <linux/cam_core.h>

#define ATXX_CAM_ID     0x27
#define ATXX_CAM_BUS    3
#define ATXX_CAM_ADDR   0x18
#define ATXX_CAM_NAME   "sp0838"
#define ATXX_CAM_SUBDEV ATXX_SUBDEV_REAR_CAM

static int flip_val = 0;

/***********************************************************************
 * Camera Common Function                                              *
 ***********************************************************************/

struct cam_reg {
	unsigned char reg;
	unsigned char val;
};

static int write_regs(struct cam_info *info, const struct cam_reg reglist[])
{
	const struct cam_reg *next = reglist;

	while (!((next->reg == REG_TERM) && (next->val == VAL_TERM)))
	{
		cam_write_byte(info->i2c_dev, next->reg, next->val);
		next++;
	}

	return 0;
}

/***********************************************************************
 * Camera Flip Function                                                *
 ***********************************************************************/

static struct cam_reg cam_reg_flipnone[] = {
	{0xfd, 0x00},
	{0x31, 0x10},

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_reg_hflip[] = {
	{0xfd, 0x00},
	{0x31, 0x30},

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_reg_vflip[] = {
	{0xfd, 0x00},
	{0x31, 0x50},

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_reg_hvflip[] = {
	{0xfd, 0x00},
	{0x31, 0x70},

	{REG_TERM, VAL_TERM}
};

struct cam_reg* cam_reg_flip[4] =
{
	cam_reg_flipnone,
	cam_reg_hflip,
	cam_reg_vflip,
	cam_reg_hvflip
};

static int cam_flip(struct cam_info *info, int val)
{
	return write_regs(info, cam_reg_flip[val]);
}
	

/***********************************************************************
 * Camera Initialize Function                                          *
 ***********************************************************************/

static struct cam_reg cam_set_initialize[] = 
{
	
	{0xfd , 0x00},// //P0
	{0x1B , 0x02},//
	{0x27 , 0xe8},//
	{0x28 , 0x0B},//
	{0x32 , 0x00},//
	{0x22 , 0xc0},//
	{0x26 , 0x10},// 
	{0x31 , 0x70},//   //Upside/mirr/Pclk inv/sub
	{0x5f , 0x11},//   //Bayer order
	{0xfd , 0x01},//   //P1
	{0x25 , 0x1a},//   //Awb start
	{0x26 , 0xfb},// 
	{0x28 , 0x75},// 
	{0x29 , 0x4e},//
	{0xfd , 0x00},//   
	{0xe7 , 0x03},// 
	{0xe7 , 0x00},// 
	{0xfd , 0x01},//
	{0x31 , 0x60},////64
	{0x32 , 0x18},//
	{0x4d , 0xdc},//
	{0x4e , 0x53},//
	{0x41 , 0x8c},//
	{0x42 , 0x57},//
	{0x55 , 0xff},//
	{0x56 , 0x00},//
	{0x59 , 0x82},//
	{0x5a , 0x00},//
	{0x5d , 0xff},//
	{0x5e , 0x6f},//
	{0x57 , 0xff},//
	{0x58 , 0x00},//
	{0x5b , 0xff},//
	{0x5c , 0xa8},//
	{0x5f , 0x75},//
	{0x60 , 0x00},//
	{0x2d , 0x00},//
	{0x2e , 0x00},//
	{0x2f , 0x00},//
	{0x30 , 0x00},//
	{0x33 , 0x00},//
	{0x34 , 0x00},//
	{0x37 , 0x00},//
	{0x38 , 0x00},//  //awb end
	{0xfd , 0x00},//  //P0
	{0x33 , 0x6f},//  //LSC BPC EN
	{0x51 , 0x3f},//  //BPC debug start
	{0x52 , 0x09},//  
	{0x53 , 0x00},//  
	{0x54 , 0x00},//
	{0x55 , 0x10},//  //BPC debug end
	{0x4f , 0x08},//  //blueedge
	{0x50 , 0x08},//
	{0x57 , 0x10},//  //Raw filter debut start
	{0x58 , 0x10},//
	{0x59 , 0x10},//
	{0x56 , 0x70},//
	{0x5a , 0x02},//
	{0x5b , 0x02},//
	{0x5c , 0x20},//  //Raw filter debut end 
	{0x65 , 0x03},//  //Sharpness debug start
	{0x66 , 0x01},//
	{0x67 , 0x03},//
	{0x68 , 0x46},//
	{0x69 , 0x7f},//
	{0x6a , 0x01},//
	{0x6b , 0x04},//
	{0x6c , 0x01},//
	{0x6d , 0x03},//  //Edge gain normal
	{0x6e , 0x46},//  //Edge gain normal
	{0x6f , 0x7f},//
	{0x70 , 0x01},//
	{0x71 , 0x05},// //ֵ 		
	{0x72 , 0x01},// //ֵ		
	{0x73 , 0x03},// //Եֵ	
	{0x74 , 0x46},// //Եֵ	
	{0x75 , 0x7f},// //ʹλ			
	{0x76 , 0x01},//  //Sharpness debug end
	{0xcb , 0x07},//  //HEQ&Saturation debug start 
	{0xcc , 0x04},//
	{0xce , 0xff},//
	{0xcf , 0x10},//
	{0xd0 , 0x20},//
	{0xd1 , 0x00},//
	{0xd2 , 0x1c},//
	{0xd3 , 0x16},//
	{0xd4 , 0x00},//
	{0xd6 , 0x1c},//
	{0xd7 , 0x16},//
	{0xdd , 0x70},//  //Contrast
	{0xde , 0x90},//  //HEQ&Saturation debug end
	{0x7f , 0xd7},//  //Color Correction start
	{0x80 , 0xbc},//						  
	{0x81 , 0xed},//						  
	{0x82 , 0xd7},//						  
	{0x83 , 0xd4},//						  
	{0x84 , 0xd6},//						  
	{0x85 , 0xff},//						  
	{0x86 , 0x89},//						  
	{0x87 , 0xf8},//						  
	{0x88 , 0x3c},//						  
	{0x89 , 0x33},//						  
	{0x8a , 0x0f},//  //Color Correction end 
	{0x8b , 0x00},//  //gamma start
	{0x8c , 0x1a},//			   
	{0x8d , 0x29},//			   
	{0x8e , 0x41},//			   
	{0x8f , 0x62},//			   
	{0x90 , 0x7c},//			   
	{0x91 , 0x90},//			   
	{0x92 , 0xa2},//			   
	{0x93 , 0xaf},//			   
	{0x94 , 0xbc},//			   
	{0x95 , 0xc5},//			   
	{0x96 , 0xcd},//			   
	{0x97 , 0xd5},//			   
	{0x98 , 0xdd},//			   
	{0x99 , 0xe5},//			   
	{0x9a , 0xed},//			   
	{0x9b , 0xf5},//			   
	{0xfd , 0x01},//  //P1		   
	{0x8d , 0xfd},//			   
	{0x8e , 0xff},//  //gamma end  
	{0xfd , 0x00},//  //P0
	{0xca , 0xcf},//
	{0xd8 , 0x48},//  //UV outdoor
	{0xd9 , 0x48},//  //UV indoor 
	{0xda , 0x48},//  //UV dummy
	{0xdb , 0x48},//  //UV lowlight
	{0xb9 , 0x00},//  //Ygamma start
	{0xba , 0x04},//
	{0xbb , 0x08},//
	{0xbc , 0x10},//
	{0xbd , 0x20},//
	{0xbe , 0x30},//
	{0xbf , 0x40},//
	{0xc0 , 0x50},//
	{0xc1 , 0x60},//
	{0xc2 , 0x70},//
	{0xc3 , 0x80},//
	{0xc4 , 0x90},//
	{0xc5 , 0xA0},//
	{0xc6 , 0xB0},//
	{0xc7 , 0xC0},//
	{0xc8 , 0xD0},//
	{0xc9 , 0xE0},//
	{0xfd , 0x01},//  //P1
	{0x89 , 0xf0},//
	{0x8a , 0xff},//  //Ygamma end
	{0xfd , 0x00},//  //P0
	{0xe8 , 0x30},//  //AEdebug start
	{0xe9 , 0x30},//
	{0xea , 0x40},//  //Alc Window sel
	{0xf4 , 0x1b},//  //outdoor mode sel
	{0xf5 , 0x80},//
	{0xf7 , 0x78},//  //AE target
	{0xf8 , 0x63},//  
	{0xf9 , 0x68},//  //AE target
	{0xfa , 0x53},//
	{0xfd , 0x01},//  //P1
	{0x09 , 0x31},//  //AE Step 3.0
	{0x0a , 0x85},//
	{0x0b , 0x0b},//  //AE Step 3.0
	{0x14 , 0x20},//
	{0x15 , 0x0f},//
	{0xfd , 0x00},//  //P0
	{0x05 , 0x00},//  
	{0x06 , 0x00},//
	{0x09 , 0x01},//
	{0x0a , 0x76},//
	{0xf0 , 0x62},//
	{0xf1 , 0x00},//
	{0xf2 , 0x5f},//
	{0xf5 , 0x78},//
	{0xfd , 0x01},//  //P1
	{0x00 , 0xba},//
	{0x0f , 0x60},//
	{0x16 , 0x60},//
	{0x17 , 0xa2},//
	{0x18 , 0xaa},//
	{0x1b , 0x60},//
	{0x1c , 0xaa},//
	{0xb4 , 0x20},//
	{0xb5 , 0x3a},//
	{0xb6 , 0x5e},//
	{0xb9 , 0x40},//
	{0xba , 0x4f},//
	{0xbb , 0x47},//
	{0xbc , 0x45},//
	{0xbd , 0x43},//
	{0xbe , 0x42},//
	{0xbf , 0x42},//
	{0xc0 , 0x42},//
	{0xc1 , 0x41},//
	{0xc2 , 0x41},//
	{0xc3 , 0x41},//
	{0xc4 , 0x41},//
	{0xc5 , 0x78},//
	{0xc6 , 0x41},//
	{0xca , 0x78},//
	{0xcb , 0x0c},//  //AEdebug end
	{0xfd , 0x00},//
	{0x05 , 0x0 },//
	{0x06 , 0x0 },//
	{0x09 , 0x1 },//
	{0x0a , 0x76},//
	{0xf0 , 0x62},//
	{0xf1 , 0x0 },//
	{0xf2 , 0x5f},//
	{0xf5 , 0x78},//
	{0xfd , 0x01},//
	{0x00 , 0xb2},//
	{0x0f , 0x60},//
	{0x16 , 0x60},//
	{0x17 , 0xa2},//
	{0x18 , 0xaa},//
	{0x1b , 0x60},//
	{0x1c , 0xaa},//
	{0xb4 , 0x20},//
	{0xb5 , 0x3a},//
	{0xb6 , 0x5e},//
	{0xb9 , 0x40},//
	{0xba , 0x4f},//
	{0xbb , 0x47},//
	{0xbc , 0x45},//
	{0xbd , 0x43},//
	{0xbe , 0x42},//
	{0xbf , 0x42},//
	{0xc0 , 0x42},//
	{0xc1 , 0x41},//
	{0xc2 , 0x41},//
	{0xc3 , 0x41},//
	{0xc4 , 0x41},//
	{0xc5 , 0x70},//
	{0xc6 , 0x41},//
	{0xca , 0x70},//
	{0xcb , 0xc },//
	{0xfd , 0x00},//
	{0xfd , 0x00},//  //P0
	{0x32 , 0x15},//  //Auto_mode set
	{0x34 , 0x66},//  //Isp_mode set
	{0x35 , 0x40},//  //out format
	{REG_TERM, VAL_TERM}
	/*[END]*/
};


static struct cam_reg cam_set_preview[] = {

	#if 1
	{0xfd, 0x00},

	{0x47, 0x00},
	{0x48, 0x00},
	{0x49, 0x01},
	{0x4a, 0xe0},

	{0x4b, 0x00},
	{0x4c, 0x00},
	{0x4d, 0x02},
	{0x4e, 0x80},

	{0xfd, 0x00},
	{0x30, 0x00},
	{0x31, 0x70},
	//{0xfd, 0x01},
	//{0x0e, 0x00},

#endif
	//sensor AE settings:
	
					
	  {0xfd , 0x00},
	  {0x05 , 0x00},
	  {0x06 , 0x64},
	  {0x07 , 0x00},
	  {0x08 , 0x64},
	  {0x09 , 0x01},
	  {0x0a , 0xf2},
	  {0xf0 , 0x59},
	  {0xf1 , 0x00},
	  {0xf2 , 0x5e},
	  {0xf5 , 0x77},
	  {0xfd , 0x01},
	  {0x00 , 0xb1},
	  {0x0f , 0x5f},
	  {0x16 , 0x5f},
	  {0x17 , 0xa1},
	  {0x18 , 0xa9},
	  {0x1b , 0x5f},
	  {0x1c , 0xa9},
	  {0xb4 , 0x21},
	  {0xb5 , 0x3d},
	  {0xb6 , 0x55},
	  {0xb9 , 0x40},
	  {0xba , 0x4f},
	  {0xbb , 0x47},
	  {0xbc , 0x45},
	  {0xbd , 0x43},
	  {0xbe , 0x42},
	  {0xbf , 0x42},
	  {0xc0 , 0x42},
	  {0xc1 , 0x41},
	  {0xc2 , 0x41},
	  {0xc3 , 0x41},
	  {0xc4 , 0x41},
	  {0xc5 , 0x70},
	  {0xc6 , 0x41},
	  {0xca , 0x70},
	  {0xcb , 0x0c},
	  {0xfd , 0x00},
	{0xe7 , 0x03}, 
	{0xe7 , 0x00},


	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_set_capture[] = {
#if 0
	{0xfd , 0x00},// //P0
	{0x31 , 0x00},//
	{0xfd, 0x00},

	{0x47, 0x00},
	{0x48, 0x00},
	{0x49, 0x01},
	{0x4a, 0xe0},

	{0x4b, 0x00},
	{0x4c, 0x00},
	{0x4d, 0x02},
	{0x4e, 0x80},

	{0xfd, 0x00},
	{0x30, 0x00},

	{0xfd, 0x01},
	{0x0e, 0x00},

#endif
	{REG_TERM, VAL_TERM}
};

struct cam_reg* cam_reg_init[3] =
{
	cam_set_initialize,
	cam_set_preview,
	cam_set_capture
};

static int cam_initialize(struct cam_info *info)
{
	return write_regs(info, cam_reg_init[0]);
}

static int cam_preview(struct cam_info *info)
{
	return write_regs(info, cam_reg_init[1]);
}

static int cam_capture(struct cam_info *info)
{
	return write_regs(info, cam_reg_init[2]);
}

/***********************************************************************
 * Camera Resolution Function                                          *
 ***********************************************************************/

static struct cam_reg cam_resolution_qcif[] = {
#if 1
	/*QCIF this is for resize para*/							 
	{0xfd, 0x00},
	{0x47, 0x00},
	{0x48, 0xa8},
	{0x49, 0x00},
	{0x4a, 0x90},
	{0x4b, 0x00},
	{0x4c, 0xe8},
	{0x4d, 0x00},
	{0x4e, 0xb0},
	{0xfd, 0x00},
	{0x30, 0x00},
	{0xfd, 0x01},
	{0x0e, 0x00},
	#endif
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_cif[] = {
	#if 1
	{0xfd, 0x00},

	{0x47, 0x00},
	{0x48, 0x90},
	{0x49, 0x01},
	{0x4a, 0x60},

	{0x4b, 0x00},
	{0x4c, 0x60},
	{0x4d, 0x01},
	{0x4e, 0x20},

	{0xfd, 0x00},
	{0x30, 0x00},
	
	{0xfd, 0x01},
	{0x0e, 0x00},
	#endif
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_qvga[] = {

#if 1//sub ²

//sensor AE settings:
{0xfd , 0x00},
{0x05 , 0x00},
{0x06 , 0x64},
{0x07 , 0x00},
{0x08 , 0x64},
{0x09 , 0x06},
{0x0a , 0xc6},
{0xf0 , 0x6a},
{0xf1 , 0x00},
{0xf2 , 0x61},
{0xf5 , 0x7a},
{0xfd , 0x01},
{0x00 , 0xb4},
{0x0f , 0x62},
{0x16 , 0x62},
{0x17 , 0xa4},
{0x18 , 0xac},
{0x1b , 0x62},
{0x1c , 0xac},
{0xb4 , 0x20},
{0xb5 , 0x3a},
{0xb6 , 0x66},
{0xb9 , 0x40},
{0xba , 0x4f},
{0xbb , 0x47},
{0xbc , 0x45},
{0xbd , 0x43},
{0xbe , 0x42},
{0xbf , 0x42},
{0xc0 , 0x42},
{0xc1 , 0x41},
{0xc2 , 0x41},
{0xc3 , 0x41},
{0xc4 , 0x41},
{0xc5 , 0x70},
{0xc6 , 0x41},
{0xca , 0x70},
{0xcb , 0x0c},
{0xfd , 0x00},
{0xe7 , 0x03}, 
{0xe7 , 0x00},
//set windows
{0xfd , 0x00},  
{0x47 , 0x00},  
{0x48 , 0x00},  
{0x49 , 0x01},  
{0x4a , 0xe0},  
{0x4b , 0x00},  
{0x4c , 0x00},  
{0x4d , 0x02},  
{0x4e , 0x80}, 
{0x31 , 0x74},
{0xe7 , 0x03}, 
{0xe7 , 0x00},
//{0x0a , 0xa0},  
//{0xf0 , 0xb2},  
//{0xf1 , 0x01},  
//{0xfd , 0x00},  
#endif
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_vga[] = {
	#if 1
	{0xfd, 0x00},

	{0x47, 0x00},
	{0x48, 0x00},
	{0x49, 0x01},
	{0x4a, 0xe0},

	{0x4b, 0x00},
	{0x4c, 0x00},
	{0x4d, 0x02},
	{0x4e, 0x80},

	{0xfd, 0x00},
	{0x30, 0x00},
	{0x31, 0x70},

	//{0xfd, 0x01},
	//{0x0e, 0x00},
	//sensor AE settings:
				  
	{0xfd , 0x00},
	{0x05 , 0x00},
	{0x06 , 0x64},
	  {0x07 , 0x00},
	  {0x08 , 0x64},
	{0x09 , 0x01},
	{0x0a , 0xf2},
	{0xf0 , 0x59},
	{0xf1 , 0x00},
	{0xf2 , 0x5e},
	{0xf5 , 0x77},
	{0xfd , 0x01},
	{0x00 , 0xb1},
	{0x0f , 0x5f},
	{0x16 , 0x5f},
	{0x17 , 0xa1},
	{0x18 , 0xa9},
	{0x1b , 0x5f},
	{0x1c , 0xa9},
	{0xb4 , 0x21},
	{0xb5 , 0x3d},
	{0xb6 , 0x55},
	{0xb9 , 0x40},
	{0xba , 0x4f},
	{0xbb , 0x47},
	{0xbc , 0x45},
	{0xbd , 0x43},
	{0xbe , 0x42},
	{0xbf , 0x42},
	{0xc0 , 0x42},
	{0xc1 , 0x41},
	{0xc2 , 0x41},
	{0xc3 , 0x41},
	{0xc4 , 0x41},
	{0xc5 , 0x70},
	{0xc6 , 0x41},
	{0xca , 0x70},
	{0xcb , 0x0c},
	{0xfd , 0x00},
#endif
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_svga[] = {
	/*SVGA this is for resize para*/							 			
	
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_xga[] = {
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_sxga[] = {
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_uxga[] = {

	/*1600x1200 this is for resize para*/
	
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_qxga[] = {
	{REG_TERM, VAL_TERM}
};

struct cam_reg* cam_reg_resolution[RES_FLAG_MAX] =
{
	cam_resolution_qcif,
	cam_resolution_cif,
	cam_resolution_qvga,
	cam_resolution_vga,
	cam_resolution_svga,
	cam_resolution_xga,
	cam_resolution_sxga,
	cam_resolution_uxga,
	cam_resolution_qxga
};

static int cam_resolution(struct cam_info *info, int val)
{
	switch (val) {
	case RES_FLAG_QCIF:
		////write_regs(info, cam_reg_resolution[RES_FLAG_VGA]);	//del by sp_yjp,20120802
		write_regs(info, cam_reg_resolution[RES_FLAG_QCIF]);
		msleep(200);
		info->res_flags = RES_FLAG_QCIF;
		break;

	case RES_FLAG_CIF:
		write_regs(info, cam_reg_resolution[RES_FLAG_CIF]);
		info->res_flags = RES_FLAG_CIF;
		break;

	case RES_FLAG_QVGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_QVGA]);
		info->res_flags = RES_FLAG_QVGA;
		break;

	case RES_FLAG_VGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_VGA]);
		msleep(500);
		info->res_flags = RES_FLAG_VGA;
		break;

	case RES_FLAG_SVGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_SVGA]);
		info->res_flags = RES_FLAG_SVGA;
		break;

	case RES_FLAG_XGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_XGA]);
		info->res_flags = RES_FLAG_XGA;
		break;

	case RES_FLAG_SXGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_SXGA]);
		info->res_flags = RES_FLAG_SXGA;
		break;

	case RES_FLAG_UXGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_UXGA]);
		info->res_flags = RES_FLAG_UXGA;
		break;

	case RES_FLAG_QXGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_QXGA]);
		info->res_flags = RES_FLAG_QXGA;
		break;

	default:
		return -EINVAL;
	}

	return 0;
}

/***********************************************************************
 * Camera WhiteBalance Function                                        *
 ***********************************************************************/

static struct cam_reg cam_wb_auto[] = {
	//SP0A18_reg_WB_auto
	{0xfd, 0x01},                                                          
	{0x28, 0x75},		                                                       
	{0x29, 0x4e},
	{0xfd, 0x00},  // AUTO 3000K~7000K   	  
	{0x32, 0x15}, 

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_sunshine[] = {
	// SP0A18_reg_WB_auto   
	{0xfd, 0x00},  //6500K                                     
	{0x32, 0x05},                                                          
	{0xfd, 0x01},                                                          
	{0x28, 0x6b},		                                                       
	{0x29, 0x48},		                                                       
	{0xfd, 0x00},

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_cloudy[] = {
	// SP0A18_reg_WB_auto   
	{0xfd, 0x00},   //7000K                                     
	{0x32, 0x05},                                                          
	{0xfd, 0x01},                                                          
	{0x28, 0x71},		                                                       
	{0x29, 0x41},		                                                       
	{0xfd, 0x00},

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_fluorescence[] = {
	//SP0A18_reg_WB_auto  ӫ 
	{0xfd, 0x00},  //4200K~5000K                                     
	{0x32, 0x05},                                                          
	{0xfd, 0x01},                                                          
	{0x28, 0x5a},		                                                       
	{0x29, 0x62},		                                                       
	{0xfd, 0x00},

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_incandescence[] = {
	// SP0A18_reg_WB_auto ׳ 
	{0xfd, 0x00},  //2800K~3000K                                     
	{0x32, 0x05},                                                          
	{0xfd, 0x01},                                                          
	{0x28, 0x41},		                                                       
	{0x29, 0x71},		                                                       
	{0xfd, 0x00}, 

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_warm_fluorescence[] = {
	// SP0A18_reg_WB_auto ȹ
	{0xfd, 0x00},  //4000K                                   
	{0x32, 0x05},                                                          
	{0xfd, 0x01},                                                          
	{0x28, 0x57},		                                                       
	{0x29, 0x66},		                                                       
	{0xfd, 0x00},

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_twilight[] = {
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_shade[] = {
	{REG_TERM, VAL_TERM}
};

struct cam_reg* cam_reg_wb[WB_MAX] =
{
	cam_wb_auto,
	cam_wb_sunshine,
	cam_wb_cloudy,
	cam_wb_fluorescence,
	cam_wb_incandescence,
	cam_wb_warm_fluorescence,
	cam_wb_twilight,
	cam_wb_shade
};

static int cam_wb(struct cam_info *info, int val)
{
	switch (val) {
	case WB_AUTO:
		write_regs(info, cam_reg_wb[WB_AUTO]);
		break;

	case WB_SUNSHINE:
		write_regs(info, cam_reg_wb[WB_SUNSHINE]);
		break;

	case WB_CLOUDY:
		write_regs(info, cam_reg_wb[WB_CLOUDY]);
		break;

	case WB_FLUORESCENCE:
		write_regs(info, cam_reg_wb[WB_FLUORESCENCE]);
		break;

	case WB_INCANDESCENCE:
		write_regs(info, cam_reg_wb[WB_INCANDESCENCE]);
		break;

	case WB_WARM_FLUORESCENCE:
		write_regs(info, cam_reg_wb[WB_WARM_FLUORESCENCE]);
		break;

	case WB_TWILIGHT:
		write_regs(info, cam_reg_wb[WB_TWILIGHT]);
		break;

	case WB_SHADE:
		write_regs(info, cam_reg_wb[WB_SHADE]);
		break;

	default:
		return -EINVAL;
	}

	return 0;
}

/***********************************************************************
 * Camera Flash Function                                               *
 ***********************************************************************/
#if defined(CONFIG_MACH_Y837)
static int cam_setflash(struct cam_info *info, int val)
{
	switch (val) {
	case FLASH_ON:
		gpio_direction_output(GPIO_CAM_FLASH_EN, 1);
		
		cam_notice("[1103-------------------SP0838----FLASH_ON]----val=%d\n",val);
		break;

	case FLASH_OFF:
		gpio_direction_output(GPIO_CAM_FLASH_EN, 0);
		
		cam_notice("[1103-------------------SP0838----FLASH_OFF]----val=%d\n",val);
		break;

	default:
		cam_err("sensor %s setflash mode error!\n", ATXX_CAM_NAME);
		return -EINVAL;
	}

	return 0;
}
#endif

#if 1
/***********************************************************************
 * Camera Power Control Function                                       *
 ***********************************************************************/

static int cam_gpio_request(void)
{
	gpio_request(GPIO_CAM_PWDN_B, "cam_pwdn_b");
	gpio_request(GPIO_CAM_PWDN_F, "cam_pwdn_f");
	gpio_request(GPIO_CAM_RST_F, "cam_rst_f");
#if defined(CONFIG_MACH_Y837)
		gpio_request(GPIO_CAM_FLASH_EN, "cam_flash_en");
#endif

	return 0;
}

static int cam_gpio_free(void)
{
	gpio_free(GPIO_CAM_PWDN_B);
	gpio_free(GPIO_CAM_PWDN_F);
	gpio_free(GPIO_CAM_RST_F);
#if defined(CONFIG_MACH_Y837)
		gpio_free(GPIO_CAM_FLASH_EN);
#endif

	return 0;
}

static void cam_poweron(void)
{
	pmu_power_set(PPS_CAMERA_IO, 2800);
	msleep(10);
	//pmu_power_set(PPS_CAMERA_CORE, 2800);	
//mu_power_set(PPS_CAMERA_CORE, 1800);//xjt add test 20121102
//sleep(10);	
	cam_notice("[1108-------xx----------SP0838----cam_poweron]");

	
}

static void cam_poweroff(void)
{
	//u_power_set(PPS_CAMERA_CORE,1800);
	//msleep(5);	
	//pmu_power_set(PPS_CAMERA_IO, 2800);//add sp_wyq 20121108
 //	msleep(5);	
cam_notice("[1108---------yy--------SP0838----cam_poweroff]");

}

static void cam_gpio_reset(void)
{
	gpio_direction_output(GPIO_CAM_PWDN_B, 0);
	msleep(50);
	gpio_direction_output(GPIO_CAM_PWDN_B, 1);
	msleep(50);
	gpio_direction_output(GPIO_CAM_PWDN_B, 0);
	msleep(50);
	
	gpio_direction_output(GPIO_CAM_RST_B, 1);
	msleep(50);
	gpio_direction_output(GPIO_CAM_RST_B, 0);
	msleep(50);
	gpio_direction_output(GPIO_CAM_RST_B, 1);
	msleep(50);
	cam_notice("[1103-------------------SP0838----cam_gpio_reset]");
	
}


static void cam_standby_mode(void)
{
	gpio_direction_output(GPIO_CAM_PWDN_B, 1);
	msleep(5);

	
	cam_notice("[1103-------------------SP0838----cam_standby_mode]");
}

static void cam_gpio_preset(void)
{
	gpio_direction_output(GPIO_CAM_PWDN_B, 1);
	msleep(15);
	gpio_direction_output(GPIO_CAM_PWDN_F, 1);
	msleep(15);
	gpio_direction_output(GPIO_CAM_RST_F, 0);
	msleep(15);

	
	cam_notice("[1103-------------------SP0838----cam_gpio_preset]");
}

static int detect_poweron(struct cam_info *info)
{
	cam_gpio_preset();
	cam_poweron();
	cam_gpio_reset();

	return 0;
}

static int detect_poweroff(struct cam_info *info)
{
	/* after detection finished, poweroff */
	cam_standby_mode();
	//m_poweroff();

	return 0;
}

static int detect_readid(struct cam_info *info)
{
	unsigned int cam_id;

	cam_id = cam_read_byte(info->i2c_dev, 0x02);
	cam_notice("read %s id = 0x%x\n", ATXX_CAM_NAME, cam_id);

	if (cam_id == ATXX_CAM_ID) {
		cam_notice("camera %s detected...\n", ATXX_CAM_NAME);
		return 0;
	}

	return -ENODEV;
}

static int cam_config(struct cam_info *info)
{
	info->minwidth  = QCIF_WIDTH;
	info->minheight = QCIF_HEIGHT;
	info->maxwidth	= VGA_WIDTH;
	info->maxheight	= VGA_HEIGHT;

	info->bl  = BAND_50;
	info->fps = 25;
	info->night_mode = NIGHT_MODE_OFF;

	return 0;
}

static int cam_open(struct cam_info *info)
{
	clk_enable(info->camera);
	msleep(10);

	cam_gpio_preset();
	cam_poweron();
	cam_gpio_reset();

	cam_initialize(info);
	cam_flip(info, flip_val);

	info->width  = VGA_WIDTH;
	info->height = VGA_HEIGHT;
	info->res_flags = RES_FLAG_MAX;	
	cam_notice("[1103-------------------SP0838----cam_open]");

	return 0;
}

static int cam_close(struct cam_info *info)
{
	cam_standby_mode();
	cam_poweroff();

	clk_disable(info->camera); 
	cam_notice("[1103-------------------SP0838----cam_close]");

	return 0;
}
#endif
/***********************************************************************
 * Camera Function List                                                *
 ***********************************************************************/

static struct camera_fmt cam_fmt_list[] = {
	{
		.name	= "4:2:2 planar, Y-Cb-Cr",
		.fourcc	= V4L2_PIX_FMT_YUV422P,
		.depth	= 16,
	}
};

#define N_FORMATS ARRAY_SIZE(cam_fmt_list)

static struct camera_res cam_res_list[] = {
	{
		.name   = "176x144, QCIF",
		.width  = QCIF_WIDTH,
		.height = QCIF_HEIGHT
	},
	{
		.name   = "352x288, CIF",
		.width  = CIF_WIDTH,
		.height = CIF_HEIGHT
	}, 
	{
		.name   = "320x240, QVGA",
		.width  = QVGA_WIDTH,
		.height = QVGA_HEIGHT
	}, {
		.name   = "640x480, VGA",
		.width  = VGA_WIDTH,
		.height = VGA_HEIGHT
	}
};

#define N_RESOLUTIONS ARRAY_SIZE(cam_res_list)

static struct camera_control cam_ctl_list[] = {
	{
		.qc = {
			.id      = V4L2_CID_DO_WHITE_BALANCE,
			.type    = V4L2_CTRL_TYPE_INTEGER,
			.name    = "WhiteBalance",
			.minimum = WB_AUTO,
			.maximum = WB_INCANDESCENCE,
			.step    = 1,
			.default_value = WB_AUTO,
		},
		.cur_value   = WB_AUTO,
		.tweak       = cam_wb,
	
#if defined(CONFIG_MACH_Y837)
				}, {
					.qc = {
						.id 	 = V4L2_CID_PRIVATE_ATXX_FLASH,
						.type	 = V4L2_CTRL_TYPE_INTEGER,
						.name	 = "flash",
						.minimum = FLASH_OFF,
						.maximum = FLASH_ON,
						.step	 = 1,
						.default_value = FLASH_OFF,
					},
					.cur_value	 = FLASH_OFF,
					.tweak		 = cam_setflash,
#endif
	}
};

#define N_CONTROLS ARRAY_SIZE(cam_ctl_list)

static const struct i2c_device_id cam_id_table[] = {
	{ ATXX_CAM_NAME,  0 },
	{ }
};
MODULE_DEVICE_TABLE(i2c, cam_id_table);//xjt 20121107

/***********************************************************************
 * Camera Priv Init                                                    *
 ***********************************************************************/

static struct cam_priv priv = {
	.name       = ATXX_CAM_NAME,
	.addr       = ATXX_CAM_ADDR,
	.i2c_bus    = ATXX_CAM_BUS,
	.subdev_id  = ATXX_CAM_SUBDEV,

	.fmt_num    = N_FORMATS,
	.res_num    = N_RESOLUTIONS,
	.ctl_num    = N_CONTROLS,

	.fmt_list   = cam_fmt_list,
	.res_list   = cam_res_list,
	.ctl_list   = cam_ctl_list,
	.id_table   = cam_id_table,

	.Open       = cam_open,
	.Close      = cam_close,

	.Set_Preview     = cam_preview,
	.Set_Capture     = cam_capture,
	.Set_Config      = cam_config,
	.Set_Resolution  = cam_resolution,

	.Request_Gpio    = cam_gpio_request,
	.Free_Gpio       = cam_gpio_free,

	.Detect_PowerON  = detect_poweron,
	.Detect_PowerOFF = detect_poweroff,
	.Detect_ReadId   = detect_readid,
};

static int __init cam_init(void)
{
	init_atxx_cam(&priv);
	cam_notice("[1103-------i2c addr = 0x18---------SP0838----cam_init]");

	return 0;
}

static void __exit cam_exit(void)
{
	cam_notice("[1103-------------------SP0838----cam_exit]");

	exit_atxx_cam(&priv);
}

module_param(flip_val, int , 0);
MODULE_PARM_DESC(flip_val, "sp0838 flip value.");

module_init(cam_init);
module_exit(cam_exit);

MODULE_DESCRIPTION("sp0838 camera Module");
MODULE_AUTHOR("weiyu Wei");
MODULE_LICENSE("Dual BSD/GPL");



#if 0	//for effect & brightness param
//Effect
 switch (Para)
  {
    case MEFFECT_OFF:
		SP2518WriteCmosSensor(0xfd, 0x00);
		SP2518WriteCmosSensor(0x62, 0x00);
		SP2518WriteCmosSensor(0x63, 0x80);
		SP2518WriteCmosSensor(0x64, 0x80);
      break;
    case MEFFECT_SEPIA:
		SP2518WriteCmosSensor(0xfd, 0x00);
		SP2518WriteCmosSensor(0x62, 0x10);
		SP2518WriteCmosSensor(0x63, 0xb0);
		SP2518WriteCmosSensor(0x64, 0x40);
      break;
    case MEFFECT_NEGATIVE://----datasheet
		SP2518WriteCmosSensor(0xfd, 0x00);
		SP2518WriteCmosSensor(0x62, 0x04);
		SP2518WriteCmosSensor(0x63, 0x80);
		SP2518WriteCmosSensor(0x64, 0x80);
      break;
    case MEFFECT_SEPIAGREEN://----datasheet aqua
		SP2518WriteCmosSensor(0xfd, 0x00);
		SP2518WriteCmosSensor(0x62, 0x10);
		SP2518WriteCmosSensor(0x63, 0x50);
		SP2518WriteCmosSensor(0x64, 0x50);
      break;
    case MEFFECT_SEPIABLUE:
		SP2518WriteCmosSensor(0xfd, 0x00);
		SP2518WriteCmosSensor(0x62, 0x10);
		SP2518WriteCmosSensor(0x63, 0x80);
		SP2518WriteCmosSensor(0x64, 0xb0);
      break;
    case MEFFECT_MONO: //----datasheet black & white
		SP2518WriteCmosSensor(0xfd, 0x00);
		SP2518WriteCmosSensor(0x62, 0x20);
		SP2518WriteCmosSensor(0x63, 0x80);
		SP2518WriteCmosSensor(0x64, 0x80);
      break;
    default:
      return KAL_FALSE;
  }


 //Brightness
   switch (Para)
  {
    case AE_EV_COMP_n13:              /* EV -2 */
		SP2518WriteCmosSensor(0xfd,0x00);
		SP2518WriteCmosSensor(0xdc,0xc0);
      break;
    case AE_EV_COMP_n10:              /* EV -1.5 */
		SP2518WriteCmosSensor(0xfd,0x00);
		SP2518WriteCmosSensor(0xdc,0xd0);
      break;
    case AE_EV_COMP_n07:              /* EV -1 */
		SP2518WriteCmosSensor(0xfd,0x00);
		SP2518WriteCmosSensor(0xdc,0xe0);
      break;
    case AE_EV_COMP_n03:              /* EV -0.5 */
		SP2518WriteCmosSensor(0xfd,0x00);
		SP2518WriteCmosSensor(0xdc,0xf0);
      break;
    case AE_EV_COMP_00:                /* EV 0 */
		SP2518WriteCmosSensor(0xfd,0x00);
		SP2518WriteCmosSensor(0xdc,0x00);
      break;
    case AE_EV_COMP_03:              /* EV +0.5 */
		SP2518WriteCmosSensor(0xfd,0x00);
		SP2518WriteCmosSensor(0xdc,0x10);
      break;
    case AE_EV_COMP_07:              /* EV +1 */
		SP2518WriteCmosSensor(0xfd,0x00);
		SP2518WriteCmosSensor(0xdc,0x20);
      break;
    case AE_EV_COMP_10:              /* EV +1.5 */
		SP2518WriteCmosSensor(0xfd,0x00);
		SP2518WriteCmosSensor(0xdc,0x30);
      break;
    case AE_EV_COMP_13:              /* EV +2 */
		SP2518WriteCmosSensor(0xfd,0x00);
		SP2518WriteCmosSensor(0xdc,0x40);
      break;
    default:
      return KAL_FALSE;
  }
#endif
