/*
********************************************************************************
*                   213x --- sensor's driver
*                (c) Copyright 2007 - 2010, Actions Co,Ld. 
*                        All Right Reserved 
*
* History:        
*      <author>      <time>       <version >    <desc>
*       houjingkun      2008/05/30     1.0      build file. 
*                                               this file is used for ov9655.
*       houjingkun      2008/06/18     1.1      modify for code checking.
*       houjingkun      2008/08/05     1.2      change name from sensor_param.c 
*                                               to sensor_module.c
*       houjingkun      2008/11/11     1.3      modified for sp0838. 
********************************************************************************/ 
/*!
 * \file   sensor_module.c
 * \brief  
 *      Sensor module's param setting function.
 * \author houjingkun
 * \par GENERAL DESCRIPTION:
 *      Sensor module's param setting function.
 *      it's internal API, just called by dirver itself.
 *      it include initializing function, and can adjust image's effect, 
 *      such as white_balance, brightness ...
 *      if sensor module is changed, please just change this file.
 * \par EXTERNALIZED FUNCTIONS:
 *       null
 *
 *     Copyright(c) 2008-2012 Actions Semiconductor, All Rights Reserved.
 *
 * \version 1.0
 * \date  2008/06/18
 *******************************************************************************/
#include "vi_drv.h"
#include "sensor_inner.h"
#include "vi_sys.h"
#include "sensor_hw.h"

/*config the power-down pin active level, if  high level enable power-down mode,define the first*/
#define  PWDN_ACTIVE_HIGH
//#define  PWDN_ACTIVE_LOW

//#define  GPIO_B_RESET
//#define  RESET_PIN_NUM  15

#define GPIO_PWND_B
#define PWND_PIN_NUM 15//16
/*config reg CMU_BTCLK, the result M_clk is : 120 / (M_CLK_DIVIDE+1) Mhz*/
//#define  M_CLK_DIVIDE 0x00000004
#define  M_CLK_DIVIDE 0x00000020    /* fix to 24MHZ. */

/******************************************************************************/
/*!                    
* \par  Description:
*       sensor module in i2c bus's slave_addr.
*******************************************************************************/
unsigned int i2c_slave_addr,0x30;     /* gc0306 write address. */


/* sp0838; 30 million pixel. from superpix file source init code. */
static unsigned char sensor_i2cdata[] =

/* init code for galaxycore. */
{
	0xfd,0x00,
	0x1B,0x02,
	0x27,0xe8,
	0x28,0x0B,
	0x32,0x00,
	0x22,0xc0,
	0x26,0x10,	
	0x31,0x50,//bit6:up&down bit5:mirror   0x10	
	0x5f,0x11,
	0xfd,0x01,
	0x25,0x1a,
	0x26,0xfb,
	0x28,0x75,
	0x29,0x4e,
	0x31,0x60,
	0x32,0x18,
	0x4d,0xdc,
	0x4e,0x53,
	0x41,0x8c,
	0x42,0x57,
	0x55,0xff,
	0x56,0x00,
	0x59,0x82,
	0x5a,0x00,
	0x5d,0xff,
	0x5e,0x6f,
	0x57,0xff,
	0x58,0x00,
	0x5b,0xff,
	0x5c,0xa8,
	0x5f,0x75,
	0x60,0x00,
	0x2d,0x00,
	0x2e,0x00,
	0x2f,0x00,
	0x30,0x00,
	0x33,0x00,
	0x34,0x00,
	0x37,0x00,
	0x38,0x00,
	0xfd,0x00,
	0x33,0x6f,
	0x51,0x3f,
	0x52,0x09,
	0x53,0x00,
	0x54,0x00,
	0x55,0x10,
	0x4f,0x08,
	0x50,0x08,  
	0x57,0x10,   //0x40
	0x58,0x10,   //0x40
	0x59,0x10,
	0x56,0x70,
	0x5a,0x02,
	0x5b,0x02,
	
	0x5c,0x20,
	0x65,0x03,  //Sharpness debug start
	0x66,0x01,
	0x67,0x03,
	0x68,0x46,
	0x69,0x7f,
	
	0x6a,0x01,
	0x6b,0x04,
	0x6c,0x01,
	0x6d,0x03,  //Edge gain normal
	0x6e,0x46, //Edge gain normal
	0x6f,0x7f,
	
	0x70,0x01,
	0x71,0x05,
	0x72,0x01,
	0x73,0x03,
	0x74,0x46,
	0x75,0x7f,
	
	0x76,0x01,
	0xcb,0x07,
	0xcc,0x04,
	0xce,0xff,
	0xcf,0x10,
	0xd0,0x20,
	0xd1,0x00,
	0xd2,0x1c,
	0xd3,0x16,
	0xd4,0x00,
	0xd6,0x1c,
	0xd7,0x16,
	0xdd,0x70, 
	0xde,0x90,
	0x7f,0xd7,
	0x80,0xbc,
	0x81,0xed,
	0x82,0xd7,
	0x83,0xd4,
	0x84,0xd6,
	0x85,0xff,
	0x86,0x89,
	0x87,0xf8,
	0x88,0x3c,
	0x89,0x33,
	0x8a,0x0f,
	0x8b,0x00,
	0x8c,0x1a,
	0x8d,0x29,
	0x8e,0x41,
	0x8f,0x62,
	0x90,0x7c,
	0x91,0x90,
	0x92,0xa2,
	0x93,0xaf,
	0x94,0xbc,
	0x95,0xc5,
	0x96,0xcd,
	0x97,0xd5,
	0x98,0xdd,
	0x99,0xe5,
	0x9a,0xed,
	0x9b,0xf5,
	0xfd,0x01,
	0x8d,0xfd,
	0x8e,0xff,
	0xfd,0x00,
	0xca,0xcf,
	0xd8,0x48,//UV outdoor
	0xd9,0x48,//UV indoor 
	0xda,0x48,//UV dummy
	0xdb,0x48,//UV lowlight
	0xb9,0x00,
	0xba,0x04,
	0xbb,0x08,
	0xbc,0x10,
	0xbd,0x20,
	0xbe,0x30,
	0xbf,0x40,
	0xc0,0x50,
	0xc1,0x60,
	0xc2,0x70,
	0xc3,0x80,
	0xc4,0x90,
	0xc5,0xA0,
	0xc6,0xB0,
	0xc7,0xC0,
	0xc8,0xD0,
	0xc9,0xE0,
	0xfd,0x01,
	0x89,0xf0,
	0x8a,0xff,
	0xfd,0x00,
	0xe8,0x30,
	0xe9,0x30,
	0xea,0x40,
	0xf4,0x1b,
	0xf5,0x80,
	0xf7,0x78,  //AE target 
	0xf8,0x63,  
	0xf9,0x68,  //AE target 
	0xfa,0x53,
	0xfd,0x01,
	0x09,0x31,
	0x0a,0x85,
	0x0b,0x0b,
	0x14,0x20,
	0x15,0x0f,
	
	0xfd,0x00,
	0x05,0x00,
	0x06,0x00,
	0x09,0x00,
	0x0a,0xaa,
	0xf0,0x75,
	0xf1,0x00,
	0xf2,0x65,
	0xf5,0x7e,
	0xfd,0x01,
	0x00,0xa3,
	0x0f,0x66,
	0x16,0x66,
	0x17,0x93,
	0x18,0x9b,
	0x1b,0x66,
	0x1c,0x9b,
	0xb4,0x21,
	0xb5,0x3d,
	0xb6,0x71,
	0xb9,0x40,
	0xba,0x4f,
	0xbb,0x47,
	0xbc,0x45,
	0xbd,0x70,
	0xbe,0x42,
	0xbf,0x42,
	0xc0,0x42,
	0xc1,0x41,
	0xc2,0x41,
	0xc3,0x41,
	0xc4,0x41,
	0xc5,0x41,
	0xc6,0x41,
	0xca,0x70,
	0xcb,0x04,
	0xfd,0x00,
	
	0xfd,0x00,
	0x32,0x15,	//
	0x34,0x66,
	0x35,0x40,     //0x40  
};


static void i2c_transmit_byte(sensor_i2cinfo_t * sensor_i2cinfo)
{
    unsigned int value;
    unsigned int count;
//    unsigned int pin0, pin1;

    count,0;
    do 
    {
        value,act_readl(I2CSTA);
        value &= 0x80;
        count++;
        if (count > 1800) 
        {
//            ss_printf("too long time not quit while act_readl(I2CSTA)!\n");
//            get_multip_reg(&pin0, &pin1, 2);
//            ss_printf("pin0: %x  pin1:%x \n", pin0, pin1);
            break;
        }
    } 
    while (!value);   /* wait for transmit complete! */

#if 1
    count,0;
    do 
    {
        value,act_readl(I2CCTL);
        value &= 0x1;
        count++;
        if (count > 1800) 
        {
//            ss_printf("too long time not quit while act_readl(I2CCTL)!\n");
//            get_multip_reg(&pin0, &pin1, 2);
//            ss_printf("pin0: %x  pin1:%x \n", pin0, pin1);
            break;
        }
    } 
    while (!value);   /* wait for ack signal! */
#endif

    /* ACK_OK ? */
    act_writel(0x80, I2CSTA);

    return;
}

/* Wait_stop */
static void i2c_wait_stop(sensor_i2cinfo_t * sensor_i2cinfo)
{
    unsigned int value;
    unsigned int cnt,0;

    do 
    {
        cnt++;
        value,act_readl(I2CSTA);
        value &= 0x40;
    } 
    while ((!value) && (cnt < 20000));   /* wait for stop condition. */

    if (cnt >= 20000)
    {
        ss_printf("i2c_wait_stop timeout!\n");
    }

    act_writel(0x40, I2CSTA);   //clear the the stop bit
}

/******************************************************************************/
/*!                    
* \par  Description:
*       read sensor's control register.
* \param[in]   reg_addr -- sensor module's register addr.
* \retval      0 sucess, else failed.
*******************************************************************************/
int sensor_i2c_read(unsigned int reg_addr)
{
    sensor_i2cinfo_t sensor_i2cinfo;

    sensor_i2cinfo.slave_addr,i2c_slave_addr;

    //DSC_I2C_RD:
    act_writel(0x00, I2CCTL);   //reset I2C 
    act_writel(0x80, I2CCTL);   //Master mode,disable IRQ

    /* 
     * here must set the PCLK to set the speed 100K or 400K 
     * I2CCLK*16=PCLK/I2CDIV 
     */
    act_writel(0x20, I2CDIV);   //set I2C 100K,here APH=12M
    act_writel(0x80, I2CCTL);   //Enable I2C 

    act_writel(sensor_i2cinfo.slave_addr, I2CADDR); //write device address of AT24C02
    act_writel(0x86, I2CCTL);   //generate start condition
    i2c_transmit_byte(&sensor_i2cinfo);

    act_writel(reg_addr, I2CDAT);
    act_writel(0x82, I2CCTL);   //release I2C; should it placed after i2c_transmit_byte??
    i2c_transmit_byte(&sensor_i2cinfo);

    act_writel(0x8b, I2CCTL);   //generate stop condition
    i2c_wait_stop(&sensor_i2cinfo);

    act_writel(sensor_i2cinfo.slave_addr + 1, I2CADDR); //write device address of AT24C02
    act_writel(0x86, I2CCTL);   //generate start condition
    i2c_transmit_byte(&sensor_i2cinfo);
    act_writel(0x83, I2CCTL);   //release I2C
    i2c_transmit_byte(&sensor_i2cinfo);

    sensor_i2cinfo.reg_data,act_readl(I2CDAT);
    act_writel(0x8b, I2CCTL);   //generate stop condition
    i2c_wait_stop(&sensor_i2cinfo);

    return (int)sensor_i2cinfo.reg_data;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       write sensor's control register.
* \param[in]   reg_addr -- sensor module's register addr.
* \param[in]   reg_data -- sensor module's register setting value.
* \retval      0 sucess, else failed.
*******************************************************************************/
int sensor_i2c_write(unsigned int reg_addr, unsigned int reg_data)
{
    sensor_i2cinfo_t sensor_i2cinfo;

    act_writel(0x0, I2CCTL);    //reset I2C

    act_writel(0x80, I2CCTL);   //Master mode,disable IRQ

    /* 
     * here must set the PCLK to set the speed 100K or 400K
     * I2CCLK*16=PCLK/I2CDIV
     */
    act_writel(0x20, I2CDIV);   //set I2C 100K,here APH=12M
    act_writel(0x80, I2CCTL);   //Enable I2C

    act_writel(i2c_slave_addr, I2CADDR);    //write device address of AT24C02
    act_writel(0x86, I2CCTL);   //generate start condition

    i2c_transmit_byte(&sensor_i2cinfo);

    act_writel(reg_addr, I2CDAT);
    act_writel(0x82, I2CCTL);   //release I2C
    i2c_transmit_byte(&sensor_i2cinfo);

    act_writel(reg_data, I2CDAT);
    act_writel(0x82, I2CCTL);   //release I2C
    i2c_transmit_byte(&sensor_i2cinfo);

    act_writel(0x8b, I2CCTL);   //generate stop condition

    i2c_wait_stop(&sensor_i2cinfo);

    return 0;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       set sensor image's white balance effect.
* \param[in]    white balance effect. refer to ss_white_balance_t.      \n
*       typedef enum {  \n
*           SS_AUTO_BALANCE,  \n
*           SS_INDOOR_BALANCE,  \n
*           SS_OUTDOOR_BALANCE,  \n
*           SS_FLUORE_BALANCE  \n
*       } white_balance_t;
*
* \retval       0 sucess, else failed.
*******************************************************************************/
int sensor_set_white_balance(unsigned int whitebalance)
{
    unsigned int is_start,0;
    OS_INT_SAVE_AREA                          /*fix the bug of broke down ,20081224. by cccw*/
    /* 
     * for most sensor module, it's not need to call ss_sensor_ioctl(SENSOR_IOSET_CYCLE_STOP, 0);
     * but there is something wrong with ov7680, if not restart dma. 
     */
    if (vi_dev.start_dma != 0)
    {
        AP_OS_ENTER_CRITICAL();              /*fix the bug of broke down ,20081224. by cccw*/
        is_start,1;
        ss_sensor_ioctl(SENSOR_IOSET_CYCLE_STOP, 0);
    }
#if 1
    //    unsigned int value;
	//temp_reg=sensor_i2c_read(0x22);


    switch (whitebalance)
	{
		case SS_AUTO_BALANCE: /* auto */
			//sp0838_reg_WB_auto         Զ
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0x32, 0x05);
			sensor_i2c_write(0xfd, 0x01);
			sensor_i2c_write(0x28, 0x75);  // AUTO 3000K~7000K
			sensor_i2c_write(0x29, 0x4e);
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0x32, 0x15);
			break;
		case SS_INDOOR_BALANCE: /* office */
			//sp0838_reg_WB_auto  ӫ
			sensor_i2c_write(0xfd,0x00);  //4200K~5000K
			sensor_i2c_write(0x32,0x05);
			sensor_i2c_write(0xfd,0x01);
			sensor_i2c_write(0x28,0x5a);
			sensor_i2c_write(0x29,0x62);
			sensor_i2c_write(0xfd,0x00);
			break;
		case SS_OUTDOOR_BALANCE: /* sunny */
			// sp0838_reg_WB_auto  
			sensor_i2c_write(0xfd,0x00);  //6500K
			sensor_i2c_write(0x32,0x05);
			sensor_i2c_write(0xfd,0x01);
			sensor_i2c_write(0x28,0x6b);
			sensor_i2c_write(0x29,0x48);
		    sensor_i2c_write(0xfd,0x00);
			break;
		case SS_FLUORE_BALANCE: /* home */
			// sp0838_reg_WB_auto ׳
			sensor_i2c_write(0xfd,0x00);  //2800K~3000K
			sensor_i2c_write(0x32,0x05);
			sensor_i2c_write(0xfd,0x01);
			sensor_i2c_write(0x28,0x41);
			sensor_i2c_write(0x29,0x71);
			sensor_i2c_write(0xfd,0x00);
		break;
		default:
		break;
	}

    vi_effect.ss_white_balance,whitebalance;
#endif
    if (is_start != 0)
    {
        ss_sensor_ioctl(SENSOR_IOSET_CYCLE_START, 0);
        	AP_OS_EXIT_CRITICAL();              /*fix the bug of broke down ,20081224. by cccw*/
}
        return 0;
    } 

/******************************************************************************/
/*!                    
* \par  Description:
*       get sensor module's current contrast value.
* \param[in]   null
* \return   value of current contrast level.
* \retval   range: 0 ~ 6.
*******************************************************************************/
unsigned int sensor_get_contrast(void)
{
    return vi_effect.ss_contrast;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       set sensor module's current contrast value.
* \param[in]   contrast -- range: 0 ~ 6.
* \retval      0 sucess, else failed.
*******************************************************************************/
int sensor_set_contrast(unsigned int contrast)
{
    vi_effect.ss_contrast,contrast;
    return 0;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       get sensor image's white balance effect.
* \param[in]   null
* \return       current white balance effect. refer to ss_white_balance_t.
*******************************************************************************/
unsigned int sensor_get_white_balance(void)
{
    return vi_effect.ss_white_balance;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       get sensor's focus level.
* \param[in]    null
* \return       focus level.
*******************************************************************************/
unsigned int sensor_get_focus(void)
{
    return 0;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       set sensor's focus level.
* \param[in]    sensor's focus level.
* \retval      0 sucess, else failed.
*******************************************************************************/
int sensor_set_focus(unsigned int focus)
{
    return 0;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       get sensor image's brightness level.
* \param[in]    null
* \retval       brightness level, 0 ~ 6.
*******************************************************************************/
unsigned int sensor_get_brightness(void)
{
    return vi_effect.ss_brightness;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       set sensor image's brightness level.
* \param[in]    brightness level, 0 ~ 6.
* \retval      0 sucess, else failed.
*******************************************************************************/
int sensor_set_brightness(unsigned int brightness)
{
    unsigned int is_start,0;
    OS_INT_SAVE_AREA                          /*fix the bug of broke down ,20081224. by cccw*/
    /* 
     * for most sensor module, it's not need to call ss_sensor_ioctl(SENSOR_IOSET_CYCLE_STOP, 0);
     * but there is something wrong with ov7680, if not restart dma. 
     */
    if (vi_dev.start_dma != 0)
    {
        AP_OS_ENTER_CRITICAL();              /*fix the bug of broke down ,20081224. by cccw*/
        is_start,1;
        ss_sensor_ioctl(SENSOR_IOSET_CYCLE_STOP, 0);
    }
    //printf("brightness type: %d \n", brightness);
    switch (brightness)
	{
		case 0:
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0xdc, 0xd0);
		break;
		case 1:
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0xdc, 0xe0);
		break;
		case 2:
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0xdc, 0xf0);
		break;
		case 3:
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0xdc, 0x00);
		break;
		case 4:
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0xdc, 0x10);
		break;
		case 5:
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0xdc, 0x20);
		break;
		case 6:
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0xdc, 0x30);
		break;
		default:
		return -1;
    }

    vi_effect.ss_brightness,brightness;
    if (is_start != 0)
    {
        mdelay(20);
        ss_sensor_ioctl(SENSOR_IOSET_CYCLE_START, 0);
        	AP_OS_EXIT_CRITICAL();              /*fix the bug of broke down ,20081224. by cccw*/
    }
    return 0;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       get sensor image's expose mode.
* \param[in]    null
* \retval       refer to ss_expose_mode_t.
*******************************************************************************/
unsigned int sensor_get_expose_mode(void)
{
    return vi_effect.ss_expose_mode;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       set sensor image's expose mode.
* \param[in]    refer to ss_expose_mode_t.
* \retval       0 sucess, else failed.
*******************************************************************************/
int sensor_set_expose_mode(unsigned int type)
{
    unsigned int is_start,0;
    OS_INT_SAVE_AREA                          /*fix the bug of broke down ,20081224. by cccw*/
    /* 
     * for most sensor module, it's not need to call ss_sensor_ioctl(SENSOR_IOSET_CYCLE_STOP, 0);
     * but there is something wrong with ov7680, if not restart dma. 
     */
    if (vi_dev.start_dma != 0)
    {
        AP_OS_ENTER_CRITICAL();              /*fix the bug of broke down ,20081224. by cccw*/
        is_start,1;
        ss_sensor_ioctl(SENSOR_IOSET_CYCLE_STOP, 0);
    }


    // temp_reg,sensor_i2c_read(0x20);
    switch (type)
	{
		case SS_AUTO_EXPOSURE:
			
			sensor_i2c_write(0xfd,0x00);
		break;

		case SS_INDOOR_EXPOSURE:/*  night mode */
			sensor_i2c_write(0xfd,0x00);
			
		break;

		case SS_OUTDOOR_EXPOSURE:
			sensor_i2c_write(0xfd,0x00);
			
		break;

		default:
		return -1;
	}

    if (is_start != 0)
    {
        mdelay(30);
        ss_sensor_ioctl(SENSOR_IOSET_CYCLE_START, 0);
        	AP_OS_EXIT_CRITICAL();              /*fix the bug of broke down ,20081224. by cccw*/
    }

    vi_effect.ss_expose_mode,type;
	return 0;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       get sensor image's special mode.
* \param[in]    null
* \retval       refer to ss_special_effect_t.
*******************************************************************************/
unsigned int sensor_get_special_mode(void)
{
    return vi_effect.ss_sp_effect;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       set sensor image's special mode.
* \param[in]    refer to ss_special_effect_t.
* \retval       0 sucess, else failed.
*******************************************************************************/
int sensor_set_special_mode(unsigned int type)
{
    unsigned int is_start,0;
    
    OS_INT_SAVE_AREA                          /*fix the bug of broke down ,20081224. by cccw*/

    /* 
     * for most sensor module, it's not need to call ss_sensor_ioctl(SENSOR_IOSET_CYCLE_STOP, 0);
     * but there is something wrong with ov7680, if not restart dma. 
     */
    if (vi_dev.start_dma != 0)
    {
        AP_OS_ENTER_CRITICAL();              /*fix the bug of broke down ,20081224. by cccw*/
        
        is_start,1;
        ss_sensor_ioctl(SENSOR_IOSET_CYCLE_STOP, 0);
    }

    switch (type)
	{
		case SS_NONE_SPECIAL: /* normal */
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0x62, 0x00);
			sensor_i2c_write(0x63, 0x80);
			sensor_i2c_write(0x64, 0x80);
		break;
		case SS_WHITE_AND_BLACK: /* black and white */
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0x62, 0x40);
			sensor_i2c_write(0x63, 0x80);
			sensor_i2c_write(0x64, 0x80);
		break;
		case SS_SEPIA: /* Antique */
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0x62, 0x20);
			sensor_i2c_write(0x63, 0xc0);
			sensor_i2c_write(0x64, 0x20);
		break;
		case SS_SOLARIZE: /* negative */
			sensor_i2c_write(0xfd, 0x00);
			sensor_i2c_write(0x62, 0x10);
			sensor_i2c_write(0x63, 0x80);
			sensor_i2c_write(0x64, 0x80);
		break;

    default:
        return -1;
    }
    if(SS_NONE_SPECIAL==type)
    {
        sensor_set_brightness(vi_effect.ss_brightness);
    }
    if (is_start != 0)
    {
        ss_sensor_ioctl(SENSOR_IOSET_CYCLE_START, 0);
        AP_OS_EXIT_CRITICAL();              /*fix the bug of broke down ,20081224. by cccw*/

    }

    vi_effect.ss_sp_effect,type;
    return 0;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       set sensor image's work mode.
* \param[in]    refer to ss_pix_fmt_t.
* \retval       0 sucess, else failed.
*******************************************************************************/
int sensor_set_pix_fmt(unsigned int type)
{
    unsigned int val;
	
    if (type == SS_YUV422) 
    {
		sensor_i2c_write(0xfd, 0x00);
		mdelay(1);
		sensor_i2c_write(0x35, 0x40);	//UYVY		//0x40
		mdelay(1);
		sensor_i2c_write(0x35, 0x40);	//UYVY		//0x40
		mdelay(1);
		sensor_i2c_write(0x35, 0x40);	//UYVY		//0x40
		mdelay(1);
		sensor_i2c_write(0x35, 0x40);	//UYVY		//0x40
		mdelay(1);
		sensor_i2c_write(0xe7, 0x03);
		sensor_i2c_write(0xe7, 0x00);
		mdelay(10);	
        if (ic_type == 0x63) 
        {
            act_writel(0x00000601, M_CSICTL);   //interface control (Cb0Y0Cr0Y1, Cb1Y2Cr1Y2, ...)
        } 
        else if (ic_type == 0x96) 
        {
        val,act_readl(M_CSICTL) | (0x1 << 7);
        act_writel(val, M_CSICTL);
        }
    else   /* SS_4Y4U4Y4V */
        {
            return -1;
        }
    } 
    else   /* SS_4Y4U4Y4V */
    {
        
        
        
		sensor_i2c_write(0xfd, 0x00);
		mdelay(1);
		sensor_i2c_write(0x35, 0x00);	//YUYV
		mdelay(1);
		sensor_i2c_write(0x35, 0x00);	//YUYV
		mdelay(1);
		sensor_i2c_write(0x35, 0x00);	//YUYV
		mdelay(1);
		sensor_i2c_write(0x35, 0x00);	//YUYV
		mdelay(1);
		sensor_i2c_write(0xe7, 0x03);
		sensor_i2c_write(0xe7, 0x00);
		mdelay(10);
        if (ic_type == 0x63) 
        {
            act_writel(0x00000201, M_CSICTL);       //interface control (Y0Y1Y2Y3, U0U1U2U3, Y4Y5Y6Y7, V0V1V2V3...)
        } 
        else if (ic_type == 0x96) 
        {

    	val,act_readl(M_CSICTL) & (~(0x1 << 7));
        act_writel(val, M_CSICTL);
        }
        else
        {
            return -1;
        }
    }
	mdelay(20);//50
    //sensor_change_resolution(vi_dev.resolution);	/* fix the bug that M_CSICTL is changed due to unkown question. */
    return 0;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       get sensor module currently supported resolution.
* \param[in]    null
* \retval       bitmap of currently supported resolution, refer to dc_resolution_t.
*******************************************************************************/
unsigned int sensor_get_pixel_bitmap(void)
{
    unsigned int iValue;

    iValue,0x0;
    //iValue |= 0x1 << DC224X176;
    iValue |= 0x1 << DC320X240;
    iValue |= 0x1 << DC640X480;
    //iValue |= 0x1 << DC1280X960;
    //iValue |= 0x1 << DC1280X1024;

    return iValue;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       get sensor module currently supported frame rate.
* \param[in]    null
* \retval       bitmap of currently supported frame rate.   \n
*               for example, if retval is 0x81, it means bit 7 and bit 0 is set, \n
*               so 8 frames/second and 1 frames/second is supported.
*******************************************************************************/
unsigned int sensor_get_f_rate_bitmap(void)
{
//    unsigned int iValue;
//
//    iValue,0x0;
//    iValue |= 0x1 << 24;    /* 24 frames per second is supported. */
//
//    return iValue;
    return 15;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       set sensor's frame rate in DV mode.
* \param[in]    f_rate -- set to f_rate frames/second.
* \retval       0 sucess, else failed.
* \par      exmaple code
* \code 
*           sensor_set_f_rate(24);  // set to 24 frames/second. 
* \endcode
*******************************************************************************/
int sensor_set_f_rate(int f_rate)
{
    return 0;
}

static int act_set_dc320x240_resolution(void)
{
    unsigned int val;

    /* 0x0000-Cmos sensor controllor. cmos sensor interface disable. */
    act_writel(0x00000012, M_CSIFIFOCTL);   //fifo control (sp bus)
    //act_writel(0x00000052, M_CSIFIFOCTL); //fifo control (ahb bus)
    if (ic_type == 0x63) 
    {

//sensor_i2c_write(0xfd,0x00);
//sensor_i2c_write(0x05,0x00);
//sensor_i2c_write(0x06,0x00);
//sensor_i2c_write(0x09,0x00);
//sensor_i2c_write(0x0a,0x81);
//sensor_i2c_write(0xf0,0x7a);
//sensor_i2c_write(0xf1,0x00);
//sensor_i2c_write(0xf2,0x65);
//sensor_i2c_write(0xf5,0x7e);
//sensor_i2c_write(0xfd,0x01);
//sensor_i2c_write(0x00,0xb8);
//sensor_i2c_write(0x0f,0x66);
//sensor_i2c_write(0x16,0x66);
//sensor_i2c_write(0x17,0xa8);
//sensor_i2c_write(0x18,0xb0);
//sensor_i2c_write(0x1b,0x66);
//sensor_i2c_write(0x1c,0xb0);
//sensor_i2c_write(0xb4,0x20);
//sensor_i2c_write(0xb5,0x3a);
//sensor_i2c_write(0xb6,0x76);
//sensor_i2c_write(0xb9,0x40);
//sensor_i2c_write(0xba,0x4f);
//sensor_i2c_write(0xbb,0x47);
//sensor_i2c_write(0xbc,0x45);
//sensor_i2c_write(0xbd,0x43);
//sensor_i2c_write(0xbe,0x42);
//sensor_i2c_write(0xbf,0x42);
//sensor_i2c_write(0xc0,0x42);
//sensor_i2c_write(0xc1,0x41);
//sensor_i2c_write(0xc2,0x41);
//sensor_i2c_write(0xc3,0x41);
//sensor_i2c_write(0xc4,0x41);
//sensor_i2c_write(0xc5,0x70);
//sensor_i2c_write(0xc6,0x41);
//sensor_i2c_write(0xca,0x70);
//sensor_i2c_write(0xcb,0x0c);
//sensor_i2c_write(0xfd,0x00);
//		
//		sensor_i2c_write(0xfd,0x00);
//		sensor_i2c_write(0x31,0x10);//0x10//70
//		sensor_i2c_write(0xe7,0x03);
//		sensor_i2c_write(0xe7,0x00);
    mdelay(20);

        act_writel(0x00000000, M_CSIHSPOS); //+0x000c-Hsync start position in pclk
        act_writel(0x0000027f, M_CSIHEPOS); //+0x0010-Hsync end position in pclk (QVGA 320)
        act_writel(0x00000000, M_CSIVSPOS); //+0x001c-Vsync start position in hsync
        act_writel(0x000000ef, M_CSIVEPOS); //+0x0020-Vsync end position in hsync (QVGA 240)
    } 
    else if (ic_type == 0x96) 
    {
    val,act_readl(M_CSICTL);
    /* vertical sub sample ratio: 1/2. */
    val &= ~(0x3 << 10);
    val |= 0x1 << 10;
    /* horizontal sub sample ratio: 1/2. */
    val &= ~(0x3 << 8);
    val |= 0x1 << 8;
    act_writel(val, M_CSICTL);
    }
    else
    {
        return -1;
    }

    act_writel(0x00000003, M_CSIRQSTA); //+0x0028-Cmos sensor interface IRQ Status
    return 0;
}


static int act_set_dc320x240_camera(void)
{
    unsigned int value;

    /* 0x0000-Cmos sensor controllor. cmos sensor interface disable. */
    act_writel(0x00000012, M_CSIFIFOCTL);   //fifo control (sp bus)
    //act_writel(0x00000052, M_CSIFIFOCTL); //fifo control (ahb bus)
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    //act_writel(0x80, M_CSICTL);
    value,act_readl(M_CSICTL);
    value &= ~(0xf << 8);
    value |= 0x1 << 7;
    act_writel(value, M_CSICTL);
    
    mdelay(20);
//sensor_i2c_write(0xfd,0x00);
//sensor_i2c_write(0x05,0x00);
//sensor_i2c_write(0x06,0x00);
//sensor_i2c_write(0x09,0x00);
//sensor_i2c_write(0x0a,0x81);
//sensor_i2c_write(0xf0,0x7a);
//sensor_i2c_write(0xf1,0x00);
//sensor_i2c_write(0xf2,0x65);
//sensor_i2c_write(0xf5,0x7e);
//sensor_i2c_write(0xfd,0x01);
//sensor_i2c_write(0x00,0xb8);
//sensor_i2c_write(0x0f,0x66);
//sensor_i2c_write(0x16,0x66);
//sensor_i2c_write(0x17,0xa8);
//sensor_i2c_write(0x18,0xb0);
//sensor_i2c_write(0x1b,0x66);
//sensor_i2c_write(0x1c,0xb0);
//sensor_i2c_write(0xb4,0x20);
//sensor_i2c_write(0xb5,0x3a);
//sensor_i2c_write(0xb6,0x76);
//sensor_i2c_write(0xb9,0x40);
//sensor_i2c_write(0xba,0x4f);
//sensor_i2c_write(0xbb,0x47);
//sensor_i2c_write(0xbc,0x45);
//sensor_i2c_write(0xbd,0x43);
//sensor_i2c_write(0xbe,0x42);
//sensor_i2c_write(0xbf,0x42);
//sensor_i2c_write(0xc0,0x42);
//sensor_i2c_write(0xc1,0x41);
//sensor_i2c_write(0xc2,0x41);
//sensor_i2c_write(0xc3,0x41);
//sensor_i2c_write(0xc4,0x41);
//sensor_i2c_write(0xc5,0x70);
//sensor_i2c_write(0xc6,0x41);
//sensor_i2c_write(0xca,0x70);
//sensor_i2c_write(0xcb,0x0c);
//sensor_i2c_write(0xfd,0x00);
//		
//		sensor_i2c_write(0xfd,0x00);
//		sensor_i2c_write(0x31,0x10);//0x10//70
//		sensor_i2c_write(0xe7,0x03);
//		sensor_i2c_write(0xe7,0x00);

    //mdelay(200);

    value,act_readl(M_CSIHSPOS);
    value &= ~(0x7ff);
    value |= 0x0;
    act_writel(value, M_CSIHSPOS);  //+0x000c-Hsync start position in pclk

    value,act_readl(M_CSIHEPOS);
    value &= ~(0x1fff << 12);
    value |= 0x280 << 12;
    act_writel(value, M_CSIHEPOS);  //+0x000c-Hsync start position in pclk

    value,act_readl(M_CSIVSPOS);
    value &= ~(0x7ff);
    value |= 0x0;
    act_writel(value, M_CSIVSPOS);  //+0x000c-Hsync start position in pclk

    value,act_readl(M_CSIVEPOS);
    value &= ~(0x1fff << 12);
    value |= 0xf0 << 12;
    act_writel(value, M_CSIVEPOS);  //+0x000c-Hsync start position in pclk
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    act_writel(0x00000003, M_CSIRQSTA); //+0x0028-Cmos sensor interface IRQ Status
    
    mdelay(20);
    return 0;
}


static int act_set_dc640x480_resolution(void)
{
    unsigned int val;

    /* 0x0000-Cmos sensor controllor. cmos sensor interface disable. */
    act_writel(0x00000012, M_CSIFIFOCTL);   //fifo control (sp bus)
    //act_writel(0x00000052, M_CSIFIFOCTL); //fifo control (ahb bus)
    
    if (ic_type == 0x63) 
    {


    act_writel(0x0000000c, M_BT_MODESEL);
    mdelay(2);
//sensor_i2c_write(0xfd,0x00);
//sensor_i2c_write(0x05,0x00);
//sensor_i2c_write(0x06,0x00);
//sensor_i2c_write(0x09,0x00);
//sensor_i2c_write(0x0a,0x81);
//sensor_i2c_write(0xf0,0x7a);
//sensor_i2c_write(0xf1,0x00);
//sensor_i2c_write(0xf2,0x65);
//sensor_i2c_write(0xf5,0x7e);
//sensor_i2c_write(0xfd,0x01);
//sensor_i2c_write(0x00,0xb8);
//sensor_i2c_write(0x0f,0x66);
//sensor_i2c_write(0x16,0x66);
//sensor_i2c_write(0x17,0xa8);
//sensor_i2c_write(0x18,0xb0);
//sensor_i2c_write(0x1b,0x66);
//sensor_i2c_write(0x1c,0xb0);
//sensor_i2c_write(0xb4,0x20);
//sensor_i2c_write(0xb5,0x3a);
//sensor_i2c_write(0xb6,0x76);
//sensor_i2c_write(0xb9,0x40);
//sensor_i2c_write(0xba,0x4f);
//sensor_i2c_write(0xbb,0x47);
//sensor_i2c_write(0xbc,0x45);
//sensor_i2c_write(0xbd,0x43);
//sensor_i2c_write(0xbe,0x42);
//sensor_i2c_write(0xbf,0x42);
//sensor_i2c_write(0xc0,0x42);
//sensor_i2c_write(0xc1,0x41);
//sensor_i2c_write(0xc2,0x41);
//sensor_i2c_write(0xc3,0x41);
//sensor_i2c_write(0xc4,0x41);
//sensor_i2c_write(0xc5,0x70);
//sensor_i2c_write(0xc6,0x41);
//sensor_i2c_write(0xca,0x70);
//sensor_i2c_write(0xcb,0x0c);
//sensor_i2c_write(0xfd,0x00);
//		
//		sensor_i2c_write(0xfd,0x00);
//		sensor_i2c_write(0x31,0x10);//0x10//70
//		sensor_i2c_write(0xe7,0x03);
//		sensor_i2c_write(0xe7,0x00);


        act_writel(0x00000000, M_CSIHSPOS); //+0x000c-Hsync start position in pclk
        act_writel(0x000004ff, M_CSIHEPOS); //+0x0010-Hsync end position in pclk (VGA 640)
        act_writel(0x00000000, M_CSIVSPOS); //+0x001c-Vsync start position in hsync
        act_writel(0x000001df, M_CSIVEPOS); //+0x0020-Vsync end position in hsync (VGA 480)
    } 
    else if (ic_type == 0x96) 
    {
    val,act_readl(M_CSICTL);
    /* vertical sub sample ratio: 1/2. */
    val &= ~(0x3 << 10);
    /* horizontal sub sample ratio: 1/2. */
    val &= ~(0x3 << 8);
    act_writel(val, M_CSICTL);

    }
    else 
    {
        return -1;
    }

    act_writel(0x00000003, M_CSIRQSTA); //+0x0028-Cmos sensor interface IRQ Status
    return 0;
}

static int act_set_dc640x480_camera(void)
{
    unsigned int value;

    /* 0x0000-Cmos sensor controllor. cmos sensor interface disable. */
    act_writel(0x00000012, M_CSIFIFOCTL);   //fifo control (sp bus)
    //act_writel(0x00000052, M_CSIFIFOCTL); //fifo control (ahb bus)
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //act_writel(0x80, M_CSICTL);
    value,act_readl(M_CSICTL);
    value &= ~(0xf << 8);
    value |= 0x1 << 7;
    act_writel(value, M_CSICTL);
    
    mdelay(20);
    
//sensor_i2c_write(0xfd,0x00);
//sensor_i2c_write(0x05,0x00);
//sensor_i2c_write(0x06,0x00);
//sensor_i2c_write(0x09,0x00);
//sensor_i2c_write(0x0a,0x81);
//sensor_i2c_write(0xf0,0x7a);
//sensor_i2c_write(0xf1,0x00);
//sensor_i2c_write(0xf2,0x65);
//sensor_i2c_write(0xf5,0x7e);
//sensor_i2c_write(0xfd,0x01);
//sensor_i2c_write(0x00,0xb8);
//sensor_i2c_write(0x0f,0x66);
//sensor_i2c_write(0x16,0x66);
//sensor_i2c_write(0x17,0xa8);
//sensor_i2c_write(0x18,0xb0);
//sensor_i2c_write(0x1b,0x66);
//sensor_i2c_write(0x1c,0xb0);
//sensor_i2c_write(0xb4,0x20);
//sensor_i2c_write(0xb5,0x3a);
//sensor_i2c_write(0xb6,0x76);
//sensor_i2c_write(0xb9,0x40);
//sensor_i2c_write(0xba,0x4f);
//sensor_i2c_write(0xbb,0x47);
//sensor_i2c_write(0xbc,0x45);
//sensor_i2c_write(0xbd,0x43);
//sensor_i2c_write(0xbe,0x42);
//sensor_i2c_write(0xbf,0x42);
//sensor_i2c_write(0xc0,0x42);
//sensor_i2c_write(0xc1,0x41);
//sensor_i2c_write(0xc2,0x41);
//sensor_i2c_write(0xc3,0x41);
//sensor_i2c_write(0xc4,0x41);
//sensor_i2c_write(0xc5,0x70);
//sensor_i2c_write(0xc6,0x41);
//sensor_i2c_write(0xca,0x70);
//sensor_i2c_write(0xcb,0x0c);
//sensor_i2c_write(0xfd,0x00);
		
//		sensor_i2c_write(0xfd,0x00);
//		sensor_i2c_write(0x31,0x10);//0x10//70
//		sensor_i2c_write(0xe7,0x03);
//		sensor_i2c_write(0xe7,0x00);

    //mdelay(200);    
	
    value,act_readl(M_CSIHSPOS);
    value &= ~(0x7ff);
    value |= 0x00000000;
    act_writel(value, M_CSIHSPOS);  //+0x000c-Hsync start position in pclk

    value,act_readl(M_CSIHEPOS);
    value &= ~(0x1fff << 12);
    value |= 0x500 << 12;
    act_writel(value, M_CSIHEPOS);  //+0x000c-Hsync start position in pclk

    value,act_readl(M_CSIVSPOS);
    value &= ~(0x7ff);
    value |= 0x0;
    act_writel(value, M_CSIVSPOS);  //+0x000c-Hsync start position in pclk

    value,act_readl(M_CSIVEPOS);
    value &= ~(0x1fff << 12);
    value |= 0x1e0 << 12;
    act_writel(value, M_CSIVEPOS);  //+0x000c-Hsync start position in pclk


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    act_writel(0x00000003, M_CSIRQSTA); //+0x0028-Cmos sensor interface IRQ Status
    
    mdelay(20);
    return 0;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       set sensor's resolution.
* \param[in]   cmd -- resolution enum, refer to dc_resolution_t.
* \retval      0 sucess, else failed.
*******************************************************************************/
int sensor_change_resolution(dc_resolution_t cmd)
{
    unsigned int len,0;
    int ret,0;

    vi_dev.phy_offset,0;

    switch (cmd) 
    {
    case DC320X240:
        len,320 * 240 * 2;
        if(vi_dev.pix_fmt == SS_YUV422)
        {    
            act_set_dc320x240_camera();
printf("act_set_dc320x240_camera\n");
        }
        else
        {
        act_set_dc320x240_resolution();
        }
        vi_dev.phy_offset,DC320X240_PHY_ADDR_OFFSET;
        vi_dev.width,320;
        vi_dev.height,240;
        break;
    case DC640X480:
        len,640 * 480 * 2;
        if(vi_dev.pix_fmt == SS_YUV422)
        {    
            act_set_dc640x480_camera();
printf("act_set_dc640x480_camera\n");
        }
        else
        {
        act_set_dc640x480_resolution();
        }        
//        act_set_dc640x480_resolution();
        vi_dev.phy_offset,DC640X480_PHY_ADDR_OFFSET;
        vi_dev.width,640;
        vi_dev.height,480;
        break;
    default:
        ss_printf("resolution type error!\n");
        ret,-1;
        return ret;
    }

    if (len > vi_dev.mem_len) 
    {
        ss_printf("This sensor doesn't support this resolution\n");
        return -1;
    }
    vi_dev.resolution,cmd;
    vi_dev.resolution_len,len;
    vi_dev.first_frame,1;

    return ret;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       initialize sensor module.
* \param[in]   null
* \retval      0 sucess, else failed.
*******************************************************************************/
int sensor_init_param_reg(void)
{
    unsigned int i;
    unsigned int val;
    sensor_i2cinfo_t sensor_i2cinfo;

    sensor_power_off();
    mdelay(20);
    sensor_power_on();
    mdelay(20);


//    sensor_i2cinfo.slave_addr,i2c_slave_addr;
//    sensor_i2cinfo.reg_addr,0x13;
//    sensor_i2cinfo.reg_data,0x1;
//    sensor_i2c_write(sensor_i2cinfo.reg_addr, sensor_i2cinfo.reg_data);
//    mdelay(3);
//    sensor_i2cinfo.reg_data,0x0;
//    sensor_i2c_write(sensor_i2cinfo.reg_addr, sensor_i2cinfo.reg_data);
//    mdelay(3);
//    sensor_i2cinfo.reg_addr,0xf1;
//    sensor_i2cinfo.reg_data,0x00;
//    sensor_i2c_write(sensor_i2cinfo.reg_addr, sensor_i2cinfo.reg_data);
//    mdelay(3);


#if 1
    for (i=0; i < sizeof(sensor_i2cdata); i++) 
    {
        sensor_i2cinfo.reg_addr,(unsigned int)sensor_i2cdata[i];
        i++;
        sensor_i2cinfo.reg_data,(unsigned int)sensor_i2cdata[i];
        sensor_i2c_write(sensor_i2cinfo.reg_addr, sensor_i2cinfo.reg_data);
        //mdelay(4);
    }
#endif


//    if (ic_type == 0x63) 
//    {
//////        mdelay(4);
////        sensor_i2c_write(0X45, 0X48);
////        sensor_i2c_write(0x4e, 0x33);
//        
//        
//	sensor_i2c_write(	0xf0,  0x00);
//	sensor_i2c_write(	0x0e , 0x0a); //row even skip
//	sensor_i2c_write(	0x43 , 0xc0); //more boundary mode opclk output enable
//	//sensor_i2c_write(	0x44 , 0xe2); // mtk is e0
//	//sensor_i2c_write(	0x45 , 0x2b); //col subsample  
//sensor_i2c_write(	0x45,  0x28); //col subsample  
//	sensor_i2c_write(	0x4e,  0x33); //  32 opclk gate in subsample  // mtk is 33
//
//	sensor_i2c_write(	0x01,  0xd1);
//	sensor_i2c_write(	0x02,  0x82);
//	sensor_i2c_write(	0x10,  0x00);
//	sensor_i2c_write(	0xd6,  0xce);
//                            
//	sensor_i2c_write(	0x28,  0x02); //AEC_exp_level_1bit11to8   // 33.3fps
//	sensor_i2c_write(	0x29,  0x6a); //AEC_exp_level_1bit7to0 
//	sensor_i2c_write(	0x2a,  0x04); //AEC_exp_level_2bit11to8   // 20fps
//	sensor_i2c_write(	0x2b,  0x06); //AEC_exp_level_2bit7to0  
//	sensor_i2c_write(	0x2c,  0x06); //AEC_exp_level_3bit11to8    // 12.5fps
//	sensor_i2c_write(	0x2d,  0x70); //AEC_exp_level_3bit7to0           
//	sensor_i2c_write(	0x2e,  0x0c); //AEC_exp_level_4bit11to8   // 6.25fps
//	sensor_i2c_write(	0x2f,  0xe0); //AEC_exp_level_4bit7to0   
//                            
//	sensor_i2c_write(	0xe1,  0x01); //big_win_y0                                                     
//	sensor_i2c_write(	0xe3,  0x0f); //432, big_win_y1    , height                                 
//	sensor_i2c_write(	0xea,  0x16); //small_win_height1                        
//	sensor_i2c_write(	0xeb,  0x28); //small_win_height2                        
//	sensor_i2c_write(	0xec,  0x39); //small_win_heigh3 //only for AWB 
//	sensor_i2c_write(	0xae,  0x0c); //black pixel target number
//	sensor_i2c_write(	0xc3,  0x20); //number limit
//	sensor_i2c_write(	0x74,  0x1e); //lsc_row_center , 0x3c
//	sensor_i2c_write(	0x75,  0x52); //lsc_col_center , 0x52
//	
//        act_writel(0x00000000, M_CSIHSPOS); //+0x000c-Hsync start position in pclk
//        act_writel(0x0000027f, M_CSIHEPOS); //+0x0010-Hsync end position in pclk (QVGA 320)
//        act_writel(0x00000000, M_CSIVSPOS); //+0x001c-Vsync start position in hsync
//        act_writel(0x000000ef, M_CSIVEPOS); //+0x0020-Vsync end position in hsync (QVGA 240)
//    } 
//    else if (ic_type == 0x96) 
    {
        act_writel(0x508, M_CSICTL);
        
        val,act_readl(M_CSIHSPOS);
        val &= ~(0x7ff);
        val |= 0x0;
        act_writel(val, M_CSIHSPOS);  //+0x000c-Hsync start position in pclk

        val,act_readl(M_CSIHEPOS);
        val &= ~(0x1fff << 12);
        val |= 0x500 << 12;
        act_writel(val, M_CSIHEPOS);  //+0x000c-Hsync start position in pclk

        val,act_readl(M_CSIVSPOS);
        val &= ~(0x7ff);
        val |= 0x0;
        act_writel(val, M_CSIVSPOS);  //+0x000c-Hsync start position in pclk

        val,act_readl(M_CSIVEPOS);
        val &= ~(0x1fff << 12);
        val |= 0x1e0 << 12;
        act_writel(val, M_CSIVEPOS);  //+0x000c-Hsync start position in pclk
    }

    return 0;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       set sensor module to standby status.
* \param[in]   null
* \retval      0 sucess, else failed.
*******************************************************************************/
int sensor_exit_param_reg(void) 
{
    
    return 0;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       sensor module power on.
* \param[in]   null
* \retval      null.
*******************************************************************************/
void sensor_power_on(void)
{
    unsigned int value;
#if 0
    /* power on. */
#ifdef PWDN_ACTIVE_HIGH    
    value,act_readl(PMU_CTL);
    value &= ~(0x1 << 14);   
    act_writel(value, PMU_CTL);
#elif defined(PWDN_ACTIVE_LOW)
    value,act_readl(PMU_CTL);
    value |= (0x1 << 14); 
    act_writel(value, PMU_CTL);
#endif
#else

#ifdef   GPIO_A_RESET
    value,act_readl(GPIO_AOUTEN);
    value |= 0x1 << RESET_PIN_NUM;
    act_writel(value, GPIO_AOUTEN);	
    value,act_readl(GPIO_ADAT);	
    value |= 0x1 << RESET_PIN_NUM;
    act_writel(value, GPIO_ADAT);
    mdelay(40);
    value &=(~(0x1 << RESET_PIN_NUM));
    act_writel(value, GPIO_ADAT);
    mdelay(40);
#endif

#ifdef   GPIO_B_RESET
    value,act_readl(GPIO_BOUTEN);
    value |= 0x1 << RESET_PIN_NUM;
    act_writel(value, GPIO_BOUTEN);
    value,act_readl(GPIO_BDAT);
    value |= 0x1 << RESET_PIN_NUM;
    act_writel(value, GPIO_BDAT);
    lcd_mdelay(100);
    value &=(~(0x1 << RESET_PIN_NUM));
    act_writel(value, GPIO_BDAT);
    lcd_mdelay(100);
    value |= 0x1 << RESET_PIN_NUM;
    act_writel(value, GPIO_BDAT);
    lcd_mdelay(100);
#endif   

#ifdef GPIO_PWND_B
    value,act_readl(GPIO_BINEN);
    value &= ~(0x1 << PWND_PIN_NUM);	
    value,act_readl(GPIO_BOUTEN);
    value |= 0x1 << PWND_PIN_NUM;
    act_writel(value, GPIO_BOUTEN);
    value,act_readl(GPIO_BDAT);
#ifdef PWDN_ACTIVE_HIGH  	
    value &= ~(0x1 << PWND_PIN_NUM);
#elif defined(PWDN_ACTIVE_LOW)
    value |= (0x1 << PWND_PIN_NUM);
#endif
    act_writel(value, GPIO_BDAT);
#endif

#ifdef GPIO_PWND_A
    value,act_readl(GPIO_AINEN);
    value &= ~(0x1 << PWND_PIN_NUM);	
    value,act_readl(GPIO_AOUTEN);
    value |= 0x1 << PWND_PIN_NUM;
    act_writel(value, GPIO_AOUTEN);
    value,act_readl(GPIO_ADAT);
#ifdef PWDN_ACTIVE_HIGH  	
    value &= ~(0x1 << PWND_PIN_NUM);
#elif defined(PWDN_ACTIVE_LOW)
    value |= (0x1 << PWND_PIN_NUM);
#endif
    act_writel(value, GPIO_ADAT);
#endif

#endif
    return;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       sensor module power off.
* \param[in]   null
* \retval      null.
*******************************************************************************/
void sensor_power_off(void)
{
    unsigned int value;
#if 0
    /* power off. */
#ifdef PWDN_ACTIVE_HIGH    
    value,act_readl(PMU_CTL);
    value |= (0x1 << 14); 
    act_writel(value, PMU_CTL);
#elif defined(PWDN_ACTIVE_LOW)
    value,act_readl(PMU_CTL);
    value &= ~(0x1 << 14);      
    act_writel(value, PMU_CTL);
#endif
#else

#ifdef GPIO_PWND_B
    value,act_readl(GPIO_BINEN);
    value &= ~(0x1 << PWND_PIN_NUM);	
    value,act_readl(GPIO_BOUTEN);
    value |= 0x1 << PWND_PIN_NUM;
    act_writel(value, GPIO_BOUTEN);
    value,act_readl(GPIO_BDAT);
#ifdef PWDN_ACTIVE_HIGH  	
    value |= (0x1 << PWND_PIN_NUM);
#elif defined(PWDN_ACTIVE_LOW)
    value &= ~(0x1 << PWND_PIN_NUM);
#endif
    act_writel(value, GPIO_BDAT);
#endif

#ifdef GPIO_PWND_A
    value,act_readl(GPIO_AINEN);
    value &= ~(0x1 << PWND_PIN_NUM);	
    value,act_readl(GPIO_AOUTEN);
    value |= 0x1 << PWND_PIN_NUM;
    act_writel(value, GPIO_AOUTEN);
    value,act_readl(GPIO_ADAT);
#ifdef PWDN_ACTIVE_HIGH  	
    value |= (0x1 << PWND_PIN_NUM);
#elif defined(PWDN_ACTIVE_LOW)
    value &= ~(0x1 << PWND_PIN_NUM);
#endif
    act_writel(value, GPIO_ADAT);
#endif

#endif
    return;
}

/******************************************************************************/
/*!                    
* \par  Description:
*       sensor module clk in setting, it will affect frame rate.
* \param[in]   null
* \retval      null.
*******************************************************************************/
void sensor_init_m_clk(void)
{
    act_writel(M_CLK_DIVIDE, CMU_BTCLK);  /* 1/10 bus clock */    
    return;
}

